#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_load.h>
#include <vil1/vil1_save.h>
#include <misc_utility/defines.h>                // def of color_buffer_t
#include <misc_utility/loaders.h>
#include <mvl/HMatrix2DComputeLinear.h>          // Robust version not avail yet ?  it just segfaults !
//#include <mvl/HMatrix2DComputeMLESAC.h>
#include <mvl/HMatrix2D.h>
#include <misc_utility/morphing.h>

// Morphs image1 (non-black parts in mask_image) over image2
// Homography morphing fct robustly computed over
// input set of pts correspondences

int main(int argc, char** argv)
{
  // Check syntax
  if (argc != 7)
  {
    vcl_cout << "Usage: " << argv[0] 
	     << " image1.tif pts1.dat mask_image.tif image2.tif pts2.dat output_image.tif" << vcl_endl;
    return 1;
  }

  // Load points
  vcl_cout << "Loading points..." << vcl_endl;
  vcl_vector<HomgPoint2D> points1 = load_points(argv[2]);
  vcl_vector<HomgPoint2D> points2 = load_points(argv[5]);

  // Load images
  vcl_cout << "Loading images..." << vcl_endl;
  vil1_image image1 = vil1_load(argv[1]);
  vil1_image image2 = vil1_load(argv[4]);
  color_buffer_t* buf1 = new color_buffer_t(image1);
  color_buffer_t* buf2 = new color_buffer_t(image2);

  // Load mask
  vil1_image mask = vil1_load(argv[3]);
  color_buffer_t* maskbuf = new color_buffer_t(mask);

  // Fit homography to point-matches sets
  vcl_cout << "Fitting homography..." << vcl_endl;
  HMatrix2DComputeLinear computor;
  HMatrix2D H(computor.compute(points1, points2));

  // Apply morphing
  vcl_cout << "Appliying morphing..." << vcl_endl;
  homography_morph(buf1, buf2, maskbuf, H);

  // Save morphed output image
  if(vil1_save(*buf2, argv[6])) {
    vcl_cout << "Morphed image saved as " << argv[6] << vcl_endl;
  }
  else {
    vcl_cout << "Warning: cannot save " << argv[6] << vcl_endl;
  }
} // main


