// Module: FMatrixCompute example
// Author: Andrew W. Fitzgibbon, Oxford RRG, July 96
// Adapted to mvl by Peter Vanroose, January 2002.
// Bent to his needs by V. Ferrari, Nov 2002
// Updated to output fmatrix, Nov 2003
// Fixed to wotk with badly spread points as input Aug 2004

#include <cstdlib>

#include <vcl_iostream.h>
#include <vcl_fstream.h>    // files I/O
#include <vcl_vector.h>
#include <mvl/FMatrix.h>
#include <mvl/HomgPoint2D.h>
#include <vxl_mvl_fix/FMatrixComputeRANSACFix.h>  // fix by V Ferrari Aug 2004
#include <misc_utility/loaders.h>      // load_points, save_bools


void process_arguements(int argc, char** argv, float& sigma, char*& FMatrixFile){
  for(int a = 4; a < argc; a++) {
    if(strcmp(argv[a],"-sigma") == 0) {
      sigma = atof(argv[a+1]);
    }
    if(strcmp(argv[a],"-Fmatrix") == 0) {
      FMatrixFile = argv[a+1];
    }
  }
}


int main(int argc, char**argv) {
  // Check syntax
  if(argc < 4) {
    vcl_cout << "Usage: " << argv[0] << " points1.dat points2.dat inliers.dat [-sigma s] [-Fmatrix F.dat]" << vcl_endl;
    return -1;
  }

  // Load points
  vcl_vector<HomgPoint2D> points1 = load_points(argv[1]);
  vcl_vector<HomgPoint2D> points2 = load_points(argv[2]);

  // Processing arguements
  float sigma = 6.0;
  char* FMatrixFile = NULL;
  process_arguements(argc, argv, sigma, FMatrixFile);
 
  // Perform the fit using Phil Torr's RANSAC
  vcl_cout << "Sigma = " << sigma << vcl_endl;
  FMatrixComputeRANSACFix computor(true,sigma); // sigma -> error tolerance (in stdr deviation or something)
  FMatrix F = computor.compute(points1, points2);

  //vcl_cout << "FMatrixComputeRANSAC with rank truncation:\nF = " << F << vcl_endl;

  double d = 0;
  for(unsigned int i = 0; i < points1.size(); ++i) {
    double dl = F.image1_epipolar_distance_squared(&points1[i], &points2[i]);
    //vcl_cout << "dist = " << sqrt(dl) << vcl_endl;
    d += dl; 
  }
  vcl_cout << "Error = " << d/points1.size() << vcl_endl;

  // Get inliers to the epipolar geometry
  vcl_vector<bool> inliers(computor.get_inliers());

  // Output F to file
  if(FMatrixFile) {
    vcl_cout << "Saving F-matrix to file " << FMatrixFile << vcl_endl;
    vcl_ofstream outfile(FMatrixFile);
    outfile << F;
    outfile.close();
  }

  // Save inliers to output file
  save_bools(inliers, argv[3]);
} // main


