#include <vcl_sstream.h>
#include <vcl_iostream.h>
#include <vcl_fstream.h>

#include <cstdlib>
#include <cstring>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_save.h>
#include <vil1/vil1_load.h>
#include <misc_utility/defines.h>                // def of color_buffer_t
#include <region_utility/Vitto_region_file.h>
#include <region_utility/region_draw.h>          // draw_regions routine
#include <misc_utility/draw.h>                   // draw_contours routines
#include <misc_utility/count_pixels.h>


void process_arguements(int argc, char** argv, char*& overlay_filename, vil1_rgb<unsigned char>& color, int& w, bool& outline){
  for(int a = 4; a < argc; a++) {
    if(strcmp(argv[a],"-overlay") == 0) {
      overlay_filename = argv[a+1];
    }
    if(strcmp(argv[a],"-color") == 0) {
      int r = atoi(argv[a+1]);
      int g = atoi(argv[a+2]);
      int b = atoi(argv[a+3]);
      color = vil1_rgb<unsigned char>(r,g,b);
      vcl_cout << "Color set to " << r << " " << g << " " << b << vcl_endl;
    }
    if(strcmp(argv[a],"-width") == 0) {
      w = atoi(argv[a+1]);
    }
    if(strcmp(argv[a],"-outline") == 0) {
      outline = true;
    }
  }
}


// Total area of regions

int main(int argc, char** argv)
{
  if (argc < 4) {
    vcl_cout << "Usage: " << argv[0] 
	     << " regs.regmV image.tif area.dat [-overlay out.tif] [-color r g b] [-width w] [-outline]" << vcl_endl;
    return 1;
  }

  // Processing arguements
  char* overlay_filename = NULL;
  vil1_rgb<unsigned char> color(0, 255, 0);                     // default color is green
  int w = 2;                                                   // default width
  bool outline = false;
  process_arguements(argc, argv, overlay_filename, color, w, outline);

  // Load regions
  vcl_cout << "Loading regions in Vitto format..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t regions;
  vrf.load_regions(regions, argv[1], (vil1_image*)NULL);         // last param just needed by signature
  vcl_cout << regions.size() << " regions loaded" << vcl_endl;

  // Load image
  vil1_image image = vil1_load(argv[2]);
  color_buffer_t* buf_i = new color_buffer_t(image);

  // Create empty buffer the same size as the load image
  color_buffer_t* buf = new color_buffer_t(buf_i->width(), buf_i->height());  

  // Draw solid regions on empty image
  vcl_cout << "Drawing regions..." << vcl_endl;
  draw_regions(regions, color, buf);                            // draw regions on buf (ie: buf is an I/O param)

  // Count pixels
  int area = count_pixels(color, buf);                          // count pixels of color 'color' in 'buf'
  vcl_cout << "The area is " << area << " pixels" << vcl_endl;

  // Overlay area's contour on load image
  if(outline) {
    vcl_cout << "Drawing with outline" << vcl_endl;
    draw_outlined_contours(buf, color, buf_i, color, w);                  
  }
  else {
    vcl_cout << "Drawing without outline" << vcl_endl;
    draw_contours(buf, color, buf_i, color, w);                
  }

  // Save area output
  vcl_ofstream outfile(argv[3]);
  outfile << area;
  outfile.close();
  vcl_cout << "Area value saved to " << argv[3] << vcl_endl;
  
  // Save overlay image
  if(vil1_save(*buf_i, overlay_filename)) {
    vcl_cout << "Overlay image successfully saved as " << overlay_filename << vcl_endl;
  }
  else {
    vcl_cout << "Warning: cannot save " << overlay_filename << vcl_endl;
  }


  // Clean-up
  delete buf_i;
  delete buf;

  vcl_cout << "Done" << vcl_endl;
} // main


