#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_load.h>
#include <vil1/vil1_save.h>
#include <misc_utility/defines.h>                // def of color_buffer_t
#include <region_utility/Vitto_region_file.h>

#include <misc_utility/loaders.h>                // save_floats

#include <main_processing/regionsrefiner.h>


void process_arguements(int argc, char** argv, bool& small, bool& gd6d) {
  for(int a = 8; a < argc; a++) {
    if(strcmp(argv[a],"-small") == 0) {
      vcl_cout << "Small search space." << vcl_endl;
      small = true;
    }
    if(strcmp(argv[a],"-6dgd") == 0) {
      vcl_cout << "Simple 6D Gradient Descent mode." << vcl_endl;
      gd6d = true;
    }
  }
}


// Refine regions
int main(int argc, char** argv)
{
  if (argc < 8) {
    vcl_cout << "Usage: " << argv[0] 
	     << " regs1.regmV image1.tif regs2.regmV image2.tif reg2_ref.regmV pre_sims.dat post_sims.dat  [-small] [-6dgd]" << vcl_endl;
    vcl_cout << "Default: raywalk refiner, normal search space bounds." << vcl_endl;
    return 1;
  }

  // Processing arguements
  bool small = false;
  bool gd6d = false;
  process_arguements(argc, argv, small, gd6d);

  // Load regions
  vcl_cout << "Loading regions in Vitto format..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t regions1, regions2;
  vrf.load_regions(regions1, argv[1], (vil1_image*)NULL);  // last param just needed by signature
  vrf.load_regions(regions2, argv[3], (vil1_image*)NULL);
  if(regions1.size() != regions2.size()) {
    vcl_cout << "Not the same amount of regions in the two lists. Cannot refine." << vcl_endl;
    return -1;
  }
  vcl_cout << regions1.size() << " regions loaded" << vcl_endl;

  // Load images
  vcl_cout << "Loading images..." << vcl_endl;
  vil1_image image1 = vil1_load(argv[2]);
  vil1_image image2 = vil1_load(argv[4]);
  color_buffer_t* buf1 = new color_buffer_t(image1);
  color_buffer_t* buf2 = new color_buffer_t(image2);
  
  RegionsRefiner rRefiner( buf1, buf2, true, small, gd6d );
  regions_list_t regions2_ref;                             // container for refined regions
  vcl_vector<float> pre_sims;                              // initial similarities
	vcl_vector<float> ref_sims;                              // post-refinement similarities

	rRefiner.refine( regions1, regions2, regions2_ref, &pre_sims, &ref_sims );

  // Save regions output
  vrf.save_regions(regions2_ref, argv[5]);

  // Save pre- and post-ref sims output
  save_floats(pre_sims, argv[6]);
  save_floats(ref_sims, argv[7]);

  vcl_cout << "Done" << vcl_endl;
  
} // main


