#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_load.h>
#include <vil1/vil1_save.h>
#include <misc_utility/defines.h>                          // def of color_buffer_t
#include <region_utility/Vitto_region_file.h>

#include <region_utility/hue_entropy_computor.h>           // saliency measure

#include <misc_utility/loaders.h>                          // save_floats


int main(int argc, char** argv)
{
  if (argc != 4) {
    vcl_cout << "Usage: " << argv[0] 
	     << " regs.regmV image.tif saliency.dat" << vcl_endl;
    return 1;
  }

  // Load regions
  vcl_cout << "Loading regions in Vitto format..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t rs;
  vrf.load_regions(rs, argv[1], (vil1_image*)NULL);         // last param just needed by signature
  vcl_cout << rs.size() << " regions loaded" << vcl_endl;

  // Load image
  vcl_cout << "Loading image..." << vcl_endl;
  vil1_image image = vil1_load(argv[2]);
  color_buffer_t* buf = new color_buffer_t(image);
  
  // Preparing saliency computor
  hue_entropy_computor sc(buf);
  sc.set_resolution(30);
  sc.set_adaptive_resolution(true);

  // Computing regions saliencies
  vcl_cout << "Computing saliencies..." << vcl_endl;
  vcl_vector<float> sals;
  for (int rx = 0; rx < rs.size(); rx++) {
    float s =  sc.get_saliency(rs[rx]);
    sals.push_back(s);
  }

  // Save out results
  save_floats(sals, argv[3]);

  vcl_cout << "Done" << vcl_endl;

} // main

