#include <vcl/vcl_vector.h>
#include <vcl/vcl_iostream.h>
#include <vcl/vcl_fstream.h>

#include <main_processing/rscfilter.h>


void printUsage( char *pName )
{
	vcl_cout << "Usage: " << pName
	         << " coords1.dat coords2.dat output.dat [removal_history.dat]" << vcl_endl
	         << " Coordinates = 1 homogenous 2D point per line" << vcl_endl
	         << " Output file = indices of surviving/removed points (starting from 1)." << vcl_endl;
}

void process_arguments(int argc, char** argv, char*& inFile1,
                       char*& inFile2, char*& outFile, char*& histFile)
{
  inFile1 = argv[1];
  inFile2 = argv[2];
  outFile = argv[3];
  if( argc > 4 )
    histFile = argv[4];
}

int loadPoints( const char *inFile, vcl_vector<pt> &points )
{
  double x, y, w;
  int c = 0;
  vcl_ifstream input(inFile);
  while( input.good() ) {
    input >> x >> y >> w;
    if( input.bad() || input.eof() )
      break;
    if( w != 1.0 && w != 0.0 ) {
      x /= w;
      y /= w;
    }
    pt newPt( x, y );
    points.push_back(newPt);
    c++;
  }
  return c;
}


int main( int argc, char** argv ) {
  if( argc < 4 ) {
    printUsage(argv[0]);
    return 1;
  }

  char *inFile1 = NULL, *inFile2 = NULL, *outFile = NULL, *histFile = NULL;
  process_arguments(argc, argv, inFile1, inFile2, outFile, histFile );

  vcl_vector<pt> model, img;

  // Load regions
  if( loadPoints( inFile1, model ) != loadPoints( inFile2, img ) ) {
    vcl_cerr << "Not the same number of regions in the two views. Improper data format." << vcl_endl;
    return -1;
  }
  
	// perform filtering
  vcl_vector<int> indices, rmHist;
	RSCFilter rsc(true);
	rsc.filter( indices, model, img, NULL, &rmHist );

  // save indices in file
  vcl_ofstream output( outFile );
  for( int i = 0; i < indices.size(); i++ )
    output << indices.at(i) << " ";

  // save removal history in file if desired
  if( histFile != NULL ) {
    vcl_ofstream hist( histFile );
    for( int i = 0; i < rmHist.size(); i++ )
      hist << rmHist.at(i) << " ";
  }
  
  return 0;
}
