#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_load.h>
#include <vil1/vil1_save.h>
#include <misc_utility/defines.h>                // def of color_buffer_t
#include <region_utility/Vitto_region_file.h>

#include <region_matching/ncc_drgbsnorm_computor.h>   // similarity measure to be tested

// Tests output of similarity calls
int main(int argc, char** argv)
{
  if (argc != 5) {
    vcl_cout << "Usage: " << argv[0] 
	     << " regs1.regmV image1.tif regs2.regmV image2.tif" << vcl_endl;
    return 1;
  }

  // Load regions
  vcl_cout << "Loading regions in Vitto format..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t rs1, rs2;
  vrf.load_regions(rs1, argv[1], (vil1_image*)NULL);         // last param just needed by signature
  vrf.load_regions(rs2, argv[3], (vil1_image*)NULL);
  if(rs1.size() != rs2.size()) {
    vcl_cout << "Not the same amount of regions in the two images. Improper data format." << vcl_endl;
    return -1;
  }
  vcl_cout << rs1.size() << " regions loaded" << vcl_endl;

  // Load images
  vcl_cout << "Loading images..." << vcl_endl;
  vil1_image image1 = vil1_load(argv[2]);
  vil1_image image2 = vil1_load(argv[4]);
  color_buffer_t* buf1 = new color_buffer_t(image1);
  color_buffer_t* buf2 = new color_buffer_t(image2);
  
  // Preparing similarity computor
  ncc_drgbsnorm_computor sc(buf1, buf2);
  sc.set_resolution(30); 
  sc.set_adaptive_resolution(true);

  vcl_cout << "Computing similarities..." << vcl_endl;
  for (int rx = 0; rx < rs1.size(); rx++) {
    region* reg1 = rs1[rx];
    region* reg2 = rs2[rx];                                       // same index in list
    if(reg1->get_sid() != reg2->get_sid()) {
      vcl_cout << vcl_endl << vcl_endl << "WARNING: not corresponding regions ids: " << reg1->get_sid() << " != " << reg2->get_sid() << vcl_endl;
    }
      
    vcl_cout << "Region " << reg1->get_sid() << ", type = ";
    if(reg1->get_type() == region::ELL_REGIONTYPE)
      vcl_cout << "21";
    else
      vcl_cout << reg1->get_specific_type();

    vcl_cout << ", sim = " << sc.get_similarity(reg1, reg2) << vcl_endl;
  } // loop on rs1


  vcl_cout << "Done" << vcl_endl;

} // main

