#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_load.h>
#include <vil1/vil1_save.h>
#include <misc_utility/defines.h>                          // def of color_buffer_t
#include <misc_utility/gaussian_smoothing.h>               // vic_gaussian_smoothing
#include <misc_utility/loaders.h>                          // load_uniform_region
#include <regions/uniform_region.h>                         
#include <region_utility/Vitto_region_file.h>
#include <region_propagation/uniform_propagator.h>
#include <region_matching/ncc_drgb_computor.h>             // used to compute color trafo of supports
#include <region_utility/region_draw.h>                    // draw_uniform_region              
#include <misc_utility/draw.h>                             // HSVtoRGB


// !!!
// FIX REGION LOADER: IF ONLY ONE REGION -> IT CRASHES !!!!
int main(int argc, char** argv)
{
  if (argc != 8) {
    vcl_cout << "Usage: " << argv[0] 
	     << " image1.tif sup_regs1.regmV image2.tif sup_regs2.regmV cand_regs1.unif cand_regs2.unif sup_regs.dat" << vcl_endl;
    return 1;
  }

  // Load support regions
  vcl_cout << "Loading support regions in Vitto format..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t sup_rs1, sup_rs2;
  vrf.load_regions(sup_rs1, argv[2], (vil1_image*)NULL);                  // last param just needed by signature
  vrf.load_regions(sup_rs2, argv[4], (vil1_image*)NULL);
  if(sup_rs1.size() != sup_rs2.size()) {
    vcl_cout << "ERROR: Not the same amount of support regions in the two images. Improper data format." << vcl_endl;
    return -1;
  }

  // Load uniform region
  vcl_cout << "Loading uniform regions " << argv[5] << vcl_endl;
  uniform_regions_list_t* urs1 = load_uniform_regions(argv[5]);

  // Load images
  vcl_cout << "Loading images..." << vcl_endl;
  vil1_image image1 = vil1_load(argv[1]);
  vil1_image image2 = vil1_load(argv[3]);
  color_buffer_t* buf1 = new color_buffer_t(image1);
  color_buffer_t* buf2 = new color_buffer_t(image2);

  // Smooth image2 in the 3 color bands
  vcl_cout << "Smoothing image..." << vcl_endl;
  color_buffer_t* buf_smooth2 = new color_buffer_t(image2);                 
  vic_gaussian_smoothing(*buf2, *buf_smooth2);
  color_buffer_t* buf_out2 = new color_buffer_t(image2);                     // copy

  // Propagate uniform region
  vcl_cout << "Propagating uniform region..." << vcl_endl;
  ncc_drgb_computor* sc = new ncc_drgb_computor(buf1, buf2);                 // used to compute intensity trafo of supports
  uniform_propagator uprop(buf1, buf_smooth2, sc, &sup_rs1, &sup_rs2, true); // true -> verbose
  uniform_regions_list_t* urs2 = uprop.propagate(urs1);

  // Save output uniform region
  save_uniform_regions(urs2, argv[6]);

  // Save supports
  vcl_vector<int> sup_rs = uprop.get_supports();                             // get list of successful reference regions
  save_ints(sup_rs, argv[7]);

  // Save drawn region (testing)
  for(int u=0; u<urs2->size(); u++) {
    float r,g,b;
    float hue = ((*urs2)[u]->get_sid())*(360./16.);                      // 16 different colors
    hue = hue - 360.0*((int)(hue/360.0));                                // cycle to have hue in [0,360]
    vcl_cout << "sid = " << (*urs2)[u]->get_sid() << " hue = " << hue << vcl_endl;
    HSVtoRGB( &r, &g, &b, hue, 0.6, 200.0 );
    vil1_rgb<unsigned char> col((int)r, (int)g, (int)b);
    draw_uniform_region((*urs2)[u], col, buf_out2);
  }
  vil1_save(*buf_out2, "trash.tif");
  
  vcl_cout << "Done" << vcl_endl;
} // main

