#include <vcl/vcl_list.h>
#include <vil1/vil1_image.h>
#include <vil1/vil1_image_as.h>
#include <vil1/vil1_load.h>
#include <vil1/vil1_save.h>
#include <vil1/vil1_smooth.h>
#include <vil1/vil1_pixel.h>

#include <vgl/vgl_homg_point_2d.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <regions/ell_region.h>
#include <region_extraction/reg_ext_starter.h>
#include <region_extraction/reg_ext_params.h>
#include <region_extraction/ell_region_extractor.h>
#include <region_utility/Vitto_region_file.h>
#include <misc_utility/defines.h>

#include <region_matching/invariant_computor.h>


// Invariant computation parameters
bool COLOR = true;
bool SWITCH = false;
bool ROT = true;  // ell regions need rot invs, so they are less


int main(int argc, char **argv)
{
  if (argc < 4)
  {
    vcl_cout << "Usage: " << argv[0] << " image regs_in.regmV regs_out.regmV" << vcl_endl;
    return 0;
  }

  // Read image
  vcl_cout << "Loading image" << vcl_endl;
  vil1_image image = vil1_load(argv[1]);
  color_buffer_t org_buf(image);  // original buffer, as load from file
  int sizex = org_buf.cols();
  int sizey = org_buf.rows();

  // Load regions
  vcl_cout << "Loading regions " << argv[2] << " ..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t regions;
  vrf.load_regions(regions, argv[2], &image);
  vcl_cout << regions.size() << " regions loaded" << vcl_endl;

  // Computing invariants
  invariant_computor ic_ell(&org_buf,true, false, true);   // ell invs are rot inv (last param)
  int NB_INV_ELL = ic_ell.get_max_nb_inv();
  vcl_cout << "Number of invariants for ell regions: " << ic_ell.get_nb_inv(true, true) << vcl_endl;
  invariant_computor ic_par(&org_buf,true, false, false);   // par invs are not rot inv (last param)
  int NB_INV_PAR = ic_par.get_max_nb_inv();
  vcl_cout << "Number of invariants for parallelogram regions: " << ic_par.get_nb_inv(true, false) << vcl_endl;

  region* reg;
  for(int i=0; i<regions.size(); i++) {
    reg = regions[i];

    vnl_vector<double>* inv = NULL;
    if(reg->get_type() == region::ELL_REGIONTYPE) {
      inv = new vnl_vector<double>(NB_INV_ELL, 0.0);
      ic_ell.set_region((ell_region*)reg);
      ic_ell.get_invariants(*inv);
      (*inv)[0] = 21;  // just to make sure ... as some little chaos in this region tye and reg spec type story ...
      ((ell_region*)reg)->set_invariants(inv);     
    } else {
      inv = new vnl_vector<double>(NB_INV_PAR, 0.0);
      ic_par.set_region(reg);
      ic_par.get_invariants(*inv);
      reg->set_invariants(inv); 
      vcl_cout << "fucking bug. inv == " << (*inv) << vcl_endl;
    }
  }

  // Saving regions
  vcl_cout << "Saving regions to file" << vcl_endl;
  vrf.save_regions(regions, argv[3]);

  return 1;
}
