// Module: FMatrixCompute example
// Author: Andrew W. Fitzgibbon, Oxford RRG, July 96
// Adapted to mvl by Peter Vanroose, January 2002.
// Bent to his needs by V. Ferrari, Nov 2003

#include <vcl_iostream.h>
#include <vcl_fstream.h>               // files I/O
#include <vcl_vector.h>
#include <mvl/FMatrix.h>
#include <mvl/HomgPoint2D.h>
//#include <mvl/FMatrixComputeLinear.h>
//#include <mvl/FMatrixComputeLMedSq.h>
#include <vxl_mvl_fix/FMatrixComputeMLESACFix.h>  // cool, my version of it !
#include <misc_utility/loaders.h>      // load_points, save_bools


int main(int argc, char**argv) {
  // Check syntax
  if(argc < 4) {
    vcl_cout << "Usage: " << argv[0] << " points1.dat points2.dat F.dat" << vcl_endl;
    return -1;
  }

  // Load points
  vcl_vector<HomgPoint2D> points1 = load_points(argv[1]);
  vcl_vector<HomgPoint2D> points2 = load_points(argv[2]);

  // Perform the fit
  FMatrixComputeMLESACFix computor(true, 900);   // true -> trunkate to rank 2, second param = sigma; 900->just keep all matches (no filtering)
  FMatrix F = computor.compute(points1, points2);

  //double d = 0;
  //for(unsigned int i = 0; i < points1.size(); ++i) {
  //  double dl = F.image1_epipolar_distance_squared(&points1[i], &points2[i]);
    //vcl_cout << "dist = " << sqrt(dl) << vcl_endl;
  //  d += dl; 
  //}
  //vcl_cout << "Error = " << d/points1.size() << vcl_endl;
  //vcl_cout << "F = " << F << vcl_endl;

  // Output F to file
  char* FMatrixFile = argv[3];
  if(FMatrixFile) {
    //vcl_cout << "Saving F-matrix to file " << FMatrixFile << vcl_endl;
    vcl_ofstream outfile(FMatrixFile);
    outfile << F;
    outfile.close();
  }

} // main


