#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <cstdlib>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_save.h>
#include <vil1/vil1_load.h>
#include <misc_utility/defines.h>                // def of color_buffer_t
#include <region_utility/Vitto_region_file.h>
#include <region_utility/region_draw.h>          // overlap_share, draw_region
#include <region_utility/misc.h>                 // filter_size_regions
#include <misc_utility/loaders.h>                // save_ints
#include <region_utility/clusterfilter.h>

int main(int argc, char** argv) {
  if (argc != 7) {
    vcl_cout << "Usage: " << argv[0] 
	     << " regs.regmV image min_main_cluster_size min_cluster_size out.regmV out_sid.dat" << vcl_endl;
    return 1;
  }

  // Process arguement
  int min_main_cluster_size = atoi(argv[3]);
  int min_cluster_size = atoi(argv[4]);
 
  // Load regions
  vcl_cout << "Loading regions in Vitto format..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t regions;
  vrf.load_regions(regions, argv[1], (vil1_image*)NULL);         // last param just needed by signature
  vcl_cout << regions.size() << " regions loaded" << vcl_endl;

  // Load image
  vil1_image image = vil1_load(argv[2]);

	regions_list_t regs_filt;
	vcl_vector<int> sid_filt;

	ClusterFilter filt( image, true );
	filt.filter( regions, min_main_cluster_size, min_cluster_size, &sid_filt, &regs_filt );

  // Save survived regions
  vrf.save_regions(regs_filt, argv[5]);

  // Save sids of survived regions
  save_ints(sid_filt, argv[6]);

} // main


