#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <cstdlib>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_save.h>
#include <misc_utility/defines.h>                // def of color_buffer_t
#include <region_utility/Vitto_region_file.h>
#include <region_utility/region_draw.h>          // overlap_share, draw_region
#include <region_utility/misc.h>                 // filter_size_regions
#include <misc_utility/loaders.h>                // save_ints


int main(int argc, char** argv) {
  if (argc != 7) {
    vcl_cout << "Usage: " << argv[0] 
	     << " regs.regmV min_size max_size max_overlap out.regmV out_sid.dat" << vcl_endl;
    return 1;
  }

  // Process arguement
  float min_size = atof(argv[2]);
  float max_size = atof(argv[3]);
  float max_overlap = atof(argv[4]);
 
  // Load regions
  vcl_cout << "Loading regions in Vitto format..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t regions;
  vrf.load_regions(regions, argv[1], (vil1_image*)NULL);         // last param just needed by signature
  vcl_cout << regions.size() << " regions loaded" << vcl_endl;

  // Create an image
  color_buffer_t* buf = new color_buffer_t(1000,1000);          // hardwired max size, should be found by abs max of x,y coords in regions

  // Size filter
  vcl_cout << "Filtering out all regions with size < " << min_size << " or size > " << max_size << vcl_endl;
  regions_list_t rs = filter_size_regions(regions, min_size, max_size);
  vcl_cout << rs.size() << " regions kept." << vcl_endl;

  // Draw regions on image
  regions_list_t out_rs;
  vil1_rgb<unsigned char> col(255, 255, 255);                    // regs drawn in white
  vcl_cout << "Composing mosaic with max overlap " << max_overlap << vcl_endl;
  for(int rx=0; rx<rs.size(); rx++) {
    region* r = rs[rx];
    float s = overlap_share(r, col, buf);
    if(s <= max_overlap) {
      draw_region(r, col, buf);
      out_rs.push_back(r);
      vcl_cout << "  region " << r->get_sid() << " added, with " << s << " overlap." << vcl_endl;
    }
  }

  // Saving filtered regions
  vcl_cout << "Saving " << out_rs.size() << " regions to " << argv[5] << vcl_endl;
  vrf.save_regions(out_rs, argv[5]);

  // Saving sid of filtered regions
  vcl_cout << "Saving " << out_rs.size() << " sids to " << argv[6] << vcl_endl;
  vcl_vector<int> sids;
  for(int rx=0; rx<out_rs.size(); rx++) {
    sids.push_back(out_rs[rx]->get_sid());
  }
  save_ints(sids, argv[6]);

  vcl_cout << "Done." << vcl_endl;
} // main


