#include <vcl/vcl_list.h>
#include <vcl/vcl_iostream.h>
#include <vil1/vil1_image.h>
#include <vil1/vil1_load.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <region_utility/old_region_file.h>

#include <region_utility/aoi_computor.h>        // aoi computation

#include <region_extraction/tuytelaars_region_extractor.h>


void process_arguements(int argc, char** argv, bool& noback, bool& turn_aoi) {
  for(int a = 3; a < argc; a++) {
    if(strcmp(argv[a],"-noback") == 0) {
      noback = true;
      vcl_cout << "Filter out background regions." << vcl_endl;
    }

    if(strcmp(argv[a],"-turn-aoi") == 0) {
      turn_aoi = true;
      vcl_cout << "Turn regions towards aoi." << vcl_endl;
      vcl_cout << "NOT IMPLEMENTED YET. SORRY !" << vcl_endl;
    }
  }
}


int main(int argc, char **argv)
{
  if (argc < 3)
  {
    vcl_cout << "Usage: " << argv[0] << " image regs_out.rege2nfs [-noback] [-turn-aoi]" << vcl_endl;
    return 0;
  }

  // Processing arguements
  bool noback = false;
  bool turn_aoi = false;
  process_arguements(argc, argv, noback, turn_aoi);

  // Read image
  vcl_cout << "Loading image" << vcl_endl;
  vil1_image image = vil1_load(argv[1]);

  regions_list_t regions;                 // final regions to be returned

	TuyteRegionExtractor xtractor( &image, noback, true );
	xtractor.extractEllRegions( regions );

  // turn regions towards aoi
  // TODO: not done yet because of twofold ambiguity

  // Saving regions
  vcl_cout << "Saving regions to file" << vcl_endl;
  old_region_file orf;
  orf.save_regions(regions, argv[2]);

	// clean up
	for( regions_list_t::iterator i=regions.begin(); i!=regions.end(); ++i )
		delete *i;
	
  return 1;
}
