#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <cstdlib>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_load.h>
#include <vil1/vil1_save.h>
#include <misc_utility/defines.h>                          // def of color_buffer_t
#include <misc_utility/gaussian_smoothing.h>               // vic_gaussian_smoothing
#include <misc_utility/region_growing.h>                   // grow_uniform_region
#include <misc_utility/loaders.h>                          // save_uniform_region
#include <regions/uniform_region.h>
#include <region_utility/region_draw.h>                    // draw_uniform_region     

int main(int argc, char** argv)
{
  if (argc != 6) {
    vcl_cout << "Usage: " << argv[0] 
	     << " image_input.tif seed_x seed_y region_file.unif region_image.tif" << vcl_endl;
    return 1;
  }

  // Load images
  vcl_cout << "Loading image..." << vcl_endl;
  vil1_image image = vil1_load(argv[1]);
  color_buffer_t* buf = new color_buffer_t(image);

  // Get seed
  int seed_x = atoi(argv[2]);
  int seed_y = atoi(argv[3]);
  vcl_cout << "Seed = (" << seed_x << ", " << seed_y << ")" << vcl_endl;

  // Smooth image in the 3 color bands
  vcl_cout << "Smoothing image..." << vcl_endl;
  color_buffer_t* buf_smooth = new color_buffer_t(image);                 
  vic_gaussian_smoothing(*buf, *buf_smooth);
  color_buffer_t* buf_out = new color_buffer_t(image);              // copy
  vic_gaussian_smoothing(*buf, *buf_out);                           // smooth twice as cannot make a copy of a color_buffer_t, only of a vil1_image

  // Grow uniform region
  int w = buf->width();
  int h = buf->height();
  bool* xflag = new bool[w*h];                                      // grown and accepted regions flag, these pixels can't be reused, in this app, just empty
  for(int y=0; y<h; y++) {
    for(int x=0; x<w; x++) {
      xflag[x + y*w] = false;
    }
  }
  int area = 0;
  bool* flag = grow_uniform_region(buf_out, xflag, area, seed_x, seed_y);             

  // Save region to file
  uniform_region* ur = new uniform_region(flag, buf_smooth);        // but_smooth used to get mean color
  ur->set_sid(1);
  uniform_regions_list_t* urs = new uniform_regions_list_t();
  urs->push_back(ur);
  save_uniform_regions(urs, argv[4]);                 

  // Save output buffer
  vil1_rgb<unsigned char> col(0, 255, 0);
  draw_uniform_region(ur, col, buf_out);
  vil1_save(*buf_out, argv[5]);

  vcl_cout << "Done" << vcl_endl;
} // main

