#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_save.h>
#include <vil1/vil1_load.h>

#include <cstdlib>

#include <misc_utility/defines.h>                // def of color_buffer_t
#include <region_utility/Vitto_region_file.h>
#include <region_utility/region_draw.h>          // overlap_share, draw_region
#include <misc_utility/loaders.h>                // save_ints
#include <region_utility/misc.h>                 // bounding_box, bb_intersect


int main(int argc, char** argv) {
  if (argc != 6) {
    vcl_cout << "Usage: " << argv[0] 
	     << " regs1.regmV regs2.regmV min_overlap intersection.regmV difference.regmV" << vcl_endl;
    return 1;
  }

  // Process arguements
  float min_overlap = atof(argv[3]);
 
  // Load regions 1
  vcl_cout << "Loading regions " << argv[1] << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t rs1;
  vrf.load_regions(rs1, argv[1], (vil1_image*)NULL);             // last param just needed by signature
  vcl_cout << rs1.size() << " regions loaded" << vcl_endl;

  // Load regions 2
  vcl_cout << "Loading regions " << argv[2] << vcl_endl;
  regions_list_t rs2;
  vrf.load_regions(rs2, argv[2], (vil1_image*)NULL);             // last param just needed by signature
  vcl_cout << rs2.size() << " regions loaded" << vcl_endl;

  // Automatic esitmates image size needed
  point_t pmin, pmax;
  regions_list_t rs12(rs1);
  for(int r_inx = 0; r_inx < rs2.size(); r_inx++) {
    rs12.push_back(rs2[r_inx]);
  }
  vcl_cout << rs12.size() << " total regions" << vcl_endl;
  bounding_box(rs12, pmin, pmax);
  vcl_cout << "Regions distributed over box: (" << pmin.x() << "," << pmin.y() << ") -> (" << pmax.x() << "," << pmax.y() << ")" << vcl_endl;
  if(pmin.x() < 0 || pmax.y() < 0) {
    vcl_cout << "WARNING: some regions have coordinates <0. Unpredictable behavior" << vcl_endl;
  }

  // Create empty working buffer of the correct size
  color_buffer_t* buf = new color_buffer_t((int)pmax.x(), (int)pmax.y());

  // Draw regions 2 on working buffer
  vil1_rgb<unsigned char> col(255, 255, 255);                    // regs drawn in white
  vcl_cout << "Drawing " << argv[2] << " on image" << vcl_endl;
  for(int rx=0; rx<rs2.size(); rx++) {
    region* r = rs2[rx];
    draw_region(r, col, buf);
  }

  // Which regions of rs1 are overlapping with rs2 (intersection),
  // and which are not (difference) ?
  regions_list_t rs_int;
  regions_list_t rs_diff;
  vcl_cout << "Determining intersection and difference sets" << vcl_endl;
  for(int rx=0; rx<rs1.size(); rx++) {
    region* r = rs1[rx];
    float s = overlap_share(r, col, buf);
    if(s > min_overlap) {                                       // do not use '>=': if min_overlap==0 -> no reg can be non-overlapping
      rs_int.push_back(r);
    } else {
      rs_diff.push_back(r);
    }
  }
  vcl_cout << rs_int.size() << " intersecting regions in " << argv[1] << vcl_endl;
  vcl_cout << rs_diff.size() << " different regions in " << argv[1] << vcl_endl;
  
  // Saving intersecting regions
  vcl_cout << "Saving " << rs_int.size() << " regions to " << argv[4] << vcl_endl;
  vrf.save_regions(rs_int, argv[4]);

  // Saving different (non-intersecting) regions
  vcl_cout << "Saving " << rs_diff.size() << " regions to " << argv[5] << vcl_endl;
  vrf.save_regions(rs_diff, argv[5]);

  // Cleaning
  delete buf;

  vcl_cout << "Done." << vcl_endl;
} // main
