#include <vcl/vcl_list.h>
#include <vil1/vil1_image.h>
#include <vil1/vil1_image_as.h>
#include <vil1/vil1_load.h>
#include <vil1/vil1_save.h>
#include <vil1/vil1_smooth.h>
#include <vil1/vil1_pixel.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <regions/ell_region.h>
#include <region_matching/reg_match_params.h>
#include <region_matching/rotation_finder.h>
#include <region_utility/Vitto_region_file.h>
#include <region_utility/misc.h>                // copy_region
#include <misc_utility/defines.h>

int main(int argc, char **argv)
{
  if (argc < 4)
  {
    vcl_cout << "Usage: " << argv[0] << " image1 regs_in1.regmV image2 regs_in2.regmV regs_out1.regmV" << vcl_endl;
    return 0;
  }

  // Read images
  vcl_cout << "Loading image " << argv[1] << vcl_endl;
  vil1_image image1 = vil1_load(argv[1]);
  color_buffer_t buf1(image1);
  vcl_cout << "Loading image " << argv[3] << vcl_endl;
  vil1_image image2 = vil1_load(argv[3]);
  color_buffer_t buf2(image2);

  // Load regions
  vcl_cout << "Loading regions " << argv[2] << " ..." << vcl_endl;
  Vitto_region_file vrf;
  regions_list_t regions1;
  vrf.load_regions(regions1, argv[2], &image1);
  vcl_cout << regions1.size() << " regions loaded" << vcl_endl;
  vcl_cout << "Loading regions " << argv[4] << " ..." << vcl_endl;
  regions_list_t regions2;
  vrf.load_regions(regions2, argv[4], &image2);
  vcl_cout << regions2.size() << " regions loaded" << vcl_endl;

  // Rotating regions of image1 towards corresponding regions of image2
  reg_match_params params;
  rotation_finder rotf(&buf1, &buf2);                                      // check color sim meas
  rotf.set_resolution(params.rotation_finder_resolution);
  rotf.set_stepsize(params.search_for_rotation_stepsize);
  regions_list_t new_regions1;
  vcl_cout << "Fixing rotations ..." << vcl_endl;
  for(int i=0; i<regions1.size(); i++) {
    ell_region* er1 = (regions1[i])->cast_to_ellregion();
    ell_region* er2 = (regions2[i])->cast_to_ellregion();
    rotf.set_region1(er1);
    rotf.set_region2(er2);
    rotf.get_crosscorrelation();
    region* rr1t = rotf.get_rotated_region1();  // could be corrupted after rerunning rotf ! Do not use this ptr further
    new_regions1.push_back(copy_region(rr1t));
  }

  // Saving regions
  vcl_cout << "Saving rotation-fixed regions " << argv[2] << " to file" << vcl_endl;
  vrf.save_regions(new_regions1, argv[5]);

  return 1;
}
