#include <string>
using namespace std;

#include <vcl/vcl_list.h>
#include <vcl/vcl_utility.h>
#include <vcl/vcl_iostream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_load.h>

#include <cstdlib>

#include <misc_utility/defines.h>
#include <regions/parall_region.h>
#include <regions/ell_region.h>
#include <region_utility/Vitto_region_file.h>   // saving
#include <region_utility/misc.h>                // saving
#include <region_utility/region_draw.h>         // color_share
#include <misc_utility/count_pixels.h>          // border_color
#include <misc_utility/gaussian_smoothing.h>    // vic_gaussian_smoothing
#include <misc_utility/dilation.h>              // dilation
#include <region_utility/misc.h>                // greylevel_variance
#include <vil1/vil1_image_as.h>                 // greylevel conversion

#include <region_extraction/tesselator.h>


void process_arguements(int argc, char** argv, int& first_sid) {
	for(int a = 7; a < argc; a++) {
		if(strcmp(argv[a],"-first") == 0) {
			first_sid = (int) atof(argv[a+1]);
			vcl_cout << "First sid = " << first_sid << vcl_endl;
		}
	}
}

string makeMapName( const char *fName )
{
	string mapFileName = fName;
	int extPos = mapFileName.rfind('.');
	int slashPos = mapFileName.rfind('/');
	if( slashPos == string::npos )
		slashPos = -1;
	mapFileName.insert(extPos, "-map");
	mapFileName.insert(slashPos+1, "maps/");
	
	return mapFileName;
}

int fileIsValid( string & fName )
{
	ifstream test( fName.c_str() );
	return test.good();
}


int main(int argc, char** argv)
{
	if (argc < 7)
	{
		vcl_cout << "Usage: " << argv[0] << " image width height x_step y_step regs_out.regmV [-first first_sid]"  << vcl_endl;
		vcl_cout << "Default: first_sid = 1" << vcl_endl;
		abort();
	}
	
	// Processing arguements
	int first_sid = 1;
	process_arguements(argc, argv, first_sid);
	
	// load image
	vcl_cout << "Loading image " << argv[1] << " ..." << vcl_endl;
	vil1_image image = vil1_load(argv[1]);
	bool hasMap = true;
	
	// load background map, if any
	bw_buffer_t *theMap = NULL;
	string sMapName = makeMapName( argv[1] );
	if( ! fileIsValid( sMapName ) ) {
		// try other format
		int extPos = sMapName.rfind('.');
		if( sMapName.substr( extPos, 4 ) == ".png" )
			sMapName.replace( extPos, 4, ".jpg" );
		else
			sMapName.replace( extPos, 4, ".png" );
		
		if( ! fileIsValid( sMapName ) )
			hasMap = false;
	}
	
	if( hasMap ) {
		vcl_cout << "Map found in file " << sMapName << vcl_endl;
		vil1_image mapImg = vil1_image_as_byte( vil1_load( sMapName.c_str() ) );

		// segmentation map is dilated to ensure complete coverage of object
		bw_buffer_t tmpBuf(mapImg);
		theMap = new bw_buffer_t(tmpBuf);
		dilation5(tmpBuf, *theMap);
	}

	// Tesselation
	Tesselator tess( image, theMap );
	tess.setVerbose(true);

	float rw = atof(argv[2]);
	float rh = atof(argv[3]);
	float xs = atof(argv[4]);
	float ys = atof(argv[5]);
	regions_list_t rs;
	
	tess.tesselate( rs, rw, rh, xs, ys, first_sid );
	
	// Save vitto format tesselating regions
	vcl_cout << "Saving " << rs.size() << " regions to " << argv[6] << vcl_endl;
	Vitto_region_file vrf;
	vrf.save_regions(rs, argv[6]);
	
	return 1;
}
