#include <vcl/vcl_list.h>
#include <vcl/vcl_utility.h>
#include <vcl/vcl_iostream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_load.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <regions/ell_region.h>
#include <region_utility/old_region_file.h>
#include <region_matching/reg_match_params.h>
#include <region_matching/ncc_region_matcher.h>
#include <region_matching/pairwise_indexed_matches.h>
#include <region_matching/pairwise_indexed_match.h>

int main(int argc, char** argv)
{
  if (argc != 5)
  {
    vcl_cout << "Usage: " << argv[0] << " image1 regions1 image2 regions2 " << vcl_endl;
    abort();
  }

  // load image
  vcl_cout << "Loading image1 " << argv[1] << " ..." << vcl_endl;
  vil1_image image1 = vil1_load(argv[1]);
  vil1_memory_image_of<vil1_rgb_byte> buf1(image1);

  // load regions
  vcl_cout << "Loading regions1 " << argv[2] << " ..." << vcl_endl;
  old_region_file orf;
  regions_list_t regions1;
  orf.load_regions(regions1, argv[2], &image1);
  vcl_cout << regions1.size() << " regions loaded" << vcl_endl;

  // load image
  vcl_cout << "Loading image2 " << argv[3] << " ..." << vcl_endl;
  vil1_image image2 = vil1_load(argv[3]);
  vil1_memory_image_of<vil1_rgb_byte> buf2(image2);

  // load regions
  vcl_cout << "Loading regions2 " << argv[4] << " ..." << vcl_endl;
  regions_list_t regions2;
  orf.load_regions(regions2, argv[4], &image2);
  vcl_cout << regions2.size() << " regions loaded" << vcl_endl;

  // match regions
  vcl_cout << "Matching regions ... " << vcl_endl;
  reg_match_params params;
  ncc_region_matcher nrm(&image1, &image2, regions1, regions2, &params);
  nrm.match_regions();
  pairwise_indexed_matches mtches;
  nrm.get_matches(mtches);

  // print out matches
  vcl_cout << "Matches found: " << vcl_endl;
  for (mtches.reset(); mtches.next_match(); )
  {
    match* m = mtches.get_match();
    pairwise_indexed_match* pwi_m = m->cast_to_pairwise_indexed_match();
    pwi_m->print(vcl_cout);
  }

  return 1;
}
