#include <vcl_sstream.h>
#include <vcl_iostream.h> 
#include <vcl_fstream.h>

#include <vil1/vil1_image.h>

#include <misc_utility/defines.h>
#include <region_utility/Vitto_region_file.h>
#include <region_utility/VIBES_region_file.h>

// Converts *.regvib files with regions in Vitto format
// to regions in VIBES format

int main(int argc, char** argv)
{
  if (argc < 3)
  {
    vcl_cout << "Usage: " << argv[0] 
	     << " image.regvib [image.regmV]*" << vcl_endl;
    return 1;
  }

  vil1_image* image = NULL; // dummy image 

  // Load all regiondatafiles passed as input
  regions_list_t regions;
  Vitto_region_file vrf;
  int tot = 0;
  for(int a = 2; a < argc; a++) {
    vcl_cout << "Loading regions datafile " << argv[a] << vcl_endl;
    vrf.load_regions(regions, argv[a], image);
    vcl_cout << regions.size()-tot << " regions loaded" << vcl_endl;
    tot = regions.size();
  }
  vcl_cout << "Total " << tot << "  regions loaded" << vcl_endl;

  vcl_cout << "Saving regions in VIBES format..." << vcl_endl;
 
  VIBES_region_file brf;
  brf.save_regions(regions, argv[1]);
  
  vcl_cout << "Done" << vcl_endl;
} // main
