#ifndef IMG_ANALYZER_H
#define IMG_ANALYZER_H

/* This class is more or less the equivalent of the LearningSingleView.m file */

#include <vcl/vcl_vector.h>
#include <vcl/vcl_string.h>

#include <misc_utility/defines.h>


class ImageAnalyzer
{
public:
	ImageAnalyzer( const vcl_vector<vcl_string> &featureTypes, bool bVerbose=false );
	~ImageAnalyzer();

	/* Generate a tesselation and extract features for an image.
	 * The region lists will be stored in the regionSets vector,
	 *   in the same order as in the featureTypes vector.
	 * 'radius' controls the scale of the tesselation, 'secondLayer'
	 *   determines if a second layer of smaller regions will be added.
	 * Each region will get a unique ID (sid) over all sets, starting at 1
	 *   for the first tesselated region.
	 * (Don't forget to delete the regions in the lists when finished.)
	 * Return value is 0 upon success. */
	int analyzeImage( regions_list_t &tesselation,
	                  vcl_vector<regions_list_t> &regionSets,
	                  const vil1_image &image, const char *imFile=NULL,
	                  const bw_buffer_t *theMap=NULL,
	                  float radius=64.0, bool secondLayer=true );

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	bool verbose;
	vcl_vector<vcl_string> fTypes;
};

#endif //IMG_ANALYZER_H
