#ifndef order_statistic_tree_h
#define order_statistic_tree_h

#include <cassert>
//#include "stl_utils.h"
#include <limits>

//#include "boost/limits.hpp"

// hier implementiert fuer minimum
template< class T >
class order_statistic_tree {
// protected:
public:
  T *_data;
  int _size;
  //static const int oo = 0x7fffffff;
public:
  order_statistic_tree() {
    _data = 0;
    _size = 0;
  }
  virtual ~order_statistic_tree() {
    delete[] _data;
  }

  virtual T op( T a, T b ) = 0;
  virtual T neutral_element() = 0;
  virtual int is_less( T a, T b ) = 0;

  void clear() {
    delete[] _data;
    _data = 0;
    _size = 0;
  }

  void resize( int size ) {
    int tmp = 1; // zweierpotenz
    while( tmp < size )
      tmp += tmp;
    size = tmp;

    if( _size == size )
      return;
    T *data = new T[ size*2 ];

    for( int i = 0; i < std::min( size, _size ); i++ )
      data[ size+i ] = _data[ _size+i ];
    for( int i = _size; i < size; i++ )
      data[ size+i ] = neutral_element();
    for( int i = size-1; i >= 1; i-- )
      data[i] = op( data[i+i], data[i+i+1] );
    delete[] _data;
    _data = data;
    _size = size;
  }

  T get_statistic( int l, int r ) { // inside of interval
    if( l >= r )
      return neutral_element();
    l += _size;
    r += _size;
    T res = _data[l];
    while( 1 ) {
      if( r-1 <= l )
	break;      
      if( r & 1 )
	res = op( res, _data[r-1] );
      if( !( l & 1 ) )
	res = op( res, _data[l+1] );
      r /= 2;
      l /= 2;
    }
    return res;
  }

  T operator[]( int i ) const {
    return _data[ _size+i ];
  }

  void set_value( int i, T v ) {
    i += _size;
    _data[i] = v;
    while( i ) {
      _data[i>>1] = op( _data[i], _data[i^1] );
      i >>= 1;
    }
  }
};

template< class T >
class minimum_statistic : public order_statistic_tree< T > {
 public:
  int is_less( T a, T b ) { return a < b; }
  T op( T a, T b ) { return std::min( a, b ); }
  T neutral_element() { return std::numeric_limits< T >::max(); }

  int find_index_of_minimum( int l, int r ) { // inside of interval
    if( l >= r )
      return 0;
    l += this->_size;
    r += this->_size;
    int mini = l;
    while( 1 ) {
      if( r-1 <= l )
	break;      
      if( ( r & 1 ) && ( this->_data[r-1] < this->_data[mini] ) )
	mini = r-1;
      if( !( l & 1 ) && ( this->_data[l+1] < this->_data[mini] ) )
	mini = l+1;
      r /= 2;
      l /= 2;
    }
    while( mini < this->_size ) {
      mini += mini;
      if( this->_data[mini] > this->_data[mini+1] )
	mini++;
    }
    return mini - this->_size;
  }
};

template< class T >
class sum_statistic : public order_statistic_tree< T > {
 public:
  int is_less( T a, T b ) { return a < b; }
  T op( T a, T b ) { return a + b; }
  T neutral_element() { return 0; }  

  // find an interval [0,r) with r <= n that has the statistic v.
  int find_less_than( T v ) {
    int r = 1;
    T res = neutral_element();
    while( r < this->_size ) {
      r += r;
      //      std::cout << r << " " << _data[r] << " " << res << " " << v << std::endl;
      //if( !is_less( op( _data[r], res ), v ) ) {
      if( op( this->_data[r], res ) < v ) {
	res = op( this->_data[r], res );
	r++;
      }
    }
    return r - this->_size;
  }

  int find_less_or_equal_than( T v ) {
    int r = 1;
    T res = neutral_element();
    while( r < this->_size ) {
      r += r;
      //      std::cout << r << " " << _data[r] << " " << res << " " << v << std::endl;
      //if( !is_less( op( _data[r], res ), v ) ) {
      if( op( this->_data[r], res ) <= v ) {
	res = op( this->_data[r], res );
	r++;
      }
    }
    return r - this->_size;
  }
};

#endif
