#ifndef REGIONS_REFINER_H
#define REGIONS_REFINER_H

/* Class version of RegionsRefinerC */

#include <misc_utility/defines.h>
#include <region_matching/crosscorrelation_computor.h>
#include <region_propagation/bounded_refiner.h>
#include <region_propagation/aoi_refiner.h>


class RegionsRefiner
{
public:
	/* Create a new RegionsRefiner.
	 * Set 'small' to true to use small search space,
	 * set 'use6dgd' to true to use simple 6D Gradient Descent mode. */
	RegionsRefiner( const color_buffer_t *buf1, const color_buffer_t *buf2,
	                bool bVerbose, bool small=false, bool use6dgd=false );
	~RegionsRefiner();

	/* Refine a set of regions.
	 * The refined regions2 will be stored in regions2_ref.
	 * If pre_sims or ref_sims is not NULL, the pre-, resp. post-refinement
	 * similarities will be stored in them. */
	void refine( const regions_list_t &regions1, const regions_list_t &regions2,
	             regions_list_t &regions2_ref,
	             vcl_vector<float> *pre_sims=NULL,
	             vcl_vector<float> *ref_sims=NULL );

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	bool verbose;
	const color_buffer_t *imgBuf1;
	const color_buffer_t *imgBuf2;
	crosscorrelation_computor *sc;
	bounded_refiner *ref;
	aoi_refiner *aoiref;
};

#endif //REGIONS_REFINER_H
