#ifndef RSC_FILTER_H
#define RSC_FILTER_H

/* This class is the equivalent of the RSCFilterC.m file and SidenessFilter.cpp. */

#include <complex>
using namespace std;

#include <misc_utility/defines.h>


typedef complex< double > pt;


class RSCFilter
{
public:
	RSCFilter( bool bVerbose=false );
	~RSCFilter();

	/* Filter regions.
	 * ref_regs and test_regs may be sparse, i.e. NULLs will be ignored,
	 * but they must correspond.
	 * The sids of the remaining regions will be stored in surviving_sids.
	 * If filter_history is not NULL, the sequence of sids of regions
	 * that were removed will be stored into it. */
	unsigned int filter( vcl_vector<int> &surviving_sids,
	                     const regions_list_t &ref_regs,
	                     const regions_list_t &test_regs,
	                     vcl_vector<int> *filter_history=NULL );

	/* Filter points. Caution: model and img will be modified!
	 * The IDs of the remaining points will be stored in surviving_ids.
	 * If ids is NULL, each point will get as ID its position
	 * in the vector (starting with 1).
	 * If filter_history is not NULL, the sequence of IDs of points
	 * that were removed will be stored into it. */
	unsigned int filter( vcl_vector<int> &surviving_ids,
	                     vcl_vector<pt> &model,
	                     vcl_vector<pt> &img,
	                     vcl_vector<int> *ids=NULL,
	                     vcl_vector<int> *filter_history=NULL );

	void test_report_points_on_left_side();
	int testRandom();

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	double normalize_angle( double a );
	vcl_vector<int> report_points_on_left_side( int i, vcl_vector<pt> model,
	                                            vcl_vector<pt> img );
	vcl_vector<int> report_points_on_left_and_right_side( int i, vcl_vector<pt> model,
	                                                      vcl_vector<pt> img );
	void init( const vcl_vector<pt> &model, const vcl_vector<pt> &img, vcl_vector<int> &cnt );
	void remove_point( int i, vcl_vector<pt> &model, vcl_vector<pt> &img, vcl_vector<int> &cnt );

	vcl_vector<int> report_points_on_left_side_( int i, vcl_vector<pt> model, vcl_vector<pt> img );
	vcl_vector<pt> rand_points( int n );
	vcl_vector<pt> normalizerand_points( int n );
	template<class T> void print( vcl_vector<T> r );

	bool verbose;
};

#endif //RSC_FILTER_H
