#ifndef VIEWPAIRMATCHER_H
#define VIEWPAIRMATCHER_H

/* This class is the equivalent of the InitialMatching.m file */

#include <vcl/vcl_vector.h>

#include <misc_utility/defines.h>
#include <region_matching/ncc_drgbsnorm_computor.h>   // similarity measure to be tested
#include <region_matching/intgr_ncc_drgbsnorm.h>
#include <region_matching/euclideansoftmatcher.h>


class ViewPairMatcher
{
public:
	ViewPairMatcher( const vil1_image &image1, const vil1_image &image2,
	                 bool nccOnly=false, bool bVerbose=false );
	~ViewPairMatcher();

	/* Match 2 sets of regions using SIFT descriptors. Each list of regions will be
	 * matched with the corresponding list in the other set. The regions must have
	 * descriptors.
	 * The results will be merged, refined and returned in finalRegions1,2.
	 * These are 2 corresponding lists of NEW regions, independant from the originals.
	 * If maxMatches is not 0, only the best maxMatches matches will be returned.
	 * If first_sid = -1, the original sids will be preserved (beware that due to soft
	 * matching, there will be multiple regions with the same sid).
	 * Set keepDescriptors to true to copy the original descriptors to the output. */
	int matchRegionSets( const vcl_vector<regions_list_t> &regionSets1,
	                     const vcl_vector<regions_list_t> &regionSets2,
	                     regions_list_t &finalRegions1,
	                     regions_list_t &finalRegions2,
	                     unsigned int maxMatches=0,
	                     int first_sid=-1,
	                     bool keepDescriptors=false );

	/* Compute the similarities between two sets of matching regions
	 * and return them as a vector of EuclMatches. */
	void compute_similarities( vcl_vector<EuclMatch> &vMatches,
	                           const regions_list_t &regions1,
	                           const regions_list_t &regions2,
                             int id_offset=0 );

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	bool verbose;
	color_buffer_t *imgBuf1;
	color_buffer_t *imgBuf2;
	similarity_computor *simCo;
	EuclideanSoftMatcher *siftMatcher;
};

#endif //VIEWPAIRMATCHER_H
