
/***********************************************************************
*
*		Author:	Aaron Luttman
*		Date:	June 19, 2001
*		Title:	ABLTimer
*		Notes:	This is a class for keeping track of processing 
*				time.  It should be statically allocated.  Call
*				the 'Start' function when you want to start keeping
*				track of processing time, then call 'Stop' when
*				you want to stop keeping track of processing time.
*				Get the results by calling GetIntSeconds (which returns
*				the number of seconds that the process took as
*				an int, which is more in line with the normal C
*				timing stuff), GetDoubleSeconds (which returns the
*				number of seconds as a double), or GetElapsedTime
*				(which returns a char* in the format HH:MM:SS.hh,tt
*				where hh and tt are hundredths and ten-thousandths
*				of a second, respectively.  The improvement over the
*				standard structs is that it uses sub-second accuracy,
*				which time_t does not.  This should be ANSI C/C++
*				compatible as it only includes 'time.h' and 'stdio.h.'
*	
***********************************************************************/

#include "time.h"

class ABLTimer
{
public:
	ABLTimer();
	~ABLTimer();

	void Start();
	void Stop();

	int GetIntSeconds();
	double GetDoubleSeconds();
	char* GetElapsedTime();

private:
	
	clock_t hStartClock;
	clock_t hEndClock;
	char *hTime;
	bool hDeleteTime;
	double hDuration;
};