#ifndef byte_interpol_h
#define byte_interpol_h

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>
#include <misc_utility/abl_utility.h>
#include <misc_utility/defines.h>

/***************************************************************************************************
 *
 *	Utility Functions: ByteInterpolation
 *	Authors:	Tinne Tuytelaars, Vittorio Ferrari, Andreas Turina
 *
 *	Port to vxl:	Aaron Luttman
 *	Date of port:	November 8, 2001
 *
 *	Notes:	These are all fairly straightforward ports.  The only major difference is that there
 *			no macros being used here.  I am using the vxl 'vil1_memory_image_of' instead of the
 *			TargetJr 'BufferXY.' Thus I only use the functionality of the image class to
 *			access pixels.	
 *
 ***************************************************************************************************/

inline byte grey_level(vil1_memory_image_of<vil1_rgb_byte>& img, int x, int y)
{
  vil1_rgb_byte pixel = img(x, y);
  double temp = (0.177*pixel.R() + 0.813*pixel.G() + 0.011*pixel.B());
  return (byte)temp;
}


inline byte get_byte_interpolation(vil1_memory_image_of<byte>& buf, double x, double y)
{
  vcl_cout << "STILL HAS TO BE IMPLEMENTED" << vcl_endl;
  return 0;
}


inline byte get_byte_interpolation_no_check(vil1_memory_image_of<byte>& buf, double x, double y)
{
  int xl = (int)x;
  int xh = xl+1;
  int yl = (int)y;
  int yh = yl+1;
	
  return (byte) (( (xh-x) * ((yh-y)*buf(xl, yl) + (y-yl)*buf(xl, yh)) +
                   (x-xl) * ((yh-y)*buf(xh, yl) + (y-yl)*buf(xh, yh)) )/(double)((xh-xl)*(yh-yl)));
}


inline double get_byte_interpolation_no_check_slowround(vil1_memory_image_of<byte>& buf, double x, double y)
{
  int xl = (int)floor(x-0.000001); // +/- 0.000001 such that xl != xh, yl != yh
  int xh = (int)ceil(x+0.000001);
  int yl = (int)floor(y-0.000001);
  int yh = (int)ceil(y+0.000001);
	
  byte tmp =  buf(xl, yl);
  tmp =  buf(xl, yh);
  tmp =  buf(xh, yl);
  tmp =  buf(xh, yh);

  return (byte) (( (xh-x) * ((yh-y)*buf(xl, yl) + (y-yl)*buf(xl, yh)) +
                   (x-xl) * ((yh-y)*buf(xh, yl) + (y-yl)*buf(xh, yh)) )/(double)((xh-xl)*(yh-yl)));
}


inline float get_interpolation_Vic(const bw_buffer_t* buf, 
				   float x, float y) {
  int xl = (int)x;
  int xh = xl+1;
  int yl = (int)y;
  int yh = yl+1;

  if(xl <= 0 || yl <= 0 || xh >= buf->width() || yh >= buf->height()) return 0;
	    
  return (float) (( (xh-x) * ((yh-y)*((*buf)(xl,yl)) + (y-yl)* ((*buf)(xl,yh))) +
		   (x-xl) * ((yh-y)*((*buf)(xh,yl)) + (y-yl)* ((*buf)(xh,yh))) )/((float)(xh-xl)/(yh-yl)));
}


inline vil1_rgb_byte get_col_interpolation_Vic(const color_buffer_t* buf,
					      float x, float y) {
  // Fast rounding, wrong if x,y exact integers
  int xl = (int)x;
  int xh = xl+1;
  int yl = (int)y;
  int yh = yl+1;

  if(xl <= 0 || yl <= 0 || xh >= buf->width() || yh >= buf->height()) return 0;

  // it will work as vil1_rgb<byte> has operators for '+' and '*',
  // but they everytime instantiate a new object.
  // Will be faster if do it by 'hand' (below)
  /*return   ( (xh-x) * ((yh-y)*((*buf)(xl,yl)) + (y-yl)*((*buf)(xl,yh))) +
	     (x-xl) * ((yh-y)*((*buf)(xh,yl)) + (y-yl)*((*buf)(xh,yh))) )/(xh-xl)/(yh-yl);*/

  // 'by hand' version: faster because avoids creation of many vil1_rgb_byte objects,
  vil1_rgb_byte ll = (*buf)(xl,yl);
  vil1_rgb_byte lh = (*buf)(xl,yh);
  vil1_rgb_byte hl = (*buf)(xh,yl);
  vil1_rgb_byte hh = (*buf)(xh,yh);
  vxl_byte r = (vxl_byte) ( (xh-x) * ((yh-y)*ll.R() + (y-yl)*lh.R()) +
			    (x-xl) * ((yh-y)*hl.R() + (y-yl)*hh.R()) )/(xh-xl)/(yh-yl);
  vxl_byte g = (vxl_byte) ( (xh-x) * ((yh-y)*ll.G() + (y-yl)*lh.G()) +
			   (x-xl) * ((yh-y)*hl.G() + (y-yl)*hh.G()) )/(xh-xl)/(yh-yl);
  vxl_byte b = (vxl_byte) ( (xh-x) * ((yh-y)*ll.B() + (y-yl)*lh.B()) +
			    (x-xl) * ((yh-y)*hl.B() + (y-yl)*hh.B()) )/(xh-xl)/(yh-yl);

  return vil1_rgb_byte(r,g,b);
}


inline float dist_rgb(vil1_rgb_byte& pix1, vil1_rgb_byte& pix2) {
  float dr = pix1.R() - pix2.R();
  float dg = pix1.G() - pix2.G();
  float db = pix1.B() - pix2.B();

  return sqrt( dr*dr + dg*dg + db*db );
}


// now with scalefactor on both regions
inline float dist_rgb_sf(vil1_rgb_byte& pix1, float sf1, vil1_rgb_byte& pix2, float sf2) {
  float dr = pix1.R()*sf1 - pix2.R()*sf2;
  float dg = pix1.G()*sf1 - pix2.G()*sf2;
  float db = pix1.B()*sf1 - pix2.B()*sf2;

  return sqrt( dr*dr + dg*dg + db*db );
}

#endif // end of file



