
#include <cstdlib>
#include <unistd.h>
#include <sys/stat.h>
#include <vcl/vcl_string.h>
#include <vcl/vcl_iostream.h>

#include "executor.h"


Executor::Executor( bool bVerbose )
{
	verbose = bVerbose;
	char *env = getenv("EXPLORATION_TOOLS_PATH");
	if( env != NULL )
		sExpToolsPath = env;
	else {
		vcl_cerr << "Warning: $EXPLORATION_TOOLS_PATH is not set" << vcl_endl;
		sExpToolsPath = "";
	}
}

Executor::~Executor()
{
	
}


int Executor::execute( const vcl_string &command, bool usePath )
{
	vcl_string theCmd;
	if( usePath && isCustomCmd(command) )
		theCmd = sExpToolsPath + "/" + command;
	else theCmd = command;
	
	if(verbose)
		vcl_cout << "Invoking command:\n" << theCmd << vcl_endl;
	
	int result = system(theCmd.c_str());
	if( result != 0 )
		vcl_cerr << "ERROR: execution failed: " << theCmd << vcl_endl;
	
	return result;
}


bool Executor::isCustomCmd( const vcl_string &command )
{
	if( sExpToolsPath == "" )
		return false;

	vcl_string sCmd = command;
	vcl_string::size_type sPos = sCmd.find(" ");
	if( sPos != vcl_string::npos )
		sCmd = sCmd.substr( 0, sPos );
	sCmd = sExpToolsPath + "/" + sCmd;

	/* Check if the file exists */
	struct stat buffer;
	if( stat( sCmd.c_str(), &buffer ) != 0 )
		return false;

	/* Check if we can execute the file */
	if( buffer.st_uid == getuid() )
		return buffer.st_mode & S_IXUSR;
	else if( buffer.st_gid == getgid() )
		return buffer.st_mode & S_IXGRP;
	else
		return buffer.st_mode & S_IXOTH;
}
