#ifndef EXECUTOR_H
#define EXECUTOR_H

/* A simple class for executing external binaries, either at the location indicated
 * by the environment variable $EXPLORATION_TOOLS_PATH, or available in $PATH. */

#include <cstdlib>
#include <vcl_string.h>


class Executor
{
public:
	Executor( bool bVerbose=true );

	~Executor();

	/* Execute a command. If usePath is false, the command will be executed without
	 * attempting to prepend $EXPLORATION_TOOLS_PATH, this is useful for general
	 * commands. Otherwise, the Executor will first look for the command in
	 * $EXPLORATION_TOOLS_PATH, and if it is not found, it will try to execute the
	 * command as-is.
	 * The return value is 0 upon success. */
	int execute( const vcl_string &command, bool usePath=true );

	/* This will return true if the command is available at $EXPLORATION_TOOLS_PATH. */
	bool isCustomCmd( const vcl_string &command );

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	/* Data members */

	bool verbose;
	vcl_string sExpToolsPath;
};

#endif //EXECUTOR_H
