
#include <misc_utility/loaders.h>
#include <vcl_fstream.h>                                // files I/O
#include <vcl_iostream.h>                               // files I/O
#include <region_utility/region_file.h>                 // end_of_line

vcl_vector<HomgPoint2D> load_points(const char* filename) {
  // open the file
  vcl_ifstream pointsfile(filename);

  // load points
  double x,y;
  vcl_vector<HomgPoint2D> points;
  while (pointsfile >> x) {
     pointsfile >> y;
     points.push_back(HomgPoint2D(x,y));
  }

  // close the file
  pointsfile.close();

  return points;
} // load_points


void save_bools(vcl_vector<bool>& vals, const char* filename) {
  vcl_ofstream outfile(filename);

  int i = 0;
  while(i < vals.size()) {
    outfile << vals[i] << " ";
    i++;
  }

  outfile.close();
} // save_bools


void save_ints(vcl_vector<int>& vals, const char* filename) {
  vcl_ofstream outfile(filename);

  int i = 0;
  while(i < vals.size()) {
    outfile << vals[i] << " ";
    i++;
  }

  outfile.close();
} // save_ints


void save_floats(vcl_vector<float>& vals, const char* filename) {
  vcl_ofstream outfile(filename);

  int i = 0;
  while(i < vals.size()) {
    outfile << vals[i] << " ";
    i++;
  }

  outfile.close();
} // save_floats


void save_floats(vcl_vector<float>** vals, const int size1, const char* filename) {
  vcl_ofstream outfile(filename);

  for(int i = 0; i < size1; i++) {
    vcl_vector<float>* ivals = vals[i];
    for(int j = 0; j < ivals->size(); j++) {
      outfile << (*ivals)[j] << " ";
    }
    outfile << vcl_endl;
  }

  outfile.close();
} // save_floats





void save_uniform_regions(uniform_regions_list_t* urs, const char* filename) {
  vcl_ofstream outfile(filename);

  // set im_ID
  char* im_ID = new char[10];
  int i=0;
  while ( (filename[i] < '0' ) || (filename[i] > '9'))
    i++;
  int i0 = i;
  while ( (filename[i] >= '0' ) && (filename[i] <= '9')) 
  {
    im_ID[i-i0] = filename[i];
    i++;
  }
  im_ID[i-i0] = '\0';

  // loop over urs
  for(int u=0; u<urs->size(); u++) {
    uniform_region* ur = (*urs)[u];

    // Writeout im_id and sid
    outfile << im_ID << " " << ur->get_sid() << " ";

    // Writeout color mean
    vil1_rgb<unsigned char> col = ur->get_color();
    outfile << (int)col.R() << " " << (int)col.G() << " " << (int)col.B() << " ";

    // Writeout segments
    for(int s=0; s < ur->number_of_segments(); s++) {
      point_t* start = ur->get_segment_start(s); 
      point_t* end = ur->get_segment_end(s);
      
      outfile << start->x() << " " << start->y() << " ";
      outfile << end->x() << " " << end->y() << " ";
    }
    outfile << vcl_endl;
  }

  outfile.close();
} // save_uniform_region


vcl_vector<float>* load_floats(const char* filename) {
  vcl_ifstream infile(filename);

  vcl_vector<float>* out = new vcl_vector<float>;

  float val;
  while(infile >> val) {
    out->push_back(val);
  }

  infile.close();
  return out;
} // save_floats



uniform_regions_list_t* load_uniform_regions(const char* filename) {
  vcl_ifstream infile(filename);

  uniform_regions_list_t* urs = new uniform_regions_list_t();
  int r,g,b, im_ID, sid;
  while(infile >> im_ID) { 
    uniform_region* ur = new uniform_region();

    // Load sid
    infile >> sid;
    ur->set_sid(sid);

    // Load color
    infile >> r;
    infile >> g;
    infile >> b;
    vil1_rgb<unsigned char> col(r,g,b);
    ur->set_color(col);

    // Load and construct the region (segment by segment)
    int xs,ys,xe,ye;
    while(!end_of_line(infile) && infile >> xs) {
      infile >> ys;
      infile >> xe;
      infile >> ye;
      ur->add_segment(xs,ys,xe,ye);
    } // read next segment
    urs->push_back(ur);
  } // still some new line in infile ?

  infile.close();
  return urs;
} // load_uniform_region


// Helper functions
bool end_of_line(vcl_ifstream& regfile) {
  char c = regfile.get();

  // Skip spaces
  while((int)c == 32) {
    c = regfile.get();
  }

  bool eol = ((int)c==13 || (int)c==10);
  regfile.unget();  // restore correct internal stream pointer position before eating the char
  return eol;
}

bool is_digit(vcl_ifstream& regfile) {
  char c = regfile.get();

  // Skip spaces
  while((int)c == 32) {
    c = regfile.get();
  }

  bool digit = (((int)c >= 48) & ((int)c <= 57));
 
  regfile.unget();  // restore correct internal stream pointer position before eating the char
  return digit;
}

void skip_to_eol(vcl_ifstream& regfile) {
  bool eol = false;
  while(!eol) {
    char c = regfile.get();
    eol = ((int)c==13 || (int)c==10);
  }
}
