#include <misc_utility/morphing.h>


// Projects maskbuf pixels of buf1 to buf2 via H
void homography_morph(color_buffer_t* buf1, color_buffer_t* buf2, color_buffer_t* maskbuf, HMatrix2D H)
{
	// Get buf1 frame in buf2
	HomgPoint2D tl(H * HomgPoint2D(0,0));                               // top-left corner
	HomgPoint2D tr(H * HomgPoint2D(buf1->width()-1,0));                 // top-right
	HomgPoint2D bl(H * HomgPoint2D(0,buf1->height()-1));                // bottom-left
	HomgPoint2D br(H * HomgPoint2D(buf1->width()-1,buf1->height()-1));  // bottom-right
	tl = tl / tl.w(); // Fixed??? replaced z() by w(), how can this ever have compiled?
	tr = tr / tr.w();
	bl = bl / bl.w();
	br = br / br.w();

	// Compute BB in buf2
	double minx = MINFOUR(tl.x(), tr.x(), bl.x(), br.x());
	double maxx = MAXFOUR(tl.x(), tr.x(), bl.x(), br.x());
	double miny = MINFOUR(tl.y(), tr.y(), bl.y(), br.y());
	double maxy = MAXFOUR(tl.y(), tr.y(), bl.y(), br.y());

	// Clip BB to boundaries of buf2
	minx = MINTWO(MAXTWO(minx,0), buf2->width()-1);
	miny = MINTWO(MAXTWO(miny,0), buf2->height()-1);
	maxx = MINTWO(MAXTWO(maxx,0), buf2->width()-1);
	maxy = MINTWO(MAXTWO(maxy,0), buf2->height()-1);

	// Morphing (loop over BB in buf2)
	HMatrix2D Hi(H.get_inverse());                                      // invert it once for all, most costy part of the loop
	for(int x2 = (int)minx; x2 < (int)(maxx+1); x2++) {
		vcl_cout << ".";
		for(int y2 = (int)miny; y2 < (int)(maxy+1); y2++) {
			// Point in buf1
			HomgPoint2D P( Hi * HomgPoint2D(x2,y2) );
			double x1, y1;
			P.get_nonhomogeneous(x1, y1);

			// Check if in buf1
			if(x1 >= 0 && y1 >= 0 && x1 < buf1->width()-1 && y1 < buf1->height()-1) {
				// Check if in mask (ie: not black)
				if(!((*maskbuf)((int)round(x1), (int)round(y1)) == vil1_rgb<unsigned char>(0, 0, 0) )) {

					// Paint pixel
					// Subpixel interp missing (colorversion !)
					// Alpha-blending (nice idea of Raymundo)
					vil1_rgb<unsigned char> col = (*buf1)((int)round(x1), (int)round(y1));
					(*buf2)(x2, y2) = col;
				} // in mask
			} // in buf1
		} // loop over y2
	} // loop over x2

	vcl_cout << vcl_endl;
} // homography_morph

