#include "rescaling.h"

#include <vcl_cmath.h>

#include <misc_utility/defines.h>
#include <misc_utility/gaussian_smoothing.h>


// very straightforward implementation for rescaling a buffer
// to be used only for downscaling (no interpolation), 
// and with a fixed amount of smoothing
color_buffer_t* rescale_buffer(color_buffer_t* buf, 
			       float scalefactor)
{
  int sizex = buf->cols();
  int sizey = buf->rows();
  int nsizex = (int)(sizex * scalefactor);
  int nsizey = (int)(sizey * scalefactor);

  // first smooth large scale buffer
  //color_buffer_t* tmp_buf = rgb_gaussian_smoothing(buf, 4.0);
  color_buffer_t tmp_buf(sizex, sizey);
  vic_gaussian_smoothing(*buf, tmp_buf);

  // then rescale buffer
  color_buffer_t* new_buf = new color_buffer_t(nsizex, nsizey);
  for (int i=0; i < nsizex; i++)
    for (int j=0; j < nsizey; j++)
    {
      (*new_buf)(i,j) = tmp_buf((int)round(i/scalefactor), (int)round(j/scalefactor));
    }

  return new_buf;
}


// idem for greyscale buffers
bw_buffer_t* rescale_buffer(bw_buffer_t* buf, 
			    float scalefactor)
{
  int sizex = buf->cols();
  int sizey = buf->rows();
  int nsizex = (int)(sizex * scalefactor);
  int nsizey = (int)(sizey * scalefactor);

  // first smooth large scale buffer
  //bw_buffer_t* tmp_buf = new bw_buffer_t (vil1_smooth_gaussian(*buf, 4.0));
  bw_buffer_t tmp_buf(sizex, sizey);
  vic_gaussian_smoothing(*buf, tmp_buf);

  // then rescale buffer
  bw_buffer_t* new_buf = new bw_buffer_t(nsizex, nsizey);
  for (int i=0; i < nsizex; i++)
    for (int j=0; j < nsizey; j++)
    {
      (*new_buf)(i,j) = tmp_buf((int)round(i/scalefactor), (int)round(j/scalefactor));
    }

  return new_buf;
}


