#include "vic_math.h"

#include <misc_utility/defines.h>

// index of max element in p (6-vector of float)
int max6(float* p) {
  int maxi = 0;
  float maxv = p[0];
  for(int i = 1; i < 6; i++) {
    if(p[i] > maxv) {
      maxi = i;
      maxv = p[i];
    }
  }
  
  return maxi;
}


// point-to-point distance
float pdist(point_t* p1, point_t* p2) {
  return sqrt( (p2->x()-p1->x())*(p2->x()-p1->x()) + (p2->y()-p1->y())*(p2->y()-p1->y()) );
}


// generate a new vector, with same values as v (but in float precision)
vnl_vector<float>* cast_to_vector_float(const vnl_vector<double>* v) {
  vnl_vector<float>* o = new vnl_vector<float>(v->size());
  for(int i=0; i < v->size(); i++) {
    (*o)[i] = (*v)[i];
  }
  return o;
}


// counter-clockwise angle of the vector o->p wrt the
// from left-to-right orizontal axis (in radians). o is the origin.
// result always in [0, 2*Pi]
float angle(point_t* o, point_t* p) {
  float x = p->x() - o->x();
  float y = o->y() - p->y();               // trigonometric coord sys has y-axis inverted wrt image coord sys
  float pi = 3.141592;
  
  if(x == 0 && y == 0) {
    return 0.0;
  }

  if(x == 0 && y != 0) {
    return pi/2.0;
  }

  float t = atan(y/x);
  if(x > 0 && y>= 0) {
    return t;
  }
  if(x < 0 && y >= 0) {
    return t + pi;
  }
  if(x > 0 && y <= 0) {
    return t + 2*pi;
  }
  if(x < 0 && y < 0) {
    return t + pi;
  }
}


