// ----------------------------------------------------------------------
// 
// CurvedregionExtractor
//
// Extraction of curved affinely invariant regions
//
// Author: Tinne Tuytelaars
//         ESAT Leuven
//
// ----------------------------------------------------------------------

#ifndef _curvedregionextractor_h_
#define _curvedregionextractor_h_

#include <regions/parall_region.h>
#include <region_extraction/parall_region_extractor.h>

#include <vgl/vgl_homg_point_2d.h>
#include <vnl/vnl_vector.h>
#include <vnl/vnl_matrix.h>

//-----------------------------------------------------------------------------
//-- NURBS spline fitting
//-----------------------------------------------------------------------------
#include <nurbs.h>



typedef vcl_vector<vdgl_digital_curve*> edge_vector_t;
typedef vcl_vector<PLib::NurbsCurvef> spline_vector_t;



class curved_region_extractor : public parall_region_extractor
{
public:
  curved_region_extractor(reg_ext_starter *starter);
  virtual ~curved_region_extractor();

  //  Here are the region extraction functions that are purely virtual in paralparalll_region_extractor.
  bool extract_regions(point_t* pt,
                      region::region_specific_type tp = region::PARALL_CURVED);
  
  bool extract_all_regions();

  //  Computations
//  bool ComputeInvariants1D(point_t *ctr, point_t *p1, 
  //                         point_t *p2, vnl_vector<double> &result);
  
  bool compute_invariants_1d_rgb(const point_t *ctr,
                                 const point_t *p1, 
                                 const point_t *p2,
                                 vnl_vector<double> &result);

  
  bool ComputeInvariants1D_ZW(point_t *ctr, point_t *p1, 
                              point_t *p2, vnl_vector<double> &result);

  vcl_vector<int> compute_extrema_1d(vnl_vector<double> vec, int max_index) const;
  
  vcl_vector<int> compute_scale_1d(const vnl_matrix<double> &M, int max_index,
                                 int NB_INV, vcl_vector<int> &temp_types) const;

protected:
  double min_dist_param(const PLib::NurbsCurvef&, const point_t*) const;
  vcl_vector<parall_region*> span_regions(spline_vector_t&, const point_t*);
  void geometry_check(vcl_vector<parall_region*>&) const;
  
  
  

private:
  int _min_spline_length, _max_canonic_arclength;
  int _canonic_arclength_step;
  
  double _normalized_arclength1_step, _normalized_arclength2_step;
  int _nr_of_invariants;
  
  vcl_list<double> *_Splines;  // This needs to be changed once the splines are ported

};

#endif // _curvedregionextractor_h_
