#ifndef _ell_region_extractorh_
#define _ell_region_extractorh_


//------------------------------------------------------------------------------
// .NAME EllregionExtractor - Class for extracting 
//       affinely invariant regions in an image using the intensity-based method
// 
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   EllregionExtractor.h
// .FILE   EllregionExtractor.C
//
// .SECTION Description
//    The class EllregionExtractor extracts affinely invariant region(s) from an image
//    using the intensity-based method (cfr Tuytelaars and Van Gool, BMVC2000),
//    returning elliptical regions around intensity extrema. 
//     
//  
// .SECTION Author:
//          Tinne Tuytelaars, Andreas Turina, Vittorio Ferrari
//          ESAT / KULeuven          BIWI, ETH
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

/*******************************************************************************
*
*	Ported to vxl by Aaron Luttman
*	Notes:	This class owns the following memory: _ec, _dc, _mombuf.  These
*	are all utility members, though, so they are newed and deleted a lot by
*	functions other than the constructors/destructors.  This does not own _anchor.
*
*******************************************************************************/

#include <regions/ell_region.h>
#include <region_extraction/reg_ext_starter.h>
#include <region_extraction/reg_ext_params.h>
#include <region_extraction/region_extractor.h>
#include <misc_utility/byte_interpolation.h>

#include <vsol/vsol_conic_2d.h>
#include <osl/osl_edgel_chain.h>
#include <vdgl/vdgl_digital_curve.h>


class ell_region_extractor : public region_extractor
{
  // PUBLIC INTERFACE----------------------------------------------------------
public:
  // Constructors/Destructors--------------------------------------------------
  ell_region_extractor(reg_ext_starter* starter);
  virtual ~ell_region_extractor();
	
  // Utility Methods-----------------------------------------------------------
  virtual bool extract_regions(point_t* pt, 
                               region::region_specific_type tp = region::REGION_NO_SPECIFIC_TYPE);
  virtual bool extract_all_regions();
	
protected:
  // Helper functions ---------------------------------------------------------
  bool find_points_on_rays();
  bool link_points();
  bool compute_ellipse(); 
  bool find_orientation_aoi(ell_region *Reg, vil1_image &Image);
	
private:
  point_t *_anchor;
  osl_edgel_chain *_ec;
  vdgl_digital_curve *_dc;
  vil1_memory_image_of<unsigned char> *_mombuf;
  int _sx, _sy;
  int _minx, _miny, _maxx, _maxy;
};

#endif // _ell_region_extractorh_
