// ----------------------------------------------------------------------
//
// class homogeneous_region_extractor
//
// Extraction of homogeneous, parallelogram-shaped affinely invariant
// regions.
//
// Author: Andreas Turina
//         ETH Zrich
//         
// ----------------------------------------------------------------------

#ifndef _homogeneous_region_extractor_h_
#define _homogeneous_region_extractor_h_

#include <misc_utility/defines.h>
#include <region_extraction/parall_region_extractor.h>




class homogeneous_region_extractor : public parall_region_extractor  
{
public:
  homogeneous_region_extractor(reg_ext_starter *starter);
  virtual ~homogeneous_region_extractor();

  virtual bool extract_regions(point_t* pt, 
                              region::region_specific_type tp = region::REGION_NO_SPECIFIC_TYPE);
  virtual bool extract_all_regions();


  void print_statistics() const;
  

  // INTERNALS-----------------------------------------------------------------
protected:

  points_list_t find_rgb_scales(vcl_list<int>& temp_types);
  
  bool compute_gradient_buffer(vil1_memory_image_of<float>&,
                               const vil1_memory_image_of<vxl_byte>&) const;

  points_list_t compare_grad_buffers(vil1_memory_image_of<float>& gradR,
				     vil1_memory_image_of<float>& gradG,
				     vil1_memory_image_of<float>& gradB,
				     const vil1_memory_image_of<vxl_byte>& R,
				     const vil1_memory_image_of<vxl_byte>& G,
				     const vil1_memory_image_of<vxl_byte>& B);
  
  vcl_vector<float> filter_grad_scales(vcl_vector<int*>&, const vil1_memory_image_of<vxl_byte>&);


  vcl_vector<parall_region*> maximize_regions();
  
  vcl_vector<parall_region*> 
    extract_candidates(const vcl_vector<parall_region*>& );
  

  
private:
  vil1_memory_image_of<vil1_rgb_byte> *ds_buf;
  vil1_memory_image_of<vdgl_digital_curve*> *ds_line_buf;

  vcl_vector<unsigned int> _extraction_statistics;

};


#endif // !defined _homogeneous_region_extractor_h_
