
#include <cstdlib>
#include <cstdio>

#include <regions/parall_region.h>
#include <regions/ell_region.h>
#include <region_utility/VIBES_region_file.h>
#include <misc_utility/executor.h>

#include "multi_region_extractor.h"
#include "tuytelaars_region_extractor.h"


MultiRegionExtractor::MultiRegionExtractor( const vil1_image *image,
                                            const char *fName,
                                            const bw_buffer_t *theMap ) :
	theImage(image),
	mapBuff(theMap)
{
	verbose = false;
	
	if( fName == NULL )
		sFileName = "";
	else
		sFileName = fName;
}

MultiRegionExtractor::~MultiRegionExtractor()
{
}


int MultiRegionExtractor::extractRegions( regions_list_t &rs,
                                          const vcl_string &type, bool filter ) {
	if( type == "mser" || type == "h_affine" ) {
		if( sFileName == "" ) {
			vcl_cerr << "Error: regions type '" << type << "' requires image file on disk" << vcl_endl;
			return RET_ERROR;
		}
	}

	char tempName[] = "/tmp/fileXXXXXX";
	bool tuytFilter = false;
	if( filter && ! mapBuff ) // if filtering is requested but there's no segmentation,
		tuytFilter = true;      // we use the built-in background filter for tuytelaars

	if( type == "tuytelaars_ell" ) {
		TuyteRegionExtractor xtractor( theImage, tuytFilter, verbose );
		xtractor.extractEllRegions( rs );
	}
	else if( type == "tuytelaars_parall" ) {
		TuyteRegionExtractor xtractor( theImage, tuytFilter, verbose );
		xtractor.extractParRegions( rs );
	}
	else if( type == "mser" ) { // matas
		int fd = mkstemp(tempName);
		vcl_string cmd("mser.ln -t 2 -i ");
		cmd += sFileName + vcl_string(" -o ") + tempName;
		close(fd);

		Executor exec(verbose);
		if( exec.execute(cmd) != 0 ) {
			remove(tempName);
			return RET_EXEC_FAIL;
		}

		VIBES_region_file brf;
		brf.load_regions( rs, tempName, NULL );
		remove(tempName);
	}
	else if( type == "h_affine" ) { // krystian
		int fd = mkstemp(tempName);
		vcl_string cmd("h_affine.ln -hesaff -i ");
		cmd += sFileName + vcl_string(" -o ") + tempName;
		close(fd);

		Executor exec(verbose);
		if( exec.execute(cmd) != 0 ) {
			remove(tempName);
			return RET_EXEC_FAIL;
		}

		VIBES_region_file brf;
		brf.load_regions( rs, tempName, NULL );
		remove(tempName);
	}
	else {
		vcl_cerr << "ERROR: unknown feature type '" << type << "'" << vcl_endl;
		return RET_WRONG_ARG;
	}

	if( filter && mapBuff ) {
		regions_list_t::iterator iter = rs.begin();
		int nFiltered = 0;
		while( iter != rs.end() ) {
			region *r = *iter;
			point_t *p0 = r->get_pt0();
			if( (*mapBuff)((int)p0->x(), (int)p0->y()) > 127 )
				iter++;
			else {
				delete r;
				iter = rs.erase(iter);
				nFiltered++;
			}
		}
		if(verbose)
			vcl_cout << "Filtered " << nFiltered << " regions" << vcl_endl;
	}

	return 0;
}
