#ifndef MULTI_REG_EXT_H
#define MULTI_REG_EXT_H


#include <vcl/vcl_list.h>
#include <vcl/vcl_utility.h>
#include <vcl/vcl_iostream.h>
#include <vcl/vcl_string.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_load.h>

#include <misc_utility/defines.h>


class MultiRegionExtractor
{
public:
	/* Create a new Extractor for an image and an optional segmentation mask.
	 * If you want to use external binaries, you must provide a file name,
	 * the file must be in a format supported by the binary (like PNG, PGM, PPM) */
	MultiRegionExtractor( const vil1_image *image, const char *fName=NULL,
	                      const bw_buffer_t *theMap=NULL );
	
	~MultiRegionExtractor();

	/* Extract regions.
	 * + optionally filter according to segmentation mask
	 * Supported region types:
	 *   tuytelaars_ell, tuytelaars_parall, mser, h_affine
	 * Return value is 0 upon success. */
	int extractRegions( regions_list_t &rs, const vcl_string &type, bool filter=true );

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	/* Data members */
	const vil1_image  *theImage;
	vcl_string sFileName;
	const bw_buffer_t *mapBuff;
	vil1_rgb<unsigned char> bcol;

	bool verbose;
};


#endif //MULTI_REG_EXT_H
