#ifndef _reg_ext_params_h
#define _reg_ext_params_h

//----------------------------------------------------------------------------
// .NAME  - reg_ext_params Class for storing region extraction relevant parameters
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   reg_ext_params.h
// .FILE   reg_ext_params.C
//
// .SECTION Description
//    The class reg_ext_params is a class for
//  storing various parameters for the extraction of (affinely invariant) regions.
// 
//  At design time, it is assumed that all region extraction processes
//  use the same parameters values for region extraction. The class is
//  therefore implemented as a singleton.
//
// .SECTION Author:
//          Tinne Tuytelaars - Andrew Turina - Vittorio Ferrari
//          ESAT / KULeuven           BIWI / ETH Zurich
//
// .SECTION Modifications:
//     
//
//----------------------------------------------------------------------------
#include <osl/osl_harris.h>

class reg_ext_params 
{
public:
  reg_ext_params();
  ~reg_ext_params();
	
  int max_size;          // Maximum region size
  int min_size;          // Minimum region size
  int max_dist;          // Max. distance to look for intersections
  int min_length;        // Minimum length of edge to be retained
	
  // Canny parameters
  int   follow_strategy_CannyOX;
  int   edge_min_CannyOX;     
  int   min_length_CannyOX;  
  int   border_size_CannyOX;
  int   max_width_CannyOX;
  float sigma_CannyOX;
  float gauss_tail_CannyOX;
  float low_CannyOX;
  float high_CannyOX;
  float border_value_CannyOX;
  float scale_CannyOX;
  bool join_flag_CannyOX;
  bool junction_option_CannyOX;
	
  // Straight line fitting parameters
  int    min_fit_length_FitLines; 
  double threshold_FitLines;
  double theta_FitLines;
	
  // Harris parameters
  osl_harris_params params_Harris;
  inline osl_harris_params *GetHarrisParams() { return &params_Harris; }
	
  // Intensity extrema parameters
  int   size_IntExtrema;
  float sigma_IntExtrema;
	
  // Ell region extraction parameters
  int min_size_EllRegExt;
  int max_size_EllRegExt;
  int number_of_rays_EllRegExt;
  float resize_factor_EllRegExt;
	
  // Parall region extraction parameters
  int MaxSideLength;
  int MinSideLength;
  double MaxDistanceFromPointToEdge;
	
};



#endif // end of class reg_ext_params
