#ifndef _region_extractorh_
#define _region_extractorh_


//------------------------------------------------------------------------------
// .NAME region_extractor - Abstract base class for extracting 
//       affinely invariant regions in an image
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   region_extractor.h
// .FILE   region_extractor.C
//
// .SECTION Description
//    The class region_extractor extracts affinely invariant region(s) from an image. 
//     
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//			Port to vxl by Aaron Luttman, September 19, 2001
//
//
//----------------------------------------------------------------------------

/*******************************************************************************
*
*	Ported to vxl by Aaron Luttman
*	Notes:	This class does not own any dynamic memory.  That is a bit of a trick
*	though, because it has a statically allocated list of dynamically allocated
*	regions.  Thus it must deallocate the regions that are in the list.
*
*******************************************************************************/

#include <regions/region.h>
#include <region_extraction/reg_ext_params.h>
#include <region_extraction/reg_ext_starter.h>

#include <vcl/vcl_vector.h>
#include <vil1/vil1_memory_image_of.h>
#include <vgl/vgl_homg_point_2d.h>

typedef vcl_vector<region*> regions_list_t; 

class region_extractor
{
  // PUBLIC INTERFACE----------------------------------------------------------
public:
  // Constructors/Destructors--------------------------------------------------
  region_extractor(reg_ext_starter* starter);  
  virtual ~region_extractor();  
  
  // Utility Methods-----------------------------------------------------------
  virtual bool extract_regions(point_t* pt, 
                               region::region_specific_type tp = region::REGION_NO_SPECIFIC_TYPE) = 0;
  virtual bool extract_all_regions() = 0;
  void set_show(bool show) {_show = show;}
  
  // Data Access---------------------------------------------------------------
  virtual inline bool get_regions(regions_list_t& regs);
  virtual inline bool clear_regions();
  
protected:
  regions_list_t _regions; // list of regions found so far
  reg_ext_params _params; 
  reg_ext_starter* _starter; // starter takes care of preprocessing
  bool _show;
};




bool region_extractor::get_regions(regions_list_t& regs)
{
  for (regions_list_t::iterator it = _regions.begin(); it != _regions.end(); ++it)
  {
    regs.push_back(*it);
  }
	
  return true;
}

bool region_extractor::clear_regions()
{
  while( !_regions.empty() )
  {
    _regions.erase(_regions.begin());
  }
	
  return true;
}

#endif // _region_extractorh_
