
#include <cstdio>
#include <regions/region.h>   // required to delete regions
#include <region_utility/miko1_region_file.h>
#include <misc_utility/executor.h>

#include "sift_descriptor.h"


SiftDescriptor::SiftDescriptor( const vil1_image *image,
                                const char *fName, bool bVerbose ) :
	theImage(image)
{
	verbose = bVerbose;
	
	if( fName == NULL ) {
		sFileName = "";
		vcl_cerr << "ERROR: sift descriptor currently requires external file!" << vcl_endl;
	}
	else
		sFileName = fName;
}

SiftDescriptor::~SiftDescriptor()
{
	
}


int SiftDescriptor::get_descriptors( regions_list_t &regions )
{
	if( sFileName == "" )
		return RET_ERROR;

	char tempName1[] = "/tmp/fileXXXXXX";
	char tempName2[] = "/tmp/fileXXXXXX";
	int fd1 = mkstemp(tempName1);
	int fd2 = mkstemp(tempName2);

	Miko1_region_file brf;
	close(fd1);
	brf.save_regions(regions, tempName1);

	Executor exec(verbose);
	vcl_string cmd("compute_descriptors.ln -sift -i ");
	cmd += sFileName + vcl_string(" -p1 ") + tempName1 + vcl_string(" -o1 ") + tempName2;
	close(fd2);

	if( exec.execute(cmd) != 0 ) {
		remove(tempName1);
		remove(tempName2);
		return RET_EXEC_FAIL;
	}
	
	/* Purge old regions and load from file */
	for( unsigned int i=0; i<regions.size(); i++ )
		delete regions[i];
	regions.clear();
	brf.load_regions(regions, tempName2, NULL);

	remove(tempName1);
	remove(tempName2);
	return 0;
}
