#ifndef SIFT_DESCRIPTOR_H
#define SIFT_DESCRIPTOR_H


#include <vcl/vcl_list.h>
#include <vcl/vcl_utility.h>
#include <vcl/vcl_iostream.h>
#include <vcl/vcl_string.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_load.h>

#include <misc_utility/defines.h>


class SiftDescriptor
{
public:
	/* Create a new Sift descriptor calculator for an image.
	 * Currently, it uses an external binary, so only a file name is required.
	 * The file must be in pgm, ppm or png format.
	 * But if someone ever plugs in some source code here, an image could be
	 * used directly. */
	SiftDescriptor( const vil1_image *image, const char *fName=NULL, bool bVerbose=false );
	
	~SiftDescriptor();

	/* Calculate SIFT descriptors for a set of regions.
	 * The descriptors will replace any existing descriptors!
	 * Returns 0 upon success. */
	int get_descriptors( regions_list_t &regions );

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	/* Data members */
	const vil1_image  *theImage;
	vcl_string sFileName;

	bool verbose;
};


#endif //SIFT_DESCRIPTOR_H
