//////////////////////////////////////////////////////////////////////
//
// StraightregionExtractor.h: interface for the StraightregionExtractor class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _STRAIGHTREGIONEXTRACTOR_
#define _STRAIGHTREGIONEXTRACTOR_

#include <region_extraction/parall_region_extractor.h>




class straight_region_extractor : public parall_region_extractor  
{
public:
  straight_region_extractor(reg_ext_starter *starter);
  virtual ~straight_region_extractor();

  virtual bool extract_regions(point_t* pt, 
                              region::region_specific_type tp = region::REGION_NO_SPECIFIC_TYPE);
  virtual bool extract_all_regions();

  // INTERNALS-----------------------------------------------------------------
protected:
  

  /* Computations for invariants */
  double get_interpolation(vil1_memory_image_of<float>& buf, double x, double y);
  

  bool compute_I_buffer(vil1_memory_image_of<float>& I_buf, vil1_memory_image_of<byte>& buf);
  bool compute_II_buffer(vil1_memory_image_of<float>& II_buf, vil1_memory_image_of<byte>& buf);
  bool compute_Ix_buffer(vil1_memory_image_of<float>& Ix_buf, vil1_memory_image_of<byte>& buf);
  bool compute_Iy_buffer(vil1_memory_image_of<float>& Iy_buf, vil1_memory_image_of<byte>& buf);
  bool compute_cog1_buffer(vil1_memory_image_of<float>& cog_buf, vil1_memory_image_of<byte>& buf);
  bool compute_cog1_buffer(vil1_memory_image_of<float>& cog_buf, vil1_memory_image_of<float>& I_buf, 
                         vil1_memory_image_of<float>& II_buf, vil1_memory_image_of<float>& Ix_buf, 
                         vil1_memory_image_of<float>& Iy_buf);
  bool compute_cog2_buffer(vil1_memory_image_of<float>& cog_buf, vil1_memory_image_of<byte>& buf);
  bool compute_cog2_buffer(vil1_memory_image_of<float>& cog_buf, vil1_memory_image_of<float>& I_buf,  
                         vil1_memory_image_of<float>& II_buf, vil1_memory_image_of<float>& Ix_buf, 
                         vil1_memory_image_of<float>& Iy_buf);
  bool compute_cog123_buffers(vil1_memory_image_of<float>& cog1_buf, vil1_memory_image_of<float>& cog2_buf, 
                            vil1_memory_image_of<float>& cog3_buf, vil1_memory_image_of<byte>& buf);
  bool compute_cog123_buffers(vil1_memory_image_of<float>& cog1_buf, vil1_memory_image_of<float>& cog2_buf, 
                            vil1_memory_image_of<float>& cog3_buf, vil1_memory_image_of<float>& I_buf, 
                            vil1_memory_image_of<float>& II_buf, vil1_memory_image_of<float>& Ix_buf, 
                            vil1_memory_image_of<float>& Iy_buf);
  points_list_t compare_cog_buffers(vil1_memory_image_of<float>& cog1_buf, 
                                                            vil1_memory_image_of<float>& cog2_buf, double thresh);

  inline points_list_t find_2d_scales(vcl_list<int>& temp_types,
				      bool SHOW = false);
  
 

  points_list_t find_2d_rgb_scales(vcl_list<int>& temp_types, bool SHOW);
  points_list_t find_2d_zw_scales(vcl_list<int>& temp_types, bool SHOW);
  void filter_2d_scales(points_list_t& scales);

private:
  vil1_memory_image_of<vil1_rgb_byte> *ds_buf;

};




points_list_t
straight_region_extractor::find_2d_scales(vcl_list<int>& temp_types,
                                      bool SHOW)
{
  if ( (_ColorImage == true) )
  {
    return find_2d_rgb_scales(temp_types, SHOW);
  }
  
  return find_2d_zw_scales(temp_types, SHOW);
}


#endif // !defined _STRAIGHTREGIONEXTRACTOR_
