
#include <regions/parall_region.h>
#include <regions/ell_region.h>
#include <region_utility/region_draw.h>         // color_share
#include <misc_utility/count_pixels.h>          // border_color
#include <misc_utility/gaussian_smoothing.h>    // vic_gaussian_smoothing
#include <region_utility/misc.h>                // greylevel_variance
#include <vil1/vil1_image_as.h>                 // greylevel conversion

#include <cstdlib>
#include "tesselator.h"


Tesselator::Tesselator( const vil1_image &image, const bw_buffer_t *theMap, bool bVerbose ) :
	mapBuff(theMap)
{
	verbose = bVerbose;

	w = image.width();
	h = image.height();
	buf = new color_buffer_t(image);
	
	// Smooth image in the 3 color bands
	if(verbose)
		vcl_cout << "Smoothing image..." << vcl_endl;
	buf_smooth = new color_buffer_t(image);
	vic_gaussian_smoothing(*buf, *buf_smooth);

	// Prepare greylevel buffer for variance computation
	buf_bw = new bw_buffer_t(vil1_image_as_byte(*buf));

	if( mapBuff == NULL ) {
		// estimate background color
		bcol = border_color(buf_smooth);
		if(verbose)
			vcl_cout << "Border color: " << (int)bcol.R() << " " << (int)bcol.G() << " " << (int)bcol.B() << vcl_endl;
		if(abs(bcol.R() - bcol.G()) > 15 || abs(bcol.B() - bcol.G()) > 15) {
			if(verbose)
				vcl_cout << "WARNING: Colored background, segmentation assumes it black." << vcl_endl;
			bcol = vil1_rgb<unsigned char>(0, 0, 0);
		}
		else {
			if(bcol.R() <= 100) {
				if(verbose)
					vcl_cout << "Black background detected" << vcl_endl;
				bcol = vil1_rgb<unsigned char>(0, 0, 0);
			}
			else {
				if(verbose)
					vcl_cout << "White background detected" << vcl_endl;
				bcol = vil1_rgb<unsigned char>(70, 70, 70);
			}
		}
	}
}


Tesselator::~Tesselator()
{
	delete buf;
	delete buf_smooth;
	delete buf_bw;
}


int Tesselator::tesselate( regions_list_t &rs, float rw, float rh, float xs, float ys, int first_sid )
{
	// Tesselation
	if(verbose)
		vcl_cout << "Tesselating with rX=" << rw << ", rY=" << rh << ", dX=" << xs << ", dY=" << ys << vcl_endl;
	
	int sid = first_sid;
	
	for(int rx=0; rx<(int)((w-rw)/xs)+1; rx++) {
		for(int ry=0; ry<(int)((h-rh)/ys)+1; ry++) {
			float x0 = rx*xs;
			float y0 = ry*ys;
			//point_t* p0 = new point_t(x0, y0);
			//point_t* p1 = new point_t(x0+rw, y0);
			//point_t* p2 = new point_t(x0, y0+rh);
			point_t* p0 = new point_t(x0+rw/2.0, y0+rh/2.0);
			point_t* p1 = new point_t(x0+rw, y0+rh/2.0);
			point_t* p2 = new point_t(x0+rw/2.0, y0+rh);
			ell_region* r = new ell_region(p0, p1, p2);
			
			//vcl_cout << *(r->get_pt0()) << *(r->get_pt1()) << *(r->get_pt2()) << vcl_endl;
			// do not add background regions
			if( mapBuff ? (*mapBuff)((int)p0->x(), (int)p0->y()) > 127 :
			              color_share(r, bcol, buf) < 0.6 ) {   // stdr = 0.4
				// avoid shadows (and uniform regions in the body of the object) // stdr = 100
				if( greylevel_variance(r, buf_bw) > 100 ) {
					r->set_sid(sid++);
					rs.push_back(r);
				}
				else
					delete r;
			}
		}
	}

	return sid;
}

