#ifndef TESSELATOR_H
#define TESSELATOR_H


#include <vcl/vcl_list.h>
#include <vcl/vcl_utility.h>
#include <vcl/vcl_iostream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_load.h>

#include <misc_utility/defines.h>


class Tesselator
{
public:
	/* Create a new Tesselator for an image and an optional segmentation mask. */
	Tesselator( const vil1_image &image, const bw_buffer_t *theMap = NULL, bool bVerbose=true );
	
	~Tesselator();

	/* Tesselate the image and append the result to rs.
	 * rw, rh are the width and height of the regions,
	 * xs, ys are the horizontal/vertical step size,
	 * and first_sid is the ID of the first region.
	 * Mind that when deleting rs, all regions in it must be deleted manually.
	 * The return value is 1 + the sid of the last region. */
	int tesselate( regions_list_t &rs, float rw, float rh, float xs, float ys, int first_sid=1 );

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	/* Data members */
	int w, h;
	color_buffer_t    *buf;
	color_buffer_t    *buf_smooth;
	bw_buffer_t       *buf_bw;
	const bw_buffer_t *mapBuff;
	vil1_rgb<unsigned char> bcol;

	bool verbose;
};


#endif //TESSELATOR_H
