#ifndef TUYTE_REG_EXT_H
#define TUYTE_REG_EXT_H


#include <vcl/vcl_list.h>
#include <vcl/vcl_utility.h>
#include <vcl/vcl_iostream.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>
#include <vil1/vil1_load.h>

#include <misc_utility/defines.h>


class TuyteRegionExtractor
{
public:
	/* Create a new Extractor for an image.
	 * Set noback to true to disable background filtering based on color. */
	TuyteRegionExtractor( const vil1_image *image, bool bNoback=false, bool bVerbose=true );
	
	~TuyteRegionExtractor();

	/* Extract elliptical regions. 
	 * Regions are appended to the list, remember that you need to delete
	 * each region manually before deleting the regions_list_t.  */
	void extractEllRegions( regions_list_t &regions );

	/* Extract parallelogram regions.
	 * Same comments as above. */
	void extractParRegions( regions_list_t &regions );

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

private:
	/* Data members */
	const vil1_image *theImage;
	color_buffer_t org_buf;
	vil1_rgb<unsigned char> bcol;
	bool noback;

	bool verbose;
};


#endif //TUYTE_REG_EXT_H
