#ifndef crosscorrelation_computer2h
#define crosscorrelation_computer2h

//------------------------------------------------------------------------------
// .NAME  crosscorrelation_computor - Class for computing the crosscorrelation
//                                    between two affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   crosscorrelation_computor.h
// .FILE   crosscorrelation_computor.cpp
//
// .SECTION Description
//   crosscorrelation_computor computes the crosscorrelation between two
//   affinely invariant regions, after aligning them by the appropriate 
//   affine transformation (assuming pt0, pt1 and pt2 correspond).
//   
// .SECTION Author:
//          Tinne Tuytelaars - Vittorio Ferrari
//          ESAT / KULeuven         ETHZ
//
// .SECTION Modifications:
//        - adapted to be derived from similarity_computor   (VF, Dec 2002)
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <region_matching/similarity_computor.h>

class crosscorrelation_computor : public similarity_computor {

 public:
  crosscorrelation_computor() {}
  crosscorrelation_computor(const color_buffer_t* buf1,
			    const color_buffer_t* buf2);
  crosscorrelation_computor(const bw_buffer_t* buf1,
			    const bw_buffer_t* buf2);
  crosscorrelation_computor(const color_buffer_t* buf1, 
			    const color_buffer_t* buf2, 
			    const region* reg1, const region* reg2);
  crosscorrelation_computor(const bw_buffer_t* buf1, 
			    const bw_buffer_t* buf2, 
			    const region* reg1, const region* reg2);
  ~crosscorrelation_computor();

 public:
  inline float get_crosscorrelation()           {return get_similarity();}           // backward compatibility
  inline float get_crosscorrelation_switched()  {return get_similarity_switched();}  // backward compatibility


private:
  //--- computations --------------------------------------------------
  bool compute_similarity();
  bool compute_similarity_switched();


  //--- helper functions -----------------------------------------------
  inline float get_interpolation_Vic(const bw_buffer_t* buf, float x, float y);


 private:

};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

inline float crosscorrelation_computor::get_interpolation_Vic(const bw_buffer_t* buf, 
							      float x, float y) {
  int xl = (int)x;
  int xh = xl+1;
  int yl = (int)y;
  int yh = yl+1;

  if(xl <= 0 || yl <= 0 || xh >= buf->width() || yh >= buf->height()) return 0;
	    
  return (float) (( (xh-x) * ((yh-y)*((*buf)(xl,yl)) + (y-yl)* ((*buf)(xl,yh))) +
		   (x-xl) * ((yh-y)*((*buf)(xh,yl)) + (y-yl)* ((*buf)(xh,yh))) )/(xh-xl)/(yh-yl));
}



#endif
