#include "drgb_computor.h"

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <vcl/vcl_cmath.h>
#include <vcl/vcl_vector.h>
#include <vcl/vcl_list.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_image_as.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <region_utility/misc.h>               // in_image


// -- Constructors / Destructors ------------------------------
drgb_computor::drgb_computor(const color_buffer_t* buf1,
			     const color_buffer_t* buf2)
  : similarity_computor(buf1, buf2)
{

}



drgb_computor::drgb_computor(const color_buffer_t* buf1, 
			     const color_buffer_t* buf2, 
			     const region* r1, const region* r2)
  : similarity_computor(buf1, buf2, r1, r2)
{

}



drgb_computor::~drgb_computor() {
  //vcl_cout << "In destructor of drgb_computor" << vcl_endl;

}


// -- Computations -------------------------------------------------------
bool drgb_computor::compute_similarity_switched() {
  vcl_cout << "drgb_computor::compute_similarity_switched() not yet implemented" << vcl_endl;
  return false;
}


bool drgb_computor::compute_similarity() {
  if(!in_image(_region1, _buf1->width(), _buf1->height()) || !in_image(_region2, _buf2->width(), _buf2->height())) {
    _sim = -1;
    _sim_valid = false;
    return false;
  }
  // copy&pasted from crosscorrelation_computor, maybe recycle also this code ?
  // Important: access _buf1, _buf2 fields and NOT _buffer1, _buffer2
  int nb_ = 0;
  float dist_ = 0;

  float cx1 = _region1->get_pt0()->x();
  float cy1 = _region1->get_pt0()->y();
  float cx2 = _region2->get_pt0()->x();
  float cy2 = _region2->get_pt0()->y();
  
  float x11 = _region1->get_pt1()->x();
  float y11 = _region1->get_pt1()->y();
  float x12 = _region1->get_pt2()->x();
  float y12 = _region1->get_pt2()->y();

  float x21 = _region2->get_pt1()->x();
  float y21 = _region2->get_pt1()->y();
  float x22 = _region2->get_pt2()->x();
  float y22 = _region2->get_pt2()->y();

  float incr1_x_a1, incr1_y_a1, incr1_x_a2, incr1_y_a2, xi1, yi1;
  float incr2_x_a1, incr2_y_a1, incr2_x_a2, incr2_y_a2, xi2, yi2;
  // float resol;
  int resl1 = _res -1;

  if (_region1->get_type() != region::ELL_REGIONTYPE) // NB_QUAD == 1
  {
    // Region 1
    xi1 = cx1; yi1 = cy1;
    incr1_x_a1 = (float)(x11 - cx1) / resl1; // incrementals for axe 1
    incr1_y_a1 = (float)(y11 - cy1) / resl1;

    incr1_x_a2 = (float)(x12 - cx1) / resl1; // incrementals for axe 2
    incr1_y_a2 = (float)(y12 - cy1) / resl1;

    // Region 2
    xi2 = cx2; yi2 = cy2;
    incr2_x_a1 = (float)(x21 - cx2) / resl1; // incrementals for axe 1
    incr2_y_a1 = (float)(y21 - cy2) / resl1;

    incr2_x_a2 = (float)(x22 - cx2) / resl1; // incrementals for axe 2
    incr2_y_a2 = (float)(y22 - cy2) / resl1;

    vil1_rgb_byte pix1, pix2;
    float cur1_x, cur1_y; // current position inside region 1
    float cur2_x, cur2_y; // current position inside region 2
    float xia1, yia1;     // x,y init axis for region 1
    
    for (int i=0; i<_res; i++) {
      xia1 = xi1 + i*incr1_x_a1; // cur_x|y now positioned to start of other axe
      yia1 = yi1 + i*incr1_y_a1;
      cur1_x = xia1;
      cur1_y = yia1;
      
      cur2_x = xi2 + i*incr2_x_a1;
      cur2_y = yi2 + i*incr2_y_a1;
      
      for (int j=0; j<_res; j++) {
	pix1 = get_col_interpolation_Vic(_buf1, cur1_x, cur1_y);
	pix2 = get_col_interpolation_Vic(_buf2, cur2_x, cur2_y); 
	
	nb_++;
	dist_ += dist_rgb_sf(pix1, pix2);               // scale factored distance
	
	// Position in region 1
	cur1_x += incr1_x_a2;
	cur1_y += incr1_y_a2;
	// Position in region 2
	cur2_x += incr2_x_a2;
	cur2_y += incr2_y_a2;
      }
    }
  }
  else { // NB_QUAD == 4
    // Region 1
    xi1 = cx1 - (x11 - cx1) - (x12 - cx1); // initial pt at the start of 4 quadrans
    yi1 = cy1 - (y11 - cy1) - (y12 - cy1);

    incr1_x_a1 = (float)(x11 - cx1)*2.0 / resl1; // incrementals for axe 1
    incr1_y_a1 = (float)(y11 - cy1)*2.0 / resl1;

    incr1_x_a2 = (float)(x12 - cx1)*2.0 / resl1; // incrementals for axe 2
    incr1_y_a2 = (float)(y12 - cy1)*2.0 / resl1;

    // Region 2
    xi2 = cx2 - (x21 - cx2) - (x22 - cx2); // initial pt at the start of 4 quadrans
    yi2 = cy2 - (y21 - cy2) - (y22 - cy2);

    incr2_x_a1 = (float)(x21 - cx2)*2.0 / resl1; // incrementals for axe 1
    incr2_y_a1 = (float)(y21 - cy2)*2.0 / resl1;

    incr2_x_a2 = (float)(x22 - cx2)*2.0 / resl1; // incrementals for axe 2
    incr2_y_a2 = (float)(y22 - cy2)*2.0 / resl1;

    vil1_rgb_byte pix1, pix2;
    float cur1_x, cur1_y; // current position inside region 1
    float cur2_x, cur2_y; // current position inside region 2
    float xia1, yia1;     // x,y init axis for region 1
    
    for (int i=0; i<_res; i++) {
      xia1 = xi1 + i*incr1_x_a1; // cur_x|y now positioned to start of other axe
      yia1 = yi1 + i*incr1_y_a1;
      cur1_x = xia1;
      cur1_y = yia1;
      
      cur2_x = xi2 + i*incr2_x_a1;
      cur2_y = yi2 + i*incr2_y_a1;

      for (int j=0; j<_res; j++) {
	if (_region2->inside_region(cur2_x, cur2_y)) { // sufficient to test one ellipse
	  pix1 = get_col_interpolation_Vic(_buf1, cur1_x, cur1_y); 
	  pix2 = get_col_interpolation_Vic(_buf2, cur2_x, cur2_y); 
	  
	  nb_++;
	  dist_ += dist_rgb_sf(pix1, pix2);
	}

	// Position in region 1
	cur1_x += incr1_x_a2;
	cur1_y += incr1_y_a2;
	// Position in region 2
	cur2_x += incr2_x_a2;
	cur2_y += incr2_y_a2;
      }
    }
  }

  if (nb_ == 0)
  {vcl_cout << "error in drgb_ns: nb_ = 0 " << vcl_endl;}
  
  dist_ /= nb_;
  _sim = (100.0-dist_)/100.0;
  _sim_valid = true;

  return true;
}


