#ifndef _drgb_computer2h
#define _drgb_computer2h

//------------------------------------------------------------------------------
// .NAME  drgb_computor - Class for computing the average euclidean distance in RGB space
//                        between two affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   drgb_computor.h
// .FILE   drgb_computor.cpp
//
// .SECTION Description
//   drgb_computor computes the average euclidean distance in RGB space 
//   between two affinely invariant regions, after aligning them by the
//   appropriate affine transformation (assuming pt0, pt1 and pt2 correspond).
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <region_matching/similarity_computor.h>

class drgb_computor : public similarity_computor {

 public:
  drgb_computor() {}
  drgb_computor(const color_buffer_t* buf1,
	        const color_buffer_t* buf2);
  drgb_computor(const color_buffer_t* buf1, 
			    const color_buffer_t* buf2, 
			    const region* reg1, const region* reg2);
  ~drgb_computor();

 public:
  inline float get_drgb()           {return get_similarity();}           // syntactic sugar
  inline float get_drgb_switched()  {return get_similarity_switched();}  // syntactic sugar

  //--- computations --------------------------------------------------
  bool compute_similarity();
  bool compute_similarity_switched();


private:

  //--- helper functions -----------------------------------------------
  inline float dist_rgb(vil1_rgb_byte& pix1, vil1_rgb_byte& pix2);
  virtual inline float dist_rgb_sf(vil1_rgb_byte& pix1, vil1_rgb_byte& pix2);

};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

inline float drgb_computor::dist_rgb(vil1_rgb_byte& pix1, vil1_rgb_byte& pix2) {
  float dr = pix1.R() - pix2.R();
  float dg = pix1.G() - pix2.G();
  float db = pix1.B() - pix2.B();

  return sqrt( dr*dr + dg*dg + db*db );
}


inline float drgb_computor::dist_rgb_sf(vil1_rgb_byte& pix1, vil1_rgb_byte& pix2) {
  float dr = pix1.R()*_sr1 - pix2.R();
  float dg = pix1.G()*_sg1 - pix2.G();
  float db = pix1.B()*_sb1 - pix2.B();

  return sqrt( dr*dr + dg*dg + db*db );
}

#endif
