#ifndef EUCL_SOFT_MATCH_H
#define EUCL_SOFT_MATCH_H

#include <vcl/vcl_vector.h>
#include <vnl/vnl_vector.h>
#include <misc_utility/defines.h>


/* Euclidean match struct and sort functions */
typedef struct EuclMatchSt {
  unsigned int index1, index2;
  float distance;
} EuclMatch;


bool cmpEuclMatchAsc( const EuclMatch &a, const EuclMatch &b )
{
	return a.distance < b.distance;
}
bool cmpEuclMatchDesc( const EuclMatch &a, const EuclMatch &b )
{
	return a.distance > b.distance;
}


class EuclideanSoftMatcher
{
public:
	/* Create a new EuclideanSoftMatcher with a certain matching threshold.
	 * For matching SIFT features, good values are between 40000 and 70000. */
	EuclideanSoftMatcher( double mThresh=70000.0, bool bVerbose=false );
	
	~EuclideanSoftMatcher();

	/* Find soft matches, i.e. keep all matches below the threshold.
	 * Return value is number of matches found. */
	unsigned int match_soft( vcl_vector<EuclMatch> &matches,
	                         const regions_list_t &regions1,
	                         const regions_list_t &regions2 );

	/* Find soft-3 matches, i.e. only keep the 3 best matches for each region.
	 * Return value is number of matches found. */
	unsigned int match_soft3( vcl_vector<EuclMatch> &matches,
	                          const regions_list_t &regions1,
	                          const regions_list_t &regions2 );

	/* Find hard matches, i.e. only keep the best match for each region.
	 * Return value is number of matches found. */
	unsigned int match_hard( vcl_vector<EuclMatch> &matches,
	                         const regions_list_t &regions1,
	                         const regions_list_t &regions2 );

	/* Set verbose output */
	void setThreshold( double mThresh )
	{ matchThresh = mThresh; };

	/* Set verbose output */
	void setVerbose( bool v )
	{ verbose = v; };

	/* Calculate squared Euclidean distance between two descriptors */
	inline double distSquared( const vnl_vector<double> *desc1,
	                           const vnl_vector<double> *desc2 );

private:
	double matchThresh;
	bool verbose;
};


#endif //EUCL_SOFT_MATCH_H
