#include "intgr_ncc_drgbsnorm.h"

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <vcl/vcl_cmath.h>
#include <vcl/vcl_vector.h>
#include <vcl/vcl_list.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_image_as.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <cstdlib>

#include <misc_utility/byte_interpolation.h>     // dist_rgb_sf


// -- Constructors / Destructors ------------------------------
intgr_ncc_drgbsnorm::intgr_ncc_drgbsnorm(const color_buffer_t* buf1,
					 const color_buffer_t* buf2)
  : integral_computor(buf1, buf2)
{
}


intgr_ncc_drgbsnorm::intgr_ncc_drgbsnorm(const color_buffer_t* buf1, 
					 const color_buffer_t* buf2, 
					 const region* r1, const region* r2)
  : integral_computor(buf1, buf2, r1, r2)
{
}


intgr_ncc_drgbsnorm::~intgr_ncc_drgbsnorm() {
}


bool intgr_ncc_drgbsnorm::compute_similarity() {
  // Precondition
  if(!_integral1 || !_integral2) {
    _sim = -2;
    _sim_valid = false;
    return false;
  }

  // Compute ncc and drgbsnorm simulateneously
  float avg1 = 0;
  float avg2 = 0;
  float cor1 = 0;
  float cor2 = 0;
  float cor12 = 0;
  int nb = _integral1->size();
  float dist = 0;
  for(int z=0; z<nb; z++) {
    // zone z color for the two regions
    vil1_rgb_byte pix1 = (*_integral1)[z];
    vil1_rgb_byte pix2 = (*_integral2)[z];
 
    // ncc
    float grey1 = pix1.R()*0.15 + pix1.G()*0.7 + pix1.B()*0.15;
    float grey2 = pix2.R()*0.15 + pix2.G()*0.7 + pix2.B()*0.15;
    avg1 += grey1;
    avg2 += grey2;
    cor1 += grey1*grey1;
    cor2 += grey2*grey2;
    cor12 += grey1*grey2;

    // drgbsnorm
    dist += dist_rgb_sf(pix1, _sf1, pix2, _sf2);
  }

  // ncc
  avg1 /= nb;
  avg2 /= nb;
  cor1 = cor1 - nb*avg1*avg1;
  cor2 = cor2 - nb*avg2*avg2;
  cor12 = cor12 - nb*avg1*avg2;
  cor12 /= sqrt(cor1 * cor2);
  if(!(cor12 >= -1 && cor12 <= 1)) {
    cor12 = -1;
  }

  // drgbsnorm
  dist /= nb;
  dist = (100.0-dist)/100.0;
  if(!(dist <= 1 && dist >= -1)) { // avoid erros and truncate too negative scores
    dist = -1;
  }

  // final
  _sim = cor12 + dist;
  _sim_valid = true;

  // test/debug
  //vcl_cout << "compute_similarity: " << vcl_endl;
  //vcl_cout << "_sf1 = " << _sf1 << " _sf2 = " << _sf2 << " ncc = " << cor12 << " drgbsnorm = " << dist << vcl_endl;
}


bool intgr_ncc_drgbsnorm::compute_similarity_switched() {
  vcl_cout << "intgr_ncc_drgbsnorm::compute_similarity_switched: ERROR: not implemented yet." << vcl_endl;
  exit(1);
}


