#ifndef _intgr_ncc_drgbsnorm
#define _intgr_ncc_drgbsnorm

//------------------------------------------------------------------------------
// .NAME  intgr_ncc_drgbsnorm - Class for computing integrated similarity
//                              between two affinely invariant regions.
//                              The similarity qeuals the ncc + drgbsnorm
//                              of the zones colors
//                            
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   intgr_ncc_drgbsnorm.h
// .FILE   intgr_ncc_drgbsnorm.cpp
//
// .SECTION Description
//
// .SECTION Author:
//          Vittorio Ferrari
//                ETHZ      
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>
#include <misc_utility/defines.h>
#include <regions/region.h>
#include <region_matching/integral_computor.h>

class intgr_ncc_drgbsnorm : public integral_computor {

 public:
  intgr_ncc_drgbsnorm() {}
  intgr_ncc_drgbsnorm(const color_buffer_t* buf1,
		      const color_buffer_t* buf2);
  intgr_ncc_drgbsnorm(const color_buffer_t* buf1, 
		      const color_buffer_t* buf2, 
		      const region* reg1, const region* reg2);
  virtual ~intgr_ncc_drgbsnorm();

 public:
  inline virtual void set_region1(const region* r1);
  inline virtual void set_region2(const region* r2);


protected:
  // Computations
  bool compute_similarity();
  bool compute_similarity_switched();

  inline float compute_integral_mean(vcl_vector<vil1_rgb_byte>* integral);

  //--- scalefactors ---------------------------------------------------
  float _sf1;
  float _sf2;

};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

inline void intgr_ncc_drgbsnorm::set_region1(const region* r1) {
  // DO NOT try to optimize here by computing means iff
  // r1!=_region1 -> pointer difference does not imply coordinates
  // difference ! Source of bug that costed hours !
  integral_computor::set_region1(r1);
  if(_integral1) _sf1 = 128.0 / compute_integral_mean(_integral1);  
}


inline void intgr_ncc_drgbsnorm::set_region2(const region* r2) {
  integral_computor::set_region2(r2);                
  if(_integral2) _sf2 = 128.0 / compute_integral_mean(_integral2);  
}


inline float intgr_ncc_drgbsnorm::compute_integral_mean(vcl_vector<vil1_rgb_byte>* integral) {
  float mean = 0.0;
  for(int z=0; z<integral->size(); z++) {
    vil1_rgb_byte pix = (*integral)[z];
    float grey = pix.R()*0.15 + pix.G()*0.7 + pix.B()*0.15;  // manual greylevelization ... coefficient a bit approx ...
    mean += grey;
  }
  return mean / integral->size();
}

#endif
