#ifndef _region_match_h_
#define _region_match_h_


//------------------------------------------------------------------------------
// .NAME matches - Abstract base class for representing a match
//    (either pairwise or not)  between affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   match.h
// .FILE   match.cpp
//
// .SECTION Description
//    The class match is an abstract base class for representing matches. 
//    This can be either pairwise matches or clusters, one-view, two-view or
//    multi-view matches, depending on the specific implementation.
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl/vcl_ostream.h>
#include <vcl/vcl_iostream.h>
#include <vbl/vbl_ref_count.h>

class pairwise_match;
class pairwise_region_match;
class pairwise_indexed_match;
class cluster;

class match : public vbl_ref_count
{
public:
  enum match_type 
    {
      MATCH_NO_TYPE=0,
      PAIRWISE_MATCH,
      PAIRWISE_INDEXED_MATCH,
      PAIRWISE_REGION_MATCH,
      CLUSTER,
      NB_OF_TYPES // last element in enum refers to total number of regiontypes
    };

  // PUBLIC INTERFACE----------------------------------------------------------
public:
  // Constructors/Destructors--------------------------------------------------
  match() {}; 
  virtual ~match() {};

  // Data access
  virtual inline match_type get_type() const { return MATCH_NO_TYPE; }

   //--- region I/O --------------------------------------------
  virtual void print(vcl_ostream& str) const {str << "Match " << vcl_endl; }

  //--- Safe Down Casting -------------------------------------
  virtual void* Cast() { return this; }
  virtual inline pairwise_match* cast_to_pairwise_match(){ return NULL; }
  virtual inline const pairwise_match* cast_to_pairwise_match() const { return NULL; }
  virtual inline pairwise_indexed_match* cast_to_pairwise_indexed_match(){ return NULL; }
  virtual inline const pairwise_indexed_match* cast_to_indexed_pairwise_match() const { return NULL; }
  virtual inline pairwise_region_match* cast_to_pairwise_region_match(){ return NULL; }
  virtual inline const pairwise_region_match* cast_to_pairwise_region_match() const { return NULL; }
  virtual inline cluster* cast_to_cluster() { return NULL; }
  virtual inline const cluster* cast_to_cluster() const { return NULL; }

  virtual bool operator==(match const &that) const {return true;}

protected:

};

#endif // _region_match_h_
