#ifndef _matchesh_
#define _matchesh_


//------------------------------------------------------------------------------
// .NAME matches - Abstract base class for all datastructures containing
//    matches between affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   matches.h
// .FILE   matches.cpp
//
// .SECTION Description
//    The class matches is an abstract class for storing matches. 
//    Different derived classes are foreseen, e.g. implementations 
//    based on a list of pairs of regions, based on Kd-trees (for indexing), etc. 
//    This should be sufficiently versatile to deal with both pairwise matches
//    as well as clusters, one-view, two-view and multiview matches, etc.
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <region_matching/match.h>
#include <regions/region.h>

class matches
{
  // PUBLIC INTERFACE----------------------------------------------------------
public:
  //--- Constructors/Destructors--------------------------------------------------
  matches() {}  
  virtual ~matches() {}
  
  //--- Data Access -----------------------------------------------------------
  virtual bool add(match* mtch) = 0;
  virtual bool remove(match* mtch) = 0;
  virtual int size() const = 0;

  //--- iterator-like operation to run over all matches / clusters
  virtual void reset() = 0;
  virtual bool next_match() = 0;
  virtual match* get_match() const = 0; // get current match
  virtual bool remove_match() = 0; // remove the current match

  virtual void clear() = 0; // clear all matches

protected:

};


#endif // _matchesh_
