#ifndef _index_matchesh_
#define _index_matchesh_


//------------------------------------------------------------------------------
// .NAME matches - Class for storing matches between affinely invariant regions
//       based on indices to lists of regions
//   
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   matches.h
// .FILE   matches.cpp
//
// .SECTION Description
//    The class index_matches is a derived class of the abstract class matches.
//    Matches are stored as imatches_list_t, which is a list of index_match_t,
//    i.e. a list of pairs of integers, where the integers refer to 
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <regions/region.h>
#include <vil1/vil1_image.h>
#include <vcl/vcl_vector.h>
#include <vcl/vcl_utility.h>

class matches
{
  // PUBLIC INTERFACE----------------------------------------------------------
public:
  // Constructors/Destructors--------------------------------------------------
  matches();  
  virtual ~matches();  
  
  // Data Access -----------------------------------------------------------

  virtual bool add(region_match_t mtch) = 0;
  virtual bool remove(region_match_t mtch) = 0;
  virtual bool add(index_match_t mtch) = 0;
  virtual bool remove(index_match_t mtch) = 0;
  virtual bool add(region_cluster_t clstr) = 0;
  virtual bool remove(region_cluster_t clstr) = 0;  
  virtual bool add(index_cluster_t clstr) = 0;
  virtual bool remove(index_cluster_t clstr) = 0;  

  // access matches based on index (pairwise matches)
  virtual region_match_t get_match(int indx) = 0;
  virtual index_match_t get_match(int indx) = 0;
  virtual region* get_orig_region(int indx);
  virtual region* get_ref_region(int indx);

  // iterator-like operation to run over all matches / clusters
  virtual bool reset() = 0;
  virtual bool next_match() = 0;
  virtual bool next_cluster() = 0;
  virtual bool next_match_between(vil1_image* im1, vil1_image* im2);
  virtual bool next_cluster_in(vil1_image* im);
  virtual bool next_match_with(region* orig_reg);
  virtual bool next_match_with(int orig_indx);
  virtual bool next_inverse_match_with(region* ref_reg);
  virtual bool next_inverse_match_with(int ref_indx);
  virtual region_match_t get_match() = 0; // get current pairwise match
  virtual index_match_t get_match() = 0; // get current pairwise match
  virtual region_cluster_t get_cluster() = 0; // get current cluster
  virtual index_cluster_t get_cluster() = 0; // get current cluster
  virtual bool remove_match() = 0; // remove the current match
  virtual bool remove_cluster() = 0; // remove the current cluster

  // get (first) matching region (if any)
  virtual region* get_matching_region_with(region* orig_reg);
  // get all matching regions (if any)
  virtual region_cluster_t get_matching_regions_with(region* orig_reg);
  virtual index_cluster_t get_matching_regions_with(region* orig_reg);
  virtual region_cluster_t get_inverse_matching_regions_with(region* orig_reg);
  virtual index_cluster_t get_inverse_matching_regions_with(region* orig_reg);

  // get all matching regions (if any) in image im
  virtual region_cluster_t get_matching_regions_with(region* orig_reg, vil1_image* im);
  virtual index_cluster_t get_matching_regions_with(region* orig_reg, vil1_image* im);
  virtual region_cluster_t get_inverse_matching_regions_with(region* orig_reg, vil1_image* im);
  virtual index_cluster_t get_inverse_matching_regions_with(region* orig_reg, vil1_image* im);

  // get all regions belonging to same cluster as reg (if any)
  inline virtual region_cluster_t get_cluster_with(region* reg);
  inline virtual index_cluster_t get_cluster_with(int indx);
 

protected:

};


region_cluster_t matches::get_cluster_with(region* reg)
{
  return get_matching_regions_with(reg);
}

index_cluster_t matches::get_cluster_with(int indx)
{
  return get_matching_regions_with(indx);
}


#endif // _matchesh_
