#ifndef _pairwise_matchesh_
#define _pairwise_matchesh_


//------------------------------------------------------------------------------
// .NAME matches - class for all datastructures containing
//    pairwise matches between affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   pairwise_matches.h
// .FILE   pairwise_matches.cpp
//
// .SECTION Description
//    The class pairwise_matches is a class for storing matches. 
//    Different derived classes are foreseen, e.g. implementations 
//    based on a vector of pairs of regions, based on Kd-trees (for indexing), etc. 
//    This should be sufficiently versatile to deal with both pairwise matches
//    as well as clusters, one-view, two-view and multiview matches, etc.
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <region_matching/matches.h>
#include <region_matching/match.h>
#include <region_matching/pairwise_match.h>
#include <regions/region.h>

class pairwise_matches : public matches
{
  // PUBLIC INTERFACE----------------------------------------------------------
public:
  // Constructors/Destructors--------------------------------------------------
  pairwise_matches();
  virtual ~pairwise_matches();

  //--- Data Access -----------------------------------------------------------

  //--- iterator-like operation to run over all matches / clusters
  virtual pairwise_match next_match_between(vil1_image* im1, vil1_image* im2); // pairwise matches
  virtual pairwise_match next_match_with(region* orig_reg);
  virtual pairwise_match next_inverse_match_with(region* ref_reg);
  
  virtual region* get_orig_region() const; // get orig. region of current match
  virtual region* get_ref_region() const; // get ref. region of current match

  virtual region* get_matching_region_with(region* orig_reg) const; // find (first) matching region with
  virtual region* get_inverse_matching_region_with(region* ref_reg) const; 


};


#endif // _matchesh_
