#ifndef _multi_region_matcherh_
#define _multi_region_matcherh_


//------------------------------------------------------------------------------
// .NAME multi_region_matcher - Class for matching 
//       two lists of affinely invariant regions based on 
//       Mahalanobis distance between invariants and 
//       a pixel-wise similarity measure.
//       The top few bidirectional matches per region are
//       returned.
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   multi_region_matcher.h
// .FILE   multi_region_matcher.C
//
// .SECTION Description
//    Find matches in two lists of affinely invariant regions based on 
//    Mahalanobis distance between invariants and 
//    a pixel-wise similarity measure. The top few bidirectional matches per
//    region are returned.
//     
//  
// .SECTION Author:
//          Vittorio Ferrari
//                ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <region_matching/classic_region_matcher.h>


class multi_region_matcher : public classic_region_matcher
{
  // PUBLIC INTERFACE----------------------------------------------------------
public:
  // Constructors/Destructors--------------------------------------------------
  multi_region_matcher(const vil1_image* im1,
		       const vil1_image* im2,
		       regions_list_t& reglist1, 
		       regions_list_t& reglist2,
		       similarity_computor* sc,
		       const reg_match_params* p);  
  ~multi_region_matcher();  
  
  // Utility Methods-----------------------------------------------------------
  bool match_region(region* reg1, region*& reg2, int& indx2); // match region from reglist1 (general-two way)


protected:
  // Helpers ------------------------------------------------------------------
  bool match_region_algo(region* reg1, regions_list_t& reglist);
  void find_best_sim_matches(region* reg1);
  void insert_multi_match(region* r, int indx, float sim);

  vcl_vector<region*> multi_regions;      // temp, working variables
  vcl_vector<float> multi_sims;           
  vcl_vector<int> multi_indices;

};


#endif // _multi_region_matcherh_

