#include "ncc_drgb_computor.h"

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_image_as.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>


// -- Constructors / Destructors ------------------------------
ncc_drgb_computor::ncc_drgb_computor(const color_buffer_t* buf1,
				     const color_buffer_t* buf2)
  : similarity_computor(buf1, buf2)
{
  _ncc = new crosscorrelation_computor(buf1, buf2);
  _drgb = new drgb_computor(buf1, buf2);
}



ncc_drgb_computor::ncc_drgb_computor(const color_buffer_t* buf1, 
				     const color_buffer_t* buf2, 
				     const region* r1, const region* r2)
  : similarity_computor(buf1, buf2, r1, r2)
{
  _ncc = new crosscorrelation_computor(buf1, buf2);
  _drgb = new drgb_computor(buf1, buf2);
}


ncc_drgb_computor::~ncc_drgb_computor() {
  //vcl_cout << "In destructor of drgb_computor" << vcl_endl;
  delete _ncc;
  delete _drgb;
}


void ncc_drgb_computor::set_scale_factors1(float* C) {
  _ncc->set_scale_factors1(C);
  _drgb->set_scale_factors1(C);
}


void ncc_drgb_computor::set_scale_factors2(float* C) {
  _ncc->set_scale_factors2(C);
  _drgb->set_scale_factors2(C);
}



// -- Computations -------------------------------------------------------
bool ncc_drgb_computor::compute_similarity_switched() {
  vcl_cout << "ncc_drgb_computor::compute_similarity_switched() not yet implemented" << vcl_endl;
  return false;
}


bool ncc_drgb_computor::compute_similarity() {
  _sim = _ncc->get_similarity(_region1, _region2);
  _sim += _drgb->get_similarity(_region1, _region2);
  _sim_valid = true;
  return true;
}
