#ifndef _ncc_drgb_computer2h
#define _ncc_drgb_computer2h

//------------------------------------------------------------------------------
// .NAME  ncc_drgb_computor - Class for computing a mixed similarity measure
//                            between two affinely invariant regions.
//                           
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   ncc_drgb_computor.h
// .FILE   ncc_drgb__computor.cpp
//
// .SECTION Description
//   ncc_drgb_computor computes the similarity 
//   between two affinely invariant regions, as the sum of outputs of a
//   crosscorrelation_computor and a drgb_computor.
//
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <region_matching/crosscorrelation_computor.h>
#include <region_matching/drgb_computor.h>

class ncc_drgb_computor : public similarity_computor {

 public:
  ncc_drgb_computor() {}
  ncc_drgb_computor(const color_buffer_t* buf1,
		    const color_buffer_t* buf2);
  ncc_drgb_computor(const color_buffer_t* buf1, 
		    const color_buffer_t* buf2, 
		    const region* reg1, const region* reg2);
  ~ncc_drgb_computor();

 public:
  inline float get_ncc_drgb()           {return get_similarity();}           // syntactic sugar
  inline float get_ncc_drgb_switched()  {return get_similarity_switched();}  // syntactic sugar
  inline void set_resolution(int res);
  inline void set_adaptive_resolution(bool ar);

  virtual inline void set_region1(const region* r1);
  virtual inline void set_region2(const region* r2);

  virtual void set_scale_factors1(float* C);
  virtual void set_scale_factors2(float* C);

 protected:
  //--- computations --------------------------------------------------
  bool compute_similarity();
  bool compute_similarity_switched();


private:
  crosscorrelation_computor* _ncc;
  drgb_computor*             _drgb;
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
void ncc_drgb_computor::set_adaptive_resolution(bool ar) {
  _ncc->set_adaptive_resolution(ar);
  _drgb->set_adaptive_resolution(ar);
  similarity_computor::set_adaptive_resolution(ar);
}


void ncc_drgb_computor::set_resolution(int res) {
  _ncc->set_resolution(res);
  _drgb->set_resolution(res);
  similarity_computor::set_resolution(res);
}


inline void ncc_drgb_computor::set_region1(const region* r1) {
  _ncc->set_region1(r1);
  _drgb->set_region1(r1);
 similarity_computor::set_region1(r1);
}


inline void ncc_drgb_computor::set_region2(const region* r2) {
  _ncc->set_region1(r2);
  _drgb->set_region2(r2);
  similarity_computor::set_region2(r2);
}





#endif
