#ifndef _ncc_drgbnorm_computer2h
#define _ncc_drgbnorm_computer2h

//------------------------------------------------------------------------------
// .NAME  ncc_drgbnorm_computor - Class for computing a mixed similarity measure
//                                between two affinely invariant regions.
//                           
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   ncc_drgbnorm_computor.h
// .FILE   ncc_drgbnorm__computor.cpp
//
// .SECTION Description
//   ncc_drgbnorm_computor computes the similarity 
//   between two affinely invariant regions, as the sum of outputs of a
//   crosscorrelation_computor and a drgbnorm_computor.
//
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <region_matching/crosscorrelation_computor.h>
#include <region_matching/drgbnorm_computor.h>

class ncc_drgbnorm_computor : public similarity_computor {

 public:
  ncc_drgbnorm_computor() {}
  ncc_drgbnorm_computor(const color_buffer_t* buf1,
			const color_buffer_t* buf2);
  ncc_drgbnorm_computor(const color_buffer_t* buf1, 
			const color_buffer_t* buf2, 
			const region* reg1, const region* reg2);
  ~ncc_drgbnorm_computor();

 public:
  inline float get_ncc_drgbnorm()           {return get_similarity();}           // syntactic sugar
  inline float get_ncc_drgbnorm_switched()  {return get_similarity_switched();}  // syntactic sugar
  inline void set_resolution(int res);
  inline void set_adaptive_resolution(bool ar);

  virtual inline void set_region1(const region* r1);
  virtual inline void set_region2(const region* r2);

 protected:
  //--- computations --------------------------------------------------
  bool compute_similarity();
  bool compute_similarity_switched();


private:
  crosscorrelation_computor* _ncc;
  drgbnorm_computor*         _drgb;
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
void ncc_drgbnorm_computor::set_adaptive_resolution(bool ar) {
  _ncc->set_adaptive_resolution(ar);
  _drgb->set_adaptive_resolution(ar);
  similarity_computor::set_adaptive_resolution(ar);
}

void ncc_drgbnorm_computor::set_resolution(int res) {
  _ncc->set_resolution(res);
  _drgb->set_resolution(res);
  similarity_computor::set_resolution(res);
}


inline void ncc_drgbnorm_computor::set_region1(const region* r1) {
  _ncc->set_region1(r1);
  _drgb->set_region1(r1);
 similarity_computor::set_region1(r1);
}


inline void ncc_drgbnorm_computor::set_region2(const region* r2) {
  _ncc->set_region1(r2);
  _drgb->set_region2(r2);
  similarity_computor::set_region2(r2);
}



#endif
