#include "ncc_drgbsnorm_computor.h"

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_image_as.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>


// -- Constructors / Destructors ------------------------------
ncc_drgbsnorm_computor::ncc_drgbsnorm_computor(const color_buffer_t* buf1,
					       const color_buffer_t* buf2)
  : similarity_computor(buf1, buf2)
{
  _ncc = new crosscorrelation_computor(buf1, buf2);
  _drgb = new drgbsnorm_computor(buf1, buf2);
}



ncc_drgbsnorm_computor::ncc_drgbsnorm_computor(const color_buffer_t* buf1, 
					       const color_buffer_t* buf2, 
					       const region* r1, const region* r2)
  : similarity_computor(buf1, buf2, r1, r2)
{
  _ncc = new crosscorrelation_computor(buf1, buf2);
  _drgb = new drgbsnorm_computor(buf1, buf2);
}


ncc_drgbsnorm_computor::~ncc_drgbsnorm_computor() {
  delete _ncc;
  delete _drgb;
}


// -- Computations -------------------------------------------------------
bool ncc_drgbsnorm_computor::compute_similarity_switched() {
  vcl_cout << "ncc_drgbsnorm_computor::compute_similarity_switched() not yet implemented" << vcl_endl;
  return false;
}


bool ncc_drgbsnorm_computor::compute_similarity() {
  _sim = _ncc->get_similarity(_region1, _region2);
  _sim += _drgb->get_similarity(_region1, _region2);
  _sim_valid = true;
  return true;
}
