#ifndef _ncc_region_matcherh_
#define _ncc_region_matcherh_


//------------------------------------------------------------------------------
// .NAME ncc_region_matcher - Simple class for matching 
//       two lists of affinely invariant regions based on 
//       normalized crosscorrelation
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   ncc_region_matcher.h
// .FILE   ncc_region_matcher.C
//
// .SECTION Description
//    The class ncc_region_matcher finds matches in two lists of 
//    affinely invariant regions based on 
//       normalized crosscorrelation. 
//     
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <region_matching/reg_match_params.h>
#include <region_matching/region_matcher.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_memory_image_of.h>



class ncc_region_matcher : public region_matcher
{
  // PUBLIC INTERFACE----------------------------------------------------------
public:
  // Constructors/Destructors--------------------------------------------------
  ncc_region_matcher(const vil1_image* im1,
		     const vil1_image* im2,
		     regions_list_t& reglist1, 
		     regions_list_t& reglist2,
		     const reg_match_params* p);  
  ~ncc_region_matcher();  
  
  // Utility Methods-----------------------------------------------------------
  bool match_region(region* reg1, region*& reg2, int& indx); // match region from reglist1 (general-two way)
  bool match_region1(region* reg1, region*& reg2, int& indx); // match region from reglist1
  bool match_region2(region* reg2, region*& reg1, int& indx); // match region from reglist2

protected:
  bool match_region1_parall(region* reg1, region*& reg2, int& indx); // match parall. region from reglist1
  bool match_region1_ell(region* reg1, region*& reg2, int& indx); // match ell. region from reglist1
  bool match_region2_parall(region* reg2, region*& reg1, int& indx); // match parall. region from reglist2
  bool match_region2_ell(region* reg2, region*& reg1, int& indx); // match ell. region from reglist2
  
protected:
  bool _rgb; // needed to free memory in destructors
  bw_buffer_t* _grey1;
  bw_buffer_t* _grey2;

};


#endif // _ncc_region_matcherh_
