#include <region_matching/pairwise_indexed_match.h>
#include <misc_utility/defines.h>
#include <regions/region.h>

int pairwise_indexed_match::_nb_matches = 0;
regions_list_t* pairwise_indexed_match::_orig_regions = NULL;
regions_list_t* pairwise_indexed_match::_ref_regions = NULL;

//--- Constructors/Destructors--------------------------------------------------
pairwise_indexed_match::pairwise_indexed_match()
  :_orig_indx(-1), _ref_indx(-1)
{
  _nb_matches++;
}

pairwise_indexed_match::pairwise_indexed_match(int orig_indx, int ref_indx)
  :_orig_indx(orig_indx), _ref_indx(ref_indx)
{
  _nb_matches++;
}
  
pairwise_indexed_match::pairwise_indexed_match(region* orig_reg, region* ref_reg)
{
  _orig_indx = get_orig_index(orig_reg);
  _ref_indx = get_ref_index(ref_reg);

  _nb_matches++;
}

pairwise_indexed_match::~pairwise_indexed_match()
{
  _nb_matches--;
}

//--- Data access --------------------------------------------
region* pairwise_indexed_match::get_orig_region() const
{
  if (valid_orig_index(_orig_indx))
    return (*_orig_regions)[_orig_indx];
  else
    return NULL;
}

region* pairwise_indexed_match::get_ref_region() const
{
  if (valid_ref_index(_ref_indx))
    return (*_ref_regions)[_ref_indx];
  else
    return NULL;
}


bool pairwise_indexed_match::set_orig_region(region* orig_reg)
{
  for (int indx =0; indx < _orig_regions->size(); ++indx)
  {
    if ((*_orig_regions)[indx] == orig_reg)
    {
      _orig_indx = indx;
      return true;
    }
  }

  return false;
}


bool pairwise_indexed_match::set_ref_region(region* ref_reg)
{
  for (int indx =0; indx < _ref_regions->size(); ++indx)
  {
    if ((*_ref_regions)[indx] == ref_reg)
    {
      _ref_indx = indx;
      return true;
    }
  }

  return false;
}

bool pairwise_indexed_match::set_orig_region(int orig_indx)
{
  if (!valid_orig_index(orig_indx))
    return false;

  _orig_indx = orig_indx;
  return true;
}

bool pairwise_indexed_match::set_ref_region(int ref_indx) 
{
  if (!valid_ref_index(ref_indx))
    return false;

  _ref_indx = ref_indx;
  return true;
}


void pairwise_indexed_match::set_orig_regions_list(regions_list_t* orig_regions)
{
  if (_nb_matches > 0)
    vcl_cout << "Warning: changing regions list while nb. of matches != 0" << vcl_endl;
 _orig_regions = orig_regions;
}

void pairwise_indexed_match::set_ref_regions_list(regions_list_t* ref_regions)
{
  if (_nb_matches > 0)
    vcl_cout << "Warning: changing regions list while nb. of matches != 0" << vcl_endl;
 _ref_regions = ref_regions;
}


//--- match I/O --------------------------------------------
void pairwise_indexed_match::print(vcl_ostream& str) const
{
  str << "Pairwise indexed match " << vcl_endl;
  if ( valid_orig_index(_orig_indx))
    str << (*_orig_regions)[_orig_indx]->get_pt0()->x() << ", " 
	<< (*_orig_regions)[_orig_indx]->get_pt0()->y();
  else
    str << "(invalid index) ";
  if ( valid_ref_index(_ref_indx))
    str << "  ->  " << (*_ref_regions)[_ref_indx]->get_pt0()->x() 
	<< ", " << (*_ref_regions)[_ref_indx]->get_pt0()->y() << vcl_endl;
  else
    str << "  ->  (invalid index)" << vcl_endl;
}

//--- Helper functions --------------------------------------
bool pairwise_indexed_match::valid_orig_index(int indx)
{
  if (!_orig_regions)
    return false;

  return ((indx >= 0) && (indx < _orig_regions->size()));
}

bool pairwise_indexed_match::valid_ref_index(int indx)
{
  if (!_ref_regions)
    return false;

  return ((indx >= 0) && (indx < _ref_regions->size()));
}

int pairwise_indexed_match::get_orig_index(region* orig_reg)
{
  if (!_orig_regions)
    return -1;

  for (int indx = 0; indx < _orig_regions->size(); indx++)
  {
    if ((*_orig_regions)[indx] == orig_reg)
      return indx;
  }

  return -1;
}

int pairwise_indexed_match::get_ref_index(region* ref_reg)
{
  if (!_ref_regions)
    return -1;

  for (int indx = 0; indx < _ref_regions->size(); indx++)
  {
    if ((*_ref_regions)[indx] == ref_reg)
      return indx;
  }

  return -1;
}

region* pairwise_indexed_match::get_orig_region(int indx)
{
  if (!valid_orig_index(indx))
    return NULL;
  
  return (*_orig_regions)[indx];
}

region* pairwise_indexed_match::get_ref_index(int indx)
{
  if (!valid_ref_index(indx))
    return NULL;
  
  return (*_ref_regions)[indx];
}
 

bool pairwise_indexed_match::operator==(pairwise_indexed_match const &that) const
{
  return ( (_orig_indx == that.get_orig_index()) &&
	   (_ref_indx == that.get_ref_index()));
}
