#ifndef _pairwise_indexed_match_h_
#define _pairwise_indexed_match_h_


//------------------------------------------------------------------------------
// .NAME pairwise_indexed_match - class for representing a pairwise match
//    between two affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   pairwise_indexed_match.h
// .FILE   pairwise_indexed_match.cpp
//
// .SECTION Description
//    The class pairwise_indexed_match is a class for representing pairwise matches, 
//    where a match is represented as a pair of indices into two lists of regions.
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <region_matching/pairwise_match.h>
#include <misc_utility/defines.h>
#include <regions/region.h>

class pairwise_indexed_match : public pairwise_match
{

  // PUBLIC INTERFACE----------------------------------------------------------
public:
  //--- Constructors/Destructors--------------------------------------------------
  pairwise_indexed_match();
  pairwise_indexed_match(int orig_indx, int ref_indx);
  pairwise_indexed_match(region* orig_reg, region* ref_reg); // not so efficient - try not to use this one
  virtual ~pairwise_indexed_match();

  //--- Data access --------------------------------------------
  virtual inline match_type get_type() const { return match::PAIRWISE_INDEXED_MATCH; }

  virtual region* get_orig_region() const;
  virtual region* get_ref_region() const;
  virtual int get_orig_index() const {return _orig_indx; }
  virtual int get_ref_index() const {return _ref_indx; }

  virtual bool set_orig_region(region* orig_reg) ;
  virtual bool set_ref_region(region* ref_reg) ;
  virtual bool set_orig_region(int orig_indx);
  virtual bool set_ref_region(int ref_indx);

  static regions_list_t* get_orig_regions_list() {return _orig_regions;}
  static regions_list_t* get_ref_regions_list() {return _ref_regions;}
  static void set_orig_regions_list(regions_list_t* orig_regions);
  static void set_ref_regions_list(regions_list_t* ref_regions);

  //--- match I/O --------------------------------------------
  virtual void print(vcl_ostream& str) const ;

  //--- Safe Down Casting -------------------------------------
  virtual void* Cast() { return this; }
  virtual inline pairwise_indexed_match* cast_to_pairwise_indexed_match(){ return this; }
  virtual inline const pairwise_indexed_match* cast_to_pairwise_indexed_match() const { return this; }

  //--- Helper functions --------------------------------------
  static bool valid_orig_index(int indx);
  static bool valid_ref_index(int indx);
  static int get_orig_index(region* orig_reg);
  static int get_ref_index(region* ref_reg);
  static region* get_orig_region(int indx);
  static region* get_ref_index(int indx);

  virtual bool operator==(pairwise_indexed_match const &that) const;


protected:
  static regions_list_t* _orig_regions;
  static regions_list_t* _ref_regions;

  static int _nb_matches;
  
  int _orig_indx;
  int _ref_indx;
};



#endif // _pairwise_indexed_match_h_
