#include <region_matching/pairwise_indexed_matches.h>
#include <region_matching/pairwise_indexed_match.h>
#include <region_matching/match.h>

// Constructors/Destructors--------------------------------------------------
pairwise_indexed_matches::pairwise_indexed_matches()
  :_nb_matches(0),
   _itValid(false)
{

}
  
pairwise_indexed_matches::~pairwise_indexed_matches()
{

}

  // Data Access -----------------------------------------------------------
bool pairwise_indexed_matches::add(match* mtch) 
{
  if (!mtch->cast_to_pairwise_indexed_match())
  {
    vcl_cerr << "Error: match you tried to add to pairwise_indexed_matches is not pairwise indexed match" << vcl_endl;
    return false;
  }
  
  pairwise_indexed_match* pwi_m = mtch->cast_to_pairwise_indexed_match();
  _matchlist.push_back(pwi_m); 
  _nb_matches++;
  pwi_m->ref();
 
  return true;
}


bool pairwise_indexed_matches::remove(match* mtch) 
{
  if (!mtch->cast_to_pairwise_indexed_match())
  {
    vcl_cerr << "Error: match you tried to remove from pairwise_indexed_matches is not pairwise indexed match" << vcl_endl;
     return false;
  }

  pairwise_indexed_match* pwi_m = mtch->cast_to_pairwise_indexed_match();
  _matchlist.remove(pwi_m);
  _nb_matches--;
  pwi_m->unref();

  return true;
}


//--- iterator-like operation to run over all matches / clusters

// get current match
match* pairwise_indexed_matches::get_match() const
{
  return *_it;
}


// remove the current match
bool pairwise_indexed_matches::remove_match()
{
  match* m = *_it;
  _it = _matchlist.erase(_it);
  m->unref();

  return true;
}

// remove all matches
void pairwise_indexed_matches::clear()
{
  for (_it = _matchlist.begin(); _it != _matchlist.end(); ++_it )
  {
    pairwise_indexed_match* m = *_it;
    m->unref();
  }

  _matchlist.clear();
  _itValid = false;
}
