#ifndef _pairwise_indexed_matchesh_
#define _pairwise_indexed_matchesh_


//------------------------------------------------------------------------------
// .NAME pairwise_indexed_matches - Class for storing pairwise
//    matches between affinely invariant regions based on index-lists
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   pairwise_indexed_matches.h
// .FILE   pairwise_indexed_matches.cpp
//
// .SECTION Description
//    The class pairwise_indexed_matches is a class for storing pairwise matches
//    between affinely invariant regions. Matches are of the type
//    pairwise_indexed_match, i.e. two indices in lists of regions
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//          Alexander Thomas:
//          Setting iterators to NULL or testing for it is invalid and not
//          supported in GCC 4, so added _itValid flag to indicate validity.
//
//
//----------------------------------------------------------------------------

#include <misc_utility/defines.h>
#include <region_matching/matches.h>
#include <regions/region.h>
#include <vil1/vil1_image.h>
#include <vcl/vcl_vector.h>
#include <vcl/vcl_utility.h>

class pairwise_indexed_matches : public matches
{
  // PUBLIC INTERFACE----------------------------------------------------------
public:
  // Constructors/Destructors--------------------------------------------------
  pairwise_indexed_matches();  
  virtual ~pairwise_indexed_matches();  

  // Data Access -----------------------------------------------------------
  virtual bool add(match* mtch);
  virtual bool remove(match* mtch);
  virtual int size() const {return _nb_matches;}

  // iterator-like operation to run over all matches / clusters
  virtual void reset() {_itValid = false;}
  virtual inline bool next_match();
  virtual match* get_match() const; // get current match
  virtual bool remove_match(); // remove the current match
  
  virtual void clear();

protected:
  vcl_list<pairwise_indexed_match* > _matchlist;
  vcl_list<pairwise_indexed_match* >::iterator _it;
  bool _itValid;

  int _nb_matches;
};

bool pairwise_indexed_matches::next_match()
{
  if (_itValid)
  {
    ++_it; 
    return (_it != _matchlist.end());
  }
  else
  {
    if (_nb_matches == 0)
      return false;
    
    _it = _matchlist.begin();
    _itValid = true;
    return true;
  }
}

#endif // _matchesh_
