#ifndef _pairwise_match_h_
#define _pairwise_match_h_


//------------------------------------------------------------------------------
// .NAME pairwise_match - abstract base class for representing a pairwise match
//    between two affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   pairwise_match.h
// .FILE   pairwise_match.cpp
//
// .SECTION Description
//    The class pairwise_match is a class for representing pairwise matches. 
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <region_matching/match.h>
#include <regions/region.h>


class pairwise_match : public match
{

  // PUBLIC INTERFACE----------------------------------------------------------
public:
  //--- Constructors/Destructors--------------------------------------------------
  pairwise_match() {}
  virtual ~pairwise_match() {}

  //--- Data access --------------------------------------------
  virtual inline match_type get_type() const { return match::PAIRWISE_MATCH; }
  virtual region* get_orig_region() const = 0;
  virtual region* get_ref_region() const = 0;
  virtual bool set_orig_region(region* orig_region) = 0;
  virtual bool set_ref_region(region* ref_region) = 0;

  //--- match I/O --------------------------------------------
  virtual void print(vcl_ostream& str) const {str << "Pairwise match " << vcl_endl;}

  //--- Safe Down Casting -------------------------------------
  virtual void* Cast() { return this; }
  virtual inline pairwise_match* cast_to_pairwise_match(){ return this; }
  virtual inline const pairwise_match* cast_to_pairwise_match() const { return this; }
  virtual inline pairwise_indexed_match* cast_to_pairwise_indexed_match(){ return NULL; }
  virtual inline const pairwise_indexed_match* cast_to_pairwise_indexed_match() const { return NULL; }
  virtual inline pairwise_region_match* cast_to_pairwise_region_match(){ return NULL; }
  virtual inline const pairwise_region_match* cast_to_pairwise_region_match() const { return NULL; }

  virtual inline bool operator==(pairwise_match const &that) const
    {return (get_orig_region() == that.get_orig_region()) &&
            (get_ref_region() == that.get_ref_region());}

protected:

};



#endif // _pairwise_match_h_
