#ifndef _pairwise_region_match_h_
#define _pairwise_region_match_h_


//------------------------------------------------------------------------------
// .NAME pairwise_region_match - class for representing a pairwise match
//    between two affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   pairwise_region_match.h
// .FILE   pairwise_region_match.cpp
//
// .SECTION Description
//    The class pairwise_match is a class for representing pairwise matches,
//    where a match is represented as a pair of pointers to regions.
//  
// .SECTION Author:
//          Tinne Tuytelaars
//          ESAT / KULeuven
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <region_matching/pairwise_match.h>
#include <regions/region.h>

class pairwise_region_match : public pairwise_match
{

  // PUBLIC INTERFACE----------------------------------------------------------
public:
  //--- Constructors/Destructors--------------------------------------------------
  pairwise_region_match();
  pairwise_region_match(region* orig_reg, region* ref_reg);
  virtual ~pairwise_region_match();  

  //--- Data access --------------------------------------------
  virtual inline match_type get_type() const { return match::PAIRWISE_REGION_MATCH; }
  virtual inline region* get_orig_region() const {return _orig_region;}
  virtual inline region* get_ref_region() const {return _ref_region;}
  
  virtual bool set_orig_region(region* orig_reg) { _orig_region = orig_reg; return true;}
  virtual bool set_ref_region(region* ref_reg) { _ref_region = ref_reg; return true;}


  //--- match I/O --------------------------------------------
  virtual void print(vcl_ostream& str) const;

  //--- Safe Down Casting -------------------------------------
  virtual void* Cast() { return this; }
  virtual inline pairwise_region_match* cast_to_pairwise_region_match(){ return this; }
  virtual inline const pairwise_region_match* cast_to_pairwise_region_match() const { return this; }

protected:

  region* _orig_region;
  region* _ref_region;
};



#endif // _match_h_
