#ifndef _reg_match_params_h
#define _reg_match_params_h

//----------------------------------------------------------------------------
// .NAME  - reg_match_params Class for storing region matching relevant parameters
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   reg_match_params.h
// .FILE   reg_match_params.C
//
// .SECTION Description
//    The class reg_match_params is a class for
//  storing various parameters for the matching of (affinely invariant) regions.
// 
//  At design time, it is assumed that all region matching processes
//  use the same parameters values for region extraction. The class is
//  therefore implemented as a singleton.
//
// .SECTION Author:
//          Tinne Tuytelaars - Andrew Turina - Vittorio Ferrari
//          ESAT / KULeuven           BIWI / ETH Zurich
//
// .SECTION Modifications:
//     
//
//----------------------------------------------------------------------------

class reg_match_params 
{
public:
  reg_match_params();
  ~reg_match_params();
	
  float min_sim; // min. value for pixel-wise similarity between regions
  float max_dist; // max. value for Mahalanobis distance between invariants

  int similarity_resolution;

  int rotation_finder_resolution;
  int rotation_finder_stepsize;

  int nb_of_tentative_matches; // how many tentative matches are tested further

  bool search_for_rotation;// apply extensive search over all rotations for max. ncc
  int search_for_rotation_stepsize; // discretisation step in degrees

  int max_multi_matches;     // maximum amount of regions matches to any single one (used by multi_region_matcher)
};



#endif // end of class reg_match_params
