#include "GDSixDim.h"

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <vcl/vcl_cmath.h>
#include <vcl/vcl_vector.h>
#include <vcl/vcl_list.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_image_as.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/vic_math.h> // max6
#include <region_utility/misc.h>   // copy_region, project_region



// -- Constructors / Destructors ------------------------------
GDSixDim::GDSixDim(similarity_computor* sc,
		 region* r1, region* r2,
		 bool verbose)
  : bounded_refiner(sc, r1, r2, verbose)
{
}


GDSixDim::~GDSixDim() {
}



// should not be needed (def in refiner class)
// but otherwise compiler complains that this fct does not exists ???
region* GDSixDim::refine(region* r1, region* r2) {
  //vcl_cout << "In GDSixDim::refine(r1,r2)." << vcl_endl; // strange that I can't call refiner::refine(r1,r2) ...
  set_region1(r1);
  set_region2(r2);
  refine();
}


// Main computation
// creates a new region to be returned
region* GDSixDim::refine() { 
  // Center _region2
  _cen = _region2->center();
  _region2_cen = copy_region(_region2);
  _region2_cen->set_pt0(_region2_cen->get_pt0()->x() - _cen->x(), _region2_cen->get_pt0()->y() - _cen->y());
  _region2_cen->set_pt1(_region2_cen->get_pt1()->x() - _cen->x(), _region2_cen->get_pt1()->y() - _cen->y());
  _region2_cen->set_pt2(_region2_cen->get_pt2()->x() - _cen->x(), _region2_cen->get_pt2()->y() - _cen->y());

  // Main loop
  _tot_sims = 0;
  float oldv = -2; 
  _region2_tmp = copy_region(_region2);
  float maxv = _sc->get_similarity(_region1, _region2_tmp);
  _init_sim = maxv;
  _cur_best_sim = maxv;
  float* gp = _cur_best_p; // alias for short notation
  gp[0] = 0; gp[1] = 0; gp[2] = 1; gp[3] = 1; gp[4] = 0; gp[5] = 0; // initial best pars is the identity matrix
  
  if(_verbose)
    vcl_cout << "Sim      T  S   r  s" << vcl_endl;

  while(maxv > oldv) {
    if(_verbose) {
      vcl_cout << maxv << " "; print_pars(gp); vcl_cout << vcl_endl; // will gp be pretty printed ?
    }

    // Collect maxima in 1-neighborhood of _cur_best_p
    oldv = maxv;
    gradient_descent_1step(maxv);               // alters gp (ie: also _cur_best_p)
    _cur_best_sim = maxv;
  }

  // Prepares region to be returned
  float* A = construct_affine(gp);
  project_region(_region2_cen, _region2_tmp, A);
  _region2_tmp->set_pt0(_region2_tmp->get_pt0()->x() + _cen->x(), _region2_tmp->get_pt0()->y() + _cen->y());
  _region2_tmp->set_pt1(_region2_tmp->get_pt1()->x() + _cen->x(), _region2_tmp->get_pt1()->y() + _cen->y());
  _region2_tmp->set_pt2(_region2_tmp->get_pt2()->x() + _cen->x(), _region2_tmp->get_pt2()->y() + _cen->y());
  _region2_tmp->update_region();
  _max_sim = maxv;

  delete A;
  delete _region2_cen;
  return _region2_tmp;
}



float GDSixDim::gradient_descent_1step(float& max_sim) {
  float cand_s = _cur_best_sim;
  float* gp = _cur_best_p;  // alias for short notation
  float pars[6];
  pars[0] = gp[0]; pars[1] = gp[1]; pars[2] = gp[2];  // use memcopy !!
  pars[3] = gp[3]; pars[4] = gp[4]; pars[5] = gp[5];
  float best_pars[6];
  best_pars[0] = gp[0]; best_pars[1] = gp[1]; best_pars[2] = gp[2];
  best_pars[3] = gp[3]; best_pars[4] = gp[4]; best_pars[5] = gp[5];

  // why not scale == 1.1 ?
  //vcl_cout << "_bounds[3] = " << _bounds[3][0] << " " << _bounds[3][1] << " " << _bounds[3][2] << vcl_endl;

  for(float p0 = MAXTWO(gp[0]-_bounds[0][1], _bounds[0][0]); p0 <= gp[0]+_bounds[0][1]; p0 += _bounds[0][1]) {
    for(float p1 = MAXTWO(gp[1]-_bounds[1][1], _bounds[1][0]); p1 <= gp[1]+_bounds[1][1]; p1 += _bounds[1][1]) {
      for(float p2 = MAXTWO(gp[2]-_bounds[2][1], _bounds[2][0]); p2 <= gp[2]+_bounds[2][1]; p2 += _bounds[2][1]) {
	//vcl_cout << "FUCK:uplimit for p2 = " << gp[2]+_bounds[2][1] << vcl_endl;
	for(float p3 = MAXTWO(gp[3]-_bounds[3][1], _bounds[3][0]); p3 <= gp[3]+_bounds[3][1]; p3 += _bounds[3][1]) {
	  //vcl_cout << "FUCK:uplimit for p3 = " << gp[3]+_bounds[3][1] << vcl_endl;
	  for(float p4 = MAXTWO(gp[4]-_bounds[4][1], _bounds[1][0]); p4 <= gp[4]+_bounds[4][1]; p4 += _bounds[4][1]) {
	    for(float p5 = MAXTWO(gp[5]-_bounds[5][1], _bounds[5][0]); p5 <= gp[5]+_bounds[5][1]; p5 += _bounds[5][1]) {
	      // Prepare current affine trafo
	      pars[0] = p0; pars[1] = p1; pars[2] = p2;
	      pars[3] = p3; pars[4] = p4; pars[5] = p5;
	      float* A = construct_affine(pars);

	      //if(_verbose)
		//vcl_cout << "      pars = "; print_pars(pars); vcl_cout << vcl_endl;
	      
	      // Prepare region
	      project_region(_region2_cen, _region2_tmp, A);
	      // Recenter _region2_tmp in the image
	      _region2_tmp->set_pt0(_region2_tmp->get_pt0()->x() + _cen->x(), _region2_tmp->get_pt0()->y() + _cen->y());
	      _region2_tmp->set_pt1(_region2_tmp->get_pt1()->x() + _cen->x(), _region2_tmp->get_pt1()->y() + _cen->y());
	      _region2_tmp->set_pt2(_region2_tmp->get_pt2()->x() + _cen->x(), _region2_tmp->get_pt2()->y() + _cen->y());
	      _region2_tmp->update_region(); // must be done once, to ensure proper functioning of inside_region


	      // Compute similarity
	      float s = _sc->get_similarity(_region1, _region2_tmp);
	      _tot_sims++;

	      if(s > cand_s) {
		cand_s = s;
		best_pars[0] = pars[0]; best_pars[1] = pars[1];  best_pars[2] = pars[2]; 
		best_pars[3] = pars[3]; best_pars[4] = pars[4];  best_pars[5] = pars[5]; 
	      }
	      delete A;
	    }
	  }
	}
      }
    }
  }

  max_sim = cand_s;
  gp[0] = best_pars[0]; gp[1] = best_pars[1]; gp[2] = best_pars[2];
  gp[3] = best_pars[3]; gp[4] = best_pars[4]; gp[5] = best_pars[5];
}


