#ifndef _GDSixDim_h
#define _GDSixDim_h

//------------------------------------------------------------------------------
// .NAME  GDSixDim - Class for improving the registration
//                   corresponding affinely invariant regions.
//                   Implements a simple Gradient Descent in 6 Dimensions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   GDSixDim.h
// .FILE   GDSixDim.cpp
//
// .SECTION Description
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <region_propagation/bounded_refiner.h>

class GDSixDim : public bounded_refiner {

public:
  GDSixDim() {}
  GDSixDim(similarity_computor* sc,
	   region* reg1 = NULL, region* reg2 = NULL,
	   bool verbose = false);
  ~GDSixDim();

  //--- computations ---------------------------------------------------
  region* refine();                   // returns new _region2 (NULL if fail), fills in field _max_sim and _tot_sims
  region* refine(region* r1, region* r2);    // syntactic sugar ;-)

 
protected:
  //--- helper functions -----------------------------------------------
  float gradient_descent_1step(float& max_sim);

  // temp state (during iterations)
  float _cur_best_p[6];               // current best parameters (current position in the search-hypercube)
  float _cur_best_sim;                // associated similarity value
  region* _region2_tmp;               // temporary _region2, used to be trafoed around
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------


#endif
