#include "aoi_refiner.h"

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <vcl/vcl_cmath.h>
#include <vcl/vcl_vector.h>
#include <vcl/vcl_list.h>

#include <vil1/vil1_image.h>
#include <vil1/vil1_image_as.h>
#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/vic_math.h> // max6
#include <region_utility/misc.h>   // copy_region

#include <regions/ell_region.h>



// -- Constructors / Destructors ------------------------------
aoi_refiner::aoi_refiner(similarity_computor* sc,
	         	 region* r1, region* r2,
			 bool verbose)
  : refiner(sc, r1, r2, verbose)
{
  init_bounds();
}


aoi_refiner::~aoi_refiner() {
}


void aoi_refiner::init_bounds() {
  for(int v = 0; v < 3; v++) {
    _bounds[v] = 0.0;
  }
}


void aoi_refiner::set_bounds(float* b) {
  for(int v = 0; v < 3; v++) {
    _bounds[v] = b[v];
  }
}


// should not be needed (def in refiner class)
// but otherwise compiler complains that this fct does not exists ???
region* aoi_refiner::refine(region* r1, region* r2) {
  set_region1(r1);
  set_region2(r2);
  refine();
}


// Main computation
// BEW: still no color-trafo in this version
// (change when impl also propagation)
// creates a new region to be returned
// Crashes if passed a parall-region !
region* aoi_refiner::refine() { 
  region* reg2_tmp = copy_region(_region2);

  // Inits
  _tot_sims = 0;
  float cand_s = _sc->get_similarity(_region1, reg2_tmp);
  _init_sim = cand_s;
  float opt_r = 0.0;
  ell_region* e = reg2_tmp->cast_to_ellregion();
  e->rotate_axes_fast(_bounds[0]);                 // prepare for incremental rotation algo (avoid copying region for every discr step)

  // Main loop
  for(float r = _bounds[0]; r <= _bounds[2]; r += _bounds[1]) {
    float s = _sc->get_similarity(_region1, e);
    _tot_sims++;

    if(s > cand_s) {
      cand_s = s;
      opt_r = r;
    }

    e->rotate_axes_fast(_bounds[1]);               // iteratively chg axes, to avoid having to copy the region for every new r (slow). Does NOT update_region()
  }

  // Prepares region to be returned
  region* reg_rot = copy_region(_region2);
  reg_rot->cast_to_ellregion()->rotate_axes_fast(opt_r);
  reg_rot->update_region();
  _max_sim = cand_s;

  if(_verbose)
    vcl_cout << "Best sim = " << _max_sim << ". Best rot = " << opt_r << vcl_endl;

  delete reg2_tmp;
  return reg_rot;
}






