#ifndef _aoi_refiner_h
#define _aoi_refiner_h

//------------------------------------------------------------------------------
// .NAME  aoi_refiner - Class for improving the registration
//                      of the axes-of-inertia of affinely
//                      invariant regions. Corresponds to
//                      maximising the rotational aligniment
//                      of the two regions after mappng them
//                      to a circle.
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   aoi_refiner.h
// .FILE   aoi_refiner.cpp
//
// .SECTION Description
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <region_propagation/refiner.h>

class aoi_refiner : public refiner {

public:
  aoi_refiner() {}
  aoi_refiner(similarity_computor* sc,
	      region* reg1 = NULL, region* reg2 = NULL,
	      bool verbose = false);
  ~aoi_refiner();

  //--- computations ---------------------------------------------------
  region* refine();               // returns new _region2 (NULL if fail), fills in field _max_sim and _tot_sims
  region* refine(region* r1, region* r2);

  //--- parameter setting ----------------------------------------------
  void set_bounds(float* b);      // b is a 1x3 matrix


protected:  
  // Params
  float _bounds[3];               // bounds and discretization of 1d rotation space to be searched
  
  //--- helper functions -----------------------------------------------
  void init_bounds();
  
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------


#endif
