#ifndef _blade_propagator_h
#define _blade_propagator_h

//------------------------------------------------------------------------------
// .NAME  blade_propagator - Class for propagating regions on the base of a set of
//                           reference support regions. The prespective is
//                           shifted to the support regions.
//
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   blade_propagator.h
// .FILE   blade_propagator.cpp
//
// .SECTION Description
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <region_propagation/refiner.h>
#include <region_propagation/propagator.h>

class blade_propagator : public propagator {

public:
  blade_propagator() {}
  blade_propagator(const color_buffer_t* buf1,
		   const color_buffer_t* buf2,
		   similarity_computor* sc,
		   regions_list_t* ref_rs1,  regions_list_t* ref_rs2,
		   refiner* rf,
		   bool verbose = false);
  ~blade_propagator();

  //--- computations ---------------------------------------------------
  region* propagate(int rx, regions_list_t* cand_rs1);      // returns new region (NULL if fail)
  regions_list_t* propagate(regions_list_t* cand_rs1);     // same for a list of regions
  
  //--- parameters settings --------------------------------------------
  inline void set_sections(int sections) { _sections = sections;}


protected:
  //--- helper functions -----------------------------------------------
  int find_section(point_t* o, point_t* p);
  int closest_region(point_t* c, regions_list_t* rs, vcl_vector<int>* indxs);
  void find_cand_regs(int crx, regions_list_t* cand_rs1);
  
  //--- members --------------------------------------------------------
  refiner* _rf;

  //--- parameters -----------------------------------------------------
  int _sections;                                           // amount of circular sections

  //--- data structure -------------------------------------------------
  vcl_vector< vcl_vector<int>* > refs;                     // which ref regs will attempt to prop a certain cand reg
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------




#endif
