#ifndef _bounded_refiner_h
#define _bounded_refiner_h

//------------------------------------------------------------------------------
// .NAME  bounded_refiner - Class for improving the registration
//                          corresponding affinely invariant regions.
//                          Base class for algorithms which search a bounded 
//                          sub-hypervolume (hypercube) of the affine space
//                          centered at the identity.
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   bounded_refiner.h
// .FILE   bounded_refiner.cpp
//
// .SECTION Description
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <region_propagation/refiner.h>

class bounded_refiner : public refiner {

public:
  bounded_refiner() {}
  bounded_refiner(similarity_computor* sc,
		  region* reg1 = NULL, region* reg2 = NULL,
		  bool verbose = false);
  ~bounded_refiner();

  //--- parameter setting ----------------------------------------------
  void set_bounds(float** b);         // b is a 6x3 matrix
 
protected:
  //--- helper functions -----------------------------------------------
  void init_bounds();
  void print_pars(float* p);

  // Params
  float _bounds[6][3];                // bounds and discretization of search-hypercube

  // temp state (during iterations)
  region* _region2_cen;               // centered-at-(0,0) version of _region2
  point_t* _cen;                      // original center of _region2
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------


#endif
