#include "int_propagator.h"

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <vcl/vcl_cmath.h>
#include <vcl/vcl_vector.h>
#include <vcl/vcl_list.h>

#include <region_utility/misc.h>                  // compute_affine


// -- Constructors / Destructors ------------------------------
int_propagator::int_propagator(const color_buffer_t* buf1,
			       const color_buffer_t* buf2,
			       similarity_computor* sc,
			       regions_list_t* ref_rs1, regions_list_t* ref_rs2,
			       refiner* rf,
			       bool verbose)
  : propagator(buf1, buf2, sc, ref_rs1, ref_rs2, verbose)
{
	_rf = rf;
}


int_propagator::~int_propagator() {
}



// invoke only after prepare_affines
region* int_propagator::propagate(region* cand_r1) {
  vcl_cout << "Region " << cand_r1->get_sid() << vcl_flush;
  region* cand_r2 = propagator::propagate(cand_r1);             // fills prop_regs, prop_sims and ref_indx fields
  float best_sim = -1000.0;   
  int best_ref = -1;

  if(prop_regs.size() == 0) {
    vcl_cout << "  -> no support regions with var >= " << _min_var << " within " << _max_dist << " pixels" << vcl_endl;
    return NULL;
  }

  for(int i=0; i<prop_regs.size(); i++) {
    int rx = ref_indx[i];
    _rf->get_similarity_computor()->set_scale_factors1(_ref_cols[rx]);
    region* cand_r2_ref = _rf->refine(cand_r1, prop_regs[i]);
    delete prop_regs[i];
    _sc->set_scale_factors1(_ref_cols[rx]);                     // similarity for acceptance decision with correct color factors. 
    float sim = _sc->get_similarity(cand_r1, cand_r2_ref);      // need be recomputed because _rf need not use the same sim computor as _sc
    if(sim >= best_sim) {
      cand_r2 = cand_r2_ref;
      best_ref = ((*_ref_rs1)[rx])->get_sid();
      vcl_cout << "  ref=" << best_ref << " sim=" << sim << vcl_flush;
      best_sim = sim;
    }
		else
			delete cand_r2_ref;
  }
  
  if(best_sim >= _sim_t) {                                      // if best sim > abs min acceptance thresh (_sim_t)
    vcl_cout << "  -> Successfully propagated. Sim = " << best_sim << ". Ref = " << best_ref << vcl_endl;
    _prop_count++;
    _tot_sim += best_sim;
    _supports.push_back(best_ref);
  }
	else {
		delete cand_r2;
    cand_r2 = NULL;
    vcl_cout << "  -> No propagation. Best sim = " << best_sim << ". Ref = " << best_ref << vcl_endl;
  }

  return cand_r2;
}


regions_list_t* int_propagator::propagate(regions_list_t* cand_rs1) {
  regions_list_t* _cand_rs2 = new regions_list_t;
  _supports.clear();

  for (int rx = 0; rx < cand_rs1->size(); rx++) {
    region* cand_r2 = propagate((*cand_rs1)[rx]);
    if(cand_r2)
      _cand_rs2->push_back(cand_r2);
  }
  return _cand_rs2;
}

 
