#ifndef _int_propagator_h
#define _int_propagator_h

//------------------------------------------------------------------------------
// .NAME  int_propagator - Class for propagating regions on the base of a set of
//                         reference support regions. The candidate region
//                         is first tentatively propagated via all support ones.
//                         The best tentative is refined and accepted if above
//                         similarity threshold.
//
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   int_propagator.h
// .FILE   int_propagator.cpp
//
// .SECTION Description
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <region_propagation/refiner.h>
#include <region_propagation/propagator.h>

class int_propagator : public propagator {

public:
  int_propagator() {}
  int_propagator(const color_buffer_t* buf1,
		 const color_buffer_t* buf2,
		 similarity_computor* sc,
                 regions_list_t* ref_rs1,  regions_list_t* ref_rs2,
		 refiner* rf,
	         bool verbose = false);
  ~int_propagator();

  //--- computations ---------------------------------------------------
  region* propagate(region* cand_r1);                      // returns new region (NULL if fail)
  regions_list_t* propagate(regions_list_t* cand_rs1);     // same for a list of regions
  

protected:
  //--- helper functions -----------------------------------------------
  
  //--- members --------------------------------------------------------
  refiner* _rf;
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------




#endif
