#ifndef _raywalk_h
#define _raywalk_h

//------------------------------------------------------------------------------
// .NAME  raywalk - Class for improving the registration
//                  corresponding affinely invariant regions.
//                  Implements the algorithm in CVPR03 (ferrari et al)
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   raywalk.h
// .FILE   raywalk.cpp
//
// .SECTION Description
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <vil1/vil1_memory_image_of.h>
#include <vil1/vil1_rgb_byte.h>

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <region_propagation/bounded_refiner.h>

class raywalk : public bounded_refiner {

public:
  raywalk() {}
  raywalk(similarity_computor* sc,
	  region* reg1 = NULL, region* reg2 = NULL,
	  bool verbose = false);
  ~raywalk();

  //--- computations ---------------------------------------------------
  region* refine();                   // returns new _region2 (NULL if fail), fills in field _max_sim and _tot_sims
  region* refine(region* r1, region* r2);    // syntactic sugar ;-)

 
protected:
  //--- helper functions -----------------------------------------------
  float refine_1d(int p_inx, float& max_sim);      

  // temp state (during iterations)
  float _cur_best_p[6];               // current best parameters (current position in the search-hypercube)
  float _cur_best_sim;                // associated similarity value
  region* _region2_tmp;               // temporary _region2, used to be trafoed around
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------


#endif
