#include "refiner.h"

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <vcl/vcl_cmath.h>
#include <vcl/vcl_vector.h>
#include <vcl/vcl_list.h>



// -- Constructors / Destructors ------------------------------
refiner::refiner(similarity_computor* sc,
		 region* r1, region* r2,
		 bool verbose)
{
 _region1 = r1;
 _region2 = r2;

 _init_sim = 0;
 _max_sim = 0;

 _sc = sc;

 _tot_sims = 0;

 _verbose = verbose;
}


refiner::~refiner() {
}


region* refiner::refine(region* r1, region* r2) {
  //vcl_cout << "In refiner::refine(r1,r2)" << vcl_endl;
  set_region1(r1);
  set_region2(r2);
  refine();
}



// Constructs an affine matrix from a parameter vector p
// p[0:1] = translations x,y
// p[2:3] = scale x,y
// p[4]   = rotation
// p[5]   = shear
float* refiner::construct_affine(float* p) {
  float sinr = sin(p[4]);
  float cosr = cos(p[4]);

  float* A = new float[6];
  // First line of A matrix
  A[0] = p[2]*cosr  + p[5]*p[2]*sinr;
  A[1] = -p[2]*sinr + p[5]*p[2]*cosr;
  A[2] = p[0];                              // transl x

  // Second line
  A[3] = p[3]*sinr;
  A[4] = p[3]*cosr;
  A[5] = p[1];                              // transl y

  // Do not use HMatrix2D, as it can be multipled confortably only to HomgPoint2D, but
  // region class uses vgl_homg_point_2d, so it's a mess. Better do by hand,
  // and also faster ;-)
  return A;
}

