#ifndef _refiner_h
#define _refiner_h

//------------------------------------------------------------------------------
// .NAME  refiner - Base class for improving the registration
//                  corresponding affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   refiner.h
// .FILE   refiner.cpp
//
// .SECTION Description
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <misc_utility/defines.h>
#include <regions/region.h>

#include <region_matching/similarity_computor.h>


class refiner {

public:
  refiner() {}
  refiner(similarity_computor* sc,
	  region* r1 = NULL, region* r2 = NULL,
	  bool verbose = false);
  virtual ~refiner();

  inline void set_region1(region* r1);
  inline void set_region2(region* r2);

  inline float get_init_similarity() const {return _init_sim;}
  inline float get_max_similarity() const {return _max_sim;}
  inline similarity_computor* get_similarity_computor() {return _sc;}

  //--- computations ---------------------------------------------------
  virtual region* refine() = 0;                      // returns new refined _region2 (NULL if fail), fills in field _max_sim and _tot_sims
          region* refine(region* r1, region* r2);    // syntactic sugar ;-)

  
protected:
  //--- helper functions -----------------------------------------------
  float* construct_affine(float* p);                // p isa 1x6 vector

  //--- members --------------------------------------------------------
  similarity_computor* _sc;                         // used to evaluate similarity function to be optimized     
  float _init_sim;                                  // similarity before refinement
  float _max_sim;                                   // similarity after refinement
  int _tot_sims;                                    // total amount of similarities computed
  region* _region1;
  region* _region2;
  bool _verbose;                                    // display infos on screen or not
 
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

inline void refiner::set_region1(region* r1)
{
  _region1 = r1; 
}

inline void refiner::set_region2(region* r2)
{
  _region2 = r2; 
}



#endif
