#ifndef _uniform_propagator_h
#define _uniform_propagator_h

//------------------------------------------------------------------------------
// .NAME  uniform_propagator - Class for propagating uniform regions on the base of a set of
//                             reference support classical affinely invariant regions
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   uniform_propagator.h
// .FILE   uniform_propagator.cpp
//
// .SECTION Description
//   
// .SECTION Author:
//          Vittorio Ferrari
//               ETHZ
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

// NOTE: would vbe propavbably more legant to add a layer of inheritance,
// having a top base class propagator, then a child air_propagator (with
// further children int_prop and blade_prop), and noathe child uni_propagator.
// so several non-uni-prop related memebers are no longer inherited
// (eg: average_sim, grelevel_variance)

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <misc_utility/defines.h>
#include <regions/region.h>
#include <regions/uniform_region.h>
#include <region_propagation/propagator.h>


class uniform_propagator : public propagator {

public:
  uniform_propagator() {}
  uniform_propagator(const color_buffer_t* buf1,
		     const color_buffer_t* buf2,
		     similarity_computor* sc,
		     regions_list_t* ref_rs1,  regions_list_t* ref_rs2,
		     bool verbose = false);
  virtual ~uniform_propagator();

  //--- parameter setting ----------------------------------------------

  //--- computations ---------------------------------------------------
  uniform_regions_list_t* propagate(uniform_regions_list_t* cand_urs1);     // same for a list of uniform regions
  

protected:
  //--- helper functions -----------------------------------------------
  uniform_region* propagate(uniform_region* cand_ur1);                      // returns new uniform region (NULL if fail)
  bool* project_uniform_region(uniform_region* ur1, bool* flag1, int ref_rx, int& area1, int& area2);
  void prepare_affines2to1();
  bool* uniform_region_to_flag(uniform_region* ur, const color_buffer_t* buf);
  void expand_bb(int x, int y, point_t& tl, point_t& br);
  int project12(uniform_region* ur1, float* A, vil1_rgb<unsigned char>& colu, float thresh, point_t& tl, point_t& br, bool* flag2);
  int project21(point_t& tl2, point_t& br2, bool* flag1, float* A, vil1_rgb<unsigned char>& colu, float thresh, bool* flag2);

  //--- members --------------------------------------------------------
  float** _ref_affs21;                                                      // reference affine transformations from view 2 to view 1
  int w1,h1,w2,h2;                                                          // width and height of the two images

  // parameters
  

};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------




#endif
