#include "Vitto_region_file.h"

#include <vcl_iostream.h>
#include <vcl_fstream.h>

#include <misc_utility/defines.h>
#include <regions/region.h> 
#include <regions/ell_region.h>
#include <regions/parall_region.h>
#include <vgl/vgl_homg_point_2d.h>
#include <vcl/vcl_list.h>
#include <vil1/vil1_image.h>
#include <misc_utility/loaders.h>       // end_of_line

Vitto_region_file::Vitto_region_file(bool verbose)
  : region_file(verbose)
{
}


Vitto_region_file::~Vitto_region_file()
{
}

bool Vitto_region_file::save_regions(regions_list_t& regs, 
				     const char* filename)
{
  if (_verbose) vcl_cout << "Saving regions" << vcl_endl;

  vcl_ofstream regfile(filename);

  // set im_ID
  _im_ID = new char[10];
  // CHANGED: Now uses last number in filename instead of first number.
  int i = strlen(filename);
	// Skip extension
  while( --i >= 0 && filename[i] != '.' ) {};
	if( i < 0 )
		i = strlen(filename);
  while( --i >= 0 ) {
    if( (filename[i] >= '0' ) && (filename[i] <= '9') ) // is a number
      break;
  }
  int lastDigit = i;
  while( --i >= 0 ) {
    if( (filename[i] < '0' ) || (filename[i] > '9') ) // is not a number
      break;
  }
  if( lastDigit >= 0 ) {
    strncpy( _im_ID, &(filename[i+1]), lastDigit-i );
    _im_ID[lastDigit-i] = '\0';
  }
  else // use default of 1
    strcpy( _im_ID, "1" );
  
  if (_verbose) vcl_cout << "image_id: " << _im_ID << vcl_endl;
  // write header
  if (!write_header(regs, regfile))
    return false;

  // write regions 
  region* reg;
  for (regions_list_t::iterator it = regs.begin(); it != regs.end(); ++it)
  {
    reg = *it;
    if (!write_region(reg, regfile))
      return false;
  }

  if(regs.size() == 0) {
    regfile << vcl_endl; // forces an empty file to be created
  }

  return true;
}


/* Modification by Alexander: the apparently unused 14th element of each
 * line is now used to store the descriptor/invariants length. */
bool Vitto_region_file::write_region(region* reg, vcl_ofstream& regfile)
{
	const vnl_vector<double>* inv = reg->get_invariants_ptr();
	unsigned int invLen = 0;
	if(inv)
		invLen = inv->size();

  if (reg->get_type() == region::ELL_REGIONTYPE)
  {
    // elliptical region
    regfile << _im_ID << "  " 
	    << reg->get_sid()  << "  " 
	    << "21" << "  " 
	    << reg->get_pt0()->x() << " "
	    << reg->get_pt0()->y() << "  " 
	    << reg->get_pt1()->x() << " " 
	    << reg->get_pt1()->y() << "  " 
	    << reg->get_pt2()->x() << " " 
	    << reg->get_pt2()->y() << "  " 
	    << reg->get_pt1()->x() << " " 
	    << reg->get_pt1()->y() << "  " 
	    << reg->get_pt2()->x() << " " 
	    << reg->get_pt2()->y() << "  " 
      << invLen;
  }
  else
  {
    // parall. region
    regfile << _im_ID << "  " 
	    << reg->get_sid()  << "  " 
	    << reg->get_specific_type() << "  " 
	    << reg->get_pt0()->x() << " "
	    << reg->get_pt0()->y() << "  " 
	    << reg->get_pt1()->x() << " " 
	    << reg->get_pt1()->y() << "  " 
	    << reg->get_pt2()->x() << " " 
	    << reg->get_pt2()->y() << "  " 
      << " 0 0 0 0  "
      << invLen;
  }

  // Write invariant vector
  if(inv) {
		regfile << " ";
    for (int i=0; i < inv->size(); i++) {
      regfile << " " << (*inv)[i];
    }
  }

  regfile << vcl_endl;

  return true;
}


bool Vitto_region_file::load_regions_impl(regions_list_t& regs, 
					  vcl_ifstream& regfile, 
					  vil1_image* image, 
					  points_list_t* anchors)
{
  int id, img_id, invLen;
  int regtype;
  double x,y, tmp;
  while (regfile >> img_id)    // image ID
  {
    regfile >> id;             // region ID
    regfile >> regtype;        // region type
    regfile >> x;
    regfile >> y;
    point_t* pt0 = new point_t(x,y);
    regfile >> x;
    regfile >> y;
    point_t* pt1 = new point_t(x,y);
    regfile >> x;
    regfile >> y;
    point_t* pt2 = new point_t(x,y);
    regfile >> tmp;            // trailing zeros and/or repeated coordinates
    regfile >> tmp;
    regfile >> tmp;
    regfile >> tmp;
    regfile >> invLen;         // invariants/descriptor length
    
    region* reg;
    if (regtype == 21)   // can't use region::ELL_REGIONTYPE as it's value might not be 21
      reg = new ell_region(pt0, pt1, pt2, region::ELL,
			   pt0, image);
    else
      reg = new parall_region(pt0, pt1, pt2, (region::region_specific_type)regtype,
			      pt0, image);

    reg->set_sid(id);                                   // set synchronization id


		// Load invariants (if present)
		if( invLen == 0 && !end_of_line(regfile) )
			invLen = 19;        // for backwards compatibility
			
		if( invLen > 0 ) {
			vnl_vector<double>* inv = new vnl_vector<double>(invLen);
			for (int i=0; i < invLen; i++) {
				double tmp;
				regfile >> tmp;
				(*inv)[i] = tmp;
			}
      reg->set_invariants(inv);
    }

    regs.push_back(reg);
  }

  return true;
}
