
#ifndef _aoi_computation_h
#define _aoi_computation_h

#include <regions/ell_region.h>

#include <vil1/vil1_rgb_byte.h>
#include <vsol/vsol_line_2d.h>


/***************************************************************************************
 *
 *	Class: AOIComputor
 *	Authors: Tinne Tuytelaars, Vittorio Ferrari, Andreas Turina
 *	Port:	Aaron Luttman
 *
 *	Date of port: November 8, 2001
 *
 *	Notes:  This is a fairly straightforward port of the class that supplied to the
 *			Regions repository.  The only change is that the image buffer was changed
 *			from a TargetJr 'BufferXY' to a vxl 'vil1_memory_image_of<vil1_rgb_byte>.'
 *			This class owns NO dynamic memory.
 *
 ***************************************************************************************/

class aoi_computor
{
public:
  aoi_computor(ell_region* region, vil1_image* buf);
  aoi_computor(vil1_image* buf);
  ~aoi_computor();
	
  //Data Access
	
  // returns the rotation angle to be applied to the
  // deskewed region to bring pt1 and pt2 in alignment 
  // with the AOI
  inline double get_rotation_angle() { return _theta; }
	
	
  // returns the ratio between the moment around the
  // minor axis and the moment around the major axis.
  // This gives an indication about the uncertainty 
  // on the AOI location (e.g. in case of isotropic regions).
  inline double get_ratio() { return _Mu / _Mv; }
	
  // Computations
  bool compute();
  bool compute(ell_region* reg);
	

protected:
	
  // Support functions
  void axes_of_inertia(double& th_min, double& th_max, double sumxx, double sumyy, double sumxy);
  double inertia_moment(double th, double sumxx, double sumyy, double sumxy);
	
private:
  ell_region* _region;
  vil1_image *_buf;

  vil1_memory_image_of<unsigned char>* BlackAndWhite;

	
  double _Mu, _Mv;     // moment along major / minor axes
  double _theta;       // rotation angle in deskewed region
};

#endif
