
#include <region_utility/misc.h>                 // regions_clusters

#include "clusterfilter.h"


ClusterFilter::ClusterFilter( const vil1_image &image, bool bVerbose )
{
	verbose = bVerbose;
	buf_i = new color_buffer_t(image);
}

ClusterFilter::~ClusterFilter()
{
	delete buf_i;
}


unsigned int ClusterFilter::filter( const regions_list_t &regions,
                                    int min_main_cluster_size, int min_cluster_size,
                                    vcl_vector<int> *sid_filt,
                                    regions_list_t *regs_filt,
                                    vcl_vector<int> *sid_rmv )
{
	// Get clusters
	regions_list_t** clusters;

	int cl = 0; // number of clusters;
	clusters = regions_clusters(&regions, buf_i, cl);
	if(verbose)
		vcl_cout << cl << " clusters in the image." << vcl_endl;

	// Is the main cluster large enough ?
	int main_cluster_size = 0;
	for(int c=0; c<cl; c++) {
		if(clusters[c]->size() >= main_cluster_size) {
			main_cluster_size = clusters[c]->size();
		}
	}
	if(main_cluster_size < min_main_cluster_size) {
		if(verbose) {
			vcl_cout << "Main cluster has " << main_cluster_size << " < " << min_main_cluster_size << " regions." << vcl_endl;
			vcl_cout << "Removing all clusters" << vcl_endl;
		}
		return 0;
	}

	// Remove clusters with less than min_cluster_size regions
	unsigned int cl_filt = 0;                   // #survived clusters
	unsigned int rs_filt = 0;                   // #survived regions
	for(int c=0; c<cl; c++) {
		if(clusters[c]->size() >= min_cluster_size) {
			cl_filt++;
			for(int rx=0; rx<clusters[c]->size(); rx++) {
				region* r = (*(clusters[c]))[rx];
				if(sid_filt)
					sid_filt->push_back(r->get_sid());
				if(regs_filt)
					regs_filt->push_back(r);
				rs_filt++;
			}
		}
		else if(sid_rmv) {
			for(int rx=0; rx<clusters[c]->size(); rx++) {
				region* r = (*(clusters[c]))[rx];
				sid_rmv->push_back(r->get_sid());
			}
		}
	}

	if(verbose) {
		vcl_cout << cl_filt << " clusters after filtering." << vcl_endl;
		vcl_cout << rs_filt << " regions after filtering" << vcl_endl;
	}

  // output clusters for testing
  //for(int c=0; c<cl; c++) {
  //  vcl_cout << "cluster " << c << ":";
  //  for(int rx=0; rx<clusters[c]->size(); rx++) {
  //    region* r = (*(clusters[c]))[rx];
  //    vcl_cout << " " << r->get_sid();
  //  }
  //  vcl_cout << vcl_endl;
  //}

	return rs_filt;
}
