#include <region_utility/misc.h>
#include <regions/ell_region.h>
#include <regions/parall_region.h>
#include <misc_utility/vic_math.h>            // pdist
#include <vcl_iostream.h>
#include <vcl_fstream.h>
#include <region_utility/region_draw.h>       // draw_region
#include <misc_utility/byte_interpolation.h>      // get_interpolation_Vic

// instantiante a new region object
// with the same fields as sr
// IMP: works better than copy-constructor
// as can be applied before specifying
// specific class (ell_region or parall_region)
region* copy_region(region* sr) {
  region* cr = NULL; // copied region
  if(sr->get_type() == region::ELL_REGIONTYPE) {
    cr = new ell_region(*(sr->cast_to_ellregion()));
  } else {
    parall_region* tmp = (sr->cast_to_parallregion());
    cr = new parall_region(*tmp);
  }

  return cr;
}


// alters r2->pt0, r2->pt1, r2->pt2
void project_region(region* r1, region* r2, float* A) {
  // the region::set_* fcts do not involve any recomputation (decided to use them, as not too slow ;-)
  
  r2->set_pt0( r1->get_pt0()->x()*A[0] + r1->get_pt0()->y()*A[1] + A[2], r1->get_pt0()->x()*A[3] + r1->get_pt0()->y()*A[4] + A[5]);           
  r2->set_pt1( r1->get_pt1()->x()*A[0] + r1->get_pt1()->y()*A[1] + A[2], r1->get_pt1()->x()*A[3] + r1->get_pt1()->y()*A[4] + A[5]);
  r2->set_pt2( r1->get_pt2()->x()*A[0] + r1->get_pt2()->y()*A[1] + A[2], r1->get_pt2()->x()*A[3] + r1->get_pt2()->y()*A[4] + A[5]);

  r2->update_region(); // must be done once, to ensure proper functioning of inside_region
}



// instantiates a new float[6] array.
// ported from Tinne's old code
float* compute_affine(point_t* r1c, point_t* r11, point_t* r12, point_t* r2c, point_t* r21, point_t* r22) {
  float* A = new float[6];

  // linear component
  double N = ( (r12->x() - r1c->x()) * (r11->y() - r1c->y()) -
               (r11->x() - r1c->x()) * (r12->y() - r1c->y()) );
  double T = ( (r22->x() - r2c->x()) * (r11->y() - r1c->y()) -
               (r21->x() - r2c->x()) * (r12->y() - r1c->y()) );
  A[0] = T/N;
  T = ( (r12->x() - r1c->x()) * (r21->x() - r2c->x()) -
        (r11->x() - r1c->x()) * (r22->x() - r2c->x()) );
  A[1] = T/N;
  T = ( (r11->y() - r1c->y()) * (r22->y() - r2c->y()) -
        (r21->y() - r2c->y()) * (r12->y() - r1c->y()) );
  A[3] = T/N;
  T = ( (r12->x() - r1c->x()) * (r21->y() - r2c->y()) -
        (r11->x() - r1c->x()) * (r22->y() - r2c->y()) );
  A[4] = T/N;

  // translation component
  A[2] = r2c->x() - A[0]* r1c->x() - A[1]* r1c->y();
  A[5] = r2c->y() - A[3]* r1c->x() - A[4]* r1c->y();

  return A;
}


void print_affine(float* A) {
  vcl_cout << A[0] << " " << A[1] << " " << A[2] << vcl_endl;
  vcl_cout << A[3] << " " << A[4] << " " << A[5] << vcl_endl;
}



bool pt_in_image(point_t* p, int w, int h) {
  return (p->x() >= 0 && p->y() >= 0 && p->x() < w && p->y() < h);
}



bool in_image(const region* r, int w, int h) {
  return (r->get_min_x() >=0 && r->get_max_x() < w && r->get_min_y() >= 0 && r->get_max_y() < h);
}


bool save_matches_file(regions_list_t& regs1, regions_list_t& regs2, const char* filename) {
  if(regs1.size() != regs2.size()) {
    vcl_cout << "Not the same amount of regions in the two lists. Cannot Save." << vcl_endl;
    return false;
  }

  vcl_ofstream matchfile(filename);

  // set im_ID1
  char* _im_ID1 = new char[10];
  int i=0;
  while ( (filename[i] < '0' ) || (filename[i] > '9'))
    i++;
  int i0 = i;
  while ( (filename[i] >= '0' ) && (filename[i] <= '9')) 
  {
    _im_ID1[i-i0] = filename[i];
    i++;
  }
  _im_ID1[i-i0] = '\0';

  // set im_ID2
  char* _im_ID2 = new char[10];
  while ( (filename[i] < '0' ) || (filename[i] > '9'))
    i++;
  i0 = i;
  while ( (filename[i] >= '0' ) && (filename[i] <= '9')) 
  {
    _im_ID2[i-i0] = filename[i];
    i++;
  }
  _im_ID2[i-i0] = '\0';

  vcl_cout << "imid1 = " << _im_ID1 << " imid2 = " << _im_ID2 << vcl_endl;

  // write matches
  region *reg1, *reg2;
  for (int rx = 0; rx < regs1.size(); rx++) {
    reg1 = regs1[rx];
    reg2 = regs2[rx];
    matchfile << _im_ID1 << " " << reg1->get_sid() << " " << _im_ID2 << " " << reg2->get_sid() << vcl_endl;
  }

  return true;
}


// subset of rs with regions' size in [min_size, max_size]
// 'size' = lenght of pt0pt1, pt0pt2
regions_list_t filter_size_regions(regions_list_t& rs, float min_size, float max_size) {
  float ns = min_size;
  float ms = max_size;
  regions_list_t out;

  for(int rx=0; rx<rs.size(); rx++) {
    // Get reg data
    region* r = rs[rx];
    point_t* pt0 = r->get_pt0();
    point_t* pt1 = r->get_pt1();
    point_t* pt2 = r->get_pt2();
    
    // Set min/max sizes
    if(r->get_type() == region::ELL_REGIONTYPE) {
      ns = min_size;
      ms = max_size;
    } else {
      ns = min_size*2.0;
      ms = max_size*2.0;
    }

    // Check region
    if(pdist(pt0, pt1) >= ns && pdist(pt0, pt1) <= ms &&
       pdist(pt0, pt2) >= ns && pdist(pt0, pt2) <= ms) {
      out.push_back(r);
    }
  }

  return out;
}


// very inefficient, but exact version
// !! modified by Alexander to allow const arguments. If any old code which
// uses this function behaves weird, this is where you should look.
regions_list_t** regions_clusters(const regions_list_t* rs_in, const color_buffer_t* buf, int& cl) {
  vil1_rgb<unsigned char> col(255, 255, 255);
  regions_list_t** clusters = new regions_list_t*[rs_in->size()];   // do not delete, will be returned
  regions_list_t *rs = new regions_list_t(*rs_in);                  // copy list, so we can prune it without damaging client
  cl = 0;
  while(rs->size() > 0) {
    color_buffer_t* buf_c = new color_buffer_t(buf->width(), buf->height());
    vcl_vector<int>* new_cluster = new vcl_vector<int>();
    cl++;
    clusters[cl-1] = new regions_list_t();
    region* r = (*rs)[rs->size()-1];                             // get last element
    rs->pop_back();                                              // ... and remove it (no pop_front avilable in std::vector)
    clusters[cl-1]->push_back(r);
    draw_region(r, col, buf_c);
    //vcl_cout << vcl_endl << "Forming new cluster" << vcl_endl;
    //vcl_cout << "seed region : " << r->get_sid() << vcl_endl;

    bool needmore = (rs->size() > 0);
    while(needmore) {
      needmore = false;
      regions_list_t* rs2 = new regions_list_t;
      //vcl_cout << "  New iter:";
      for(int rx=0; rx<rs->size(); rx++) {
	if(overlap_share((*rs)[rx], col, buf_c) > 0) {
	  clusters[cl-1]->push_back((*rs)[rx]);
	  needmore = true;
	  draw_region((*rs)[rx], col, buf_c);
	  //vcl_cout << " " << (*rs)[rx]->get_sid();
	} else {
	  rs2->push_back((*rs)[rx]);
	}
      }
      delete rs;
      rs = rs2;
      //vcl_cout << vcl_endl;
    } // needmore

    delete buf_c;
  } // cluster-forming loop

	delete rs;
  return clusters;
}


// Set bounding-box (tl,br) to contain region r
void init_bb(region* r, point_t& tl, point_t& br) {
  tl.set(r->get_min_x(), r->get_min_y());
  br.set(r->get_max_x(), r->get_max_y()); 
}


void expand_bb(region* r, point_t& tl, point_t& br) {
  tl.set( MINTWO(r->get_min_x(), tl.x()), MINTWO(r->get_min_y(), tl.y()) );
  br.set( MAXTWO(r->get_max_x(), br.x()), MAXTWO(r->get_max_y(), br.y()) );
}


// Bounding box of the set of regions rs
// In:  rs
// Out: tl = top-left corner
//      br = bottom-right corner
void bounding_box(regions_list_t& rs, point_t& tl, point_t& br) {
  // Pre-condition: at least one region in rs
  if(rs.size() < 1) {
    vcl_cout << "WARNING: bounding_box: empty region list. Returning unaffected BB" << vcl_endl;
  }

  // Initialize bounding box on first region
  init_bb(rs[0], tl, br);
  
  // Iteratively extend BB to all other regions
  for(int r_inx = 1; r_inx < rs.size(); r_inx++) {
    expand_bb(rs[r_inx], tl, br);
  }
}


// Does the bounding-box of r1 intersect the one of r2 ?
bool bb_intersect(region* r1, region* r2) {
  // disjointness in x and y
  bool disjx = (r1->get_max_x() < r2->get_min_x()) || (r1->get_min_x() > r2->get_max_x());
  bool disjy = (r1->get_max_y() < r2->get_min_y()) || (r1->get_min_y() > r2->get_max_y());

  // overlapping = not disjoint
  return !disjx && !disjy;
}


// variance of greylevel image patch of r
float greylevel_variance(region* r, const bw_buffer_t* buf) {
  if(!in_image(r, buf->width(), buf->height())) {
    return 0.0;
  }

  int nb_ = 0;
  float avg_ = 0.0;
  float sum2_ = 0.0;

  float cx = r->get_pt0()->x();
  float cy = r->get_pt0()->y();
  float x1 = r->get_pt1()->x();
  float y1 = r->get_pt1()->y();
  float x2 = r->get_pt2()->x();
  float y2 = r->get_pt2()->y();

  float incr_x_a1, incr_y_a1, incr_x_a2, incr_y_a2, xi, yi;

  int _res = 30;                              // not so imp to have this dynamic
  int resl1 = _res - 1;

  if (r->get_type() != region::ELL_REGIONTYPE) // NB_QUAD == 1
  {
    xi = cx; yi = cy;
    incr_x_a1 = (float)(x1 - cx) / resl1; // incrementals for axe 1
    incr_y_a1 = (float)(y1 - cy) / resl1;

    incr_x_a2 = (float)(x2 - cx) / resl1; // incrementals for axe 2
    incr_y_a2 = (float)(y2 - cy) / resl1;

    float pix;
    float cur_x, cur_y;                   // current position inside region
    float xia, yia;                       // x,y init axis
    
    for (int i=0; i<_res; i++) {
      xia = xi + i*incr_x_a1;             // cur_x|y now positioned to start of other axis
      yia = yi + i*incr_y_a1;
      cur_x = xia;
      cur_y = yia;
      
      for (int j=0; j<_res; j++) {
	pix = get_interpolation_Vic(buf, cur_x, cur_y);
	
	nb_++;
	avg_ += pix;
	sum2_ += pix*pix;
	
	cur_x += incr_x_a2;
	cur_y += incr_y_a2;
      }
    }
  }
  else { // NB_QUAD == 4
    xi = cx - (x1 - cx) - (x2 - cx); // initial pt at the start of 4 quadrans
    yi = cy - (y1 - cy) - (y2 - cy);

    incr_x_a1 = (float)(x1 - cx)*2.0 / resl1; // incrementals for axe 1
    incr_y_a1 = (float)(y1 - cy)*2.0 / resl1;

    incr_x_a2 = (float)(x2 - cx)*2.0 / resl1; // incrementals for axe 2
    incr_y_a2 = (float)(y2 - cy)*2.0 / resl1;

    float pix;
    float cur_x, cur_y; // current position inside region
    float xia, yia;                           // x,y init axis
    
    for (int i=0; i<_res; i++) {
      xia = xi + i*incr_x_a1; // cur_x|y now positioned to start of other axe
      yia = yi + i*incr_y_a1;
      cur_x = xia;
      cur_y = yia;
      
      for (int j=0; j<_res; j++) {
	if (r->inside_region(cur_x, cur_y)) {
	  pix = get_interpolation_Vic(buf, cur_x, cur_y);
	  
	  nb_++;
	  avg_ += pix;
	  sum2_ += pix*pix;
	}  // compare with Matlab

	cur_x += incr_x_a2;
	cur_y += incr_y_a2;
      }
    }
  }

  if (nb_ == 0)
    {vcl_cout << "error in greylevel_variance: nb_ = 0 " << vcl_endl;}
  

  avg_ /= (float)nb_;

  return (sum2_/(float)nb_) - avg_*avg_;
}


void sparsifyRegions( regions_list_t &sparse, const regions_list_t &dense, unsigned int minSize )
/* Create a sparse copy of a regular regions list.
 * At least minSize+1 elements will be allocated.
 * For the sparse list holds: 
 *   sparse[s] == NULL or sparse[s]->get_sid() == s */
{
	int maxSid = -1;
	for( unsigned int i=0; i<dense.size(); i++ ) {
		if( dense[i]->get_sid() > maxSid )
			maxSid = dense[i]->get_sid();
	}
	if( minSize > maxSid )
		maxSid = minSize;
	sparse.assign( maxSid+1, (region*)NULL );
	for( unsigned int i=0; i<dense.size(); i++ )
		sparse[ dense[i]->get_sid() ] = dense[i];
}


void deSparsifyRegions( regions_list_t &dense, const regions_list_t &sparse )
/* Create a dense copy of the sparse regions list. */
{
	for( unsigned int s=0; s<sparse.size(); s++ ) {
		if( sparse[s] )
			dense.push_back(sparse[s]);
	}
}
