#ifndef __misc_h
#define __misc_h

#include <misc_utility/defines.h>
#include <regions/region.h>

region* copy_region(region* sr);
void project_region(region* r1, region* r2, float* A);  // alters r2
float* compute_affine(point_t* r1c, point_t* r11, point_t* r12, point_t* r2c, point_t* r21, point_t* r22);
void print_affine(float* A);
inline bool in_image(float x, float y, int w, int h);
bool pt_in_image(point_t* p, int w, int h);      
bool in_image(const region* r, int w, int h);
bool save_matches_file(regions_list_t& regs1, regions_list_t& regs2, const char* filename);
regions_list_t filter_size_regions(regions_list_t& rs, float min_size, float max_size);
regions_list_t** regions_clusters(const regions_list_t* rs_in, const color_buffer_t* buf, int& cl);
void bounding_box(regions_list_t& rs, point_t& tl, point_t& br);
void init_bb(region* r, point_t& tl, point_t& br);
void expand_bb(region* r, point_t& tl, point_t& br);
bool bb_intersect(region* r1, region* r2);
bool bb_intersect1t2(region* r1, region* r2);
float greylevel_variance(region* r, const bw_buffer_t* buf);

// inline fcts MUST be defined in the .h file (NOT in the .cpp)
inline bool in_image(float x, float y, int w, int h) {
  return (x >= 0 && y >= 0 && x < w && y < h);
}


inline float* compute_affine(region* r1, region* r2) {
  return compute_affine(r1->get_pt0(), r1->get_pt1(), r1->get_pt2(), 
			r2->get_pt0(), r2->get_pt1(), r2->get_pt2());   // DO NOT use r1->center() -> generates 3 aligned pts for parall regs
}


// Deletes all regions in a regions_list_t and clears the list
inline void delete_regions( regions_list_t &regs )
{
	for( unsigned int i=0; i<regs.size(); i++ )
		delete regs[i];
	regs.clear();
}

/* Create a sparse copy of a regular regions list.
 * At least minSize+1 elements will be allocated.
 * For the sparse list holds: 
 *   sparse[s] == NULL or sparse[s]->get_sid() == s */
void sparsifyRegions( regions_list_t &sparse, const regions_list_t &dense, unsigned int minSize=0 );

/* Create a dense copy of the sparse regions list. */
void deSparsifyRegions( regions_list_t &dense, const regions_list_t &sparse );

#endif
