#include "old_region_file.h"

#include <vcl_iostream.h>
#include <vcl_fstream.h>

#include <misc_utility/defines.h>
#include <regions/region.h> 
#include <regions/ell_region.h>
#include <regions/parall_region.h>
#include <vgl/vgl_homg_point_2d.h>
#include <vcl/vcl_list.h>
#include <vil1/vil1_image.h>
#include <misc_utility/loaders.h>   // is_digit, skip_to_eol

old_region_file::old_region_file(bool verbose)
  :region_file(verbose)
{
}


old_region_file::~old_region_file()
{
}

bool old_region_file::write_header(regions_list_t& regs, 
				     vcl_ofstream& regfile)
{
  regfile << "1  3  1" << vcl_endl;
  
  regfile << regs.size() << vcl_endl << vcl_endl;

  if (_verbose) vcl_cout << regs.size() << " regions to be saved" << vcl_endl;

  return true;
}

bool old_region_file::load_header(vcl_ifstream& regfile)
{
  int nb_qu, color, nb_regions, new_inv;
  regfile >> nb_qu;
  regfile >> color;
  regfile >> new_inv;
  regfile >> nb_regions;
  if (_verbose) 
    vcl_cout << nb_qu << "   " << color << "   " << new_inv << "   " << nb_regions << vcl_endl;

  return true;
}

bool old_region_file::write_region(region* reg, 
				   vcl_ofstream& regfile)
{
  regfile << reg->get_anchor_point()->x() << "  " 
	  << reg->get_anchor_point()->y() << "  1" << vcl_endl;
  regfile << reg->get_pt0()->x() << "  " << reg->get_pt0()->y() << "  1" << vcl_endl;
  regfile << reg->get_pt1()->x() << "  " << reg->get_pt1()->y() << "  1" << vcl_endl;
  regfile << reg->get_pt2()->x() << "  " << reg->get_pt2()->y() << "  1" << vcl_endl;
  
  // Write invariant vector
  const vnl_vector<double>* inv = reg->get_invariants_ptr();
  if(inv) {

    if(reg->get_type() == region::ELL_REGIONTYPE)
      regfile << "21";
    else
      regfile << (*inv)[0];

    for (int i=1; i < inv->size(); i++) {
      regfile << " " << (*inv)[i];
    }
  } else {
    vcl_cout << "WARNING: old_region_file::write_region: cannot find feature vector for region centered at : " << *(reg->get_pt0()) << vcl_endl;
  }

  regfile << vcl_endl;

  return true;
}

bool old_region_file::load_regions_impl(regions_list_t& regs, 
					vcl_ifstream& regfile,
					vil1_image* image, 
					points_list_t* anchors)
{
  double x, y, z;
  float xo = 0, yo = 0; // coordinates of previous anchorpoint
  point_t* prev_anchor = NULL;
  point_t* anchor = NULL;
  point_t *pt0, *pt1, *pt2;  
  int nb_points = 0;
  while (regfile >> x)
    {
      //vcl_cout << vcl_endl << "Loading new region" << vcl_endl;

      // load anchorpoint
      regfile >> y;
      regfile >> z;

      if ( (xo != x) || (yo != y))	
      {
	  anchor = new point_t(x, y);
	  if (anchors)
	    anchors->push_back(anchor);
	  nb_points++;
	  xo = x; yo = y;
	  prev_anchor = anchor;
	}
      else
      {
	anchor = prev_anchor;
      }

      // load region
      regfile >> x;
      regfile >> y;
      regfile >> z;
      pt0 = new point_t(x,y);

      regfile >> x;
      regfile >> y;
      regfile >> z;
      pt1 = new point_t(x, y);
 
      regfile >> x;
      regfile >> y;
      regfile >> z;
      pt2 = new point_t(x, y);

      //vcl_cout << "x = " << x << " y = " << y << vcl_endl;

      // load region type
      int type;
      regfile >> type;
      region* reg;

      switch(type)
      {
      case 21:
	reg = new ell_region(pt0, pt1, pt2,
			     (region::region_specific_type)region::ELL,
			     anchor, image);
	break;
      default:
	reg = new parall_region(pt0, pt1, pt2,
				(region::region_specific_type)type,
				anchor, image);
	break;
      }

      // load invariants
      vnl_vector<double>* inv = new vnl_vector<double>(19);
      inv[0] = type;
      if(is_digit(regfile)) {
	for (int i=0; i < 18; i++) {
	  double tmp;
	  regfile >> tmp;
	  //vcl_cout << "inv[ " << i << "] = " << tmp << vcl_endl;
	  (*inv)[i+1] = tmp;
	}
      } else {
        // 'nan' feature vector, or something corrupted
        //vcl_cout << "Corrupted feature vector. Setting all invariants to 0.0" << vcl_endl;
	for (int i=0; i < 18; i++) {
	  (*inv)[i+1] = 0.0;
	}
        skip_to_eol(regfile);
      }
      reg->set_invariants(inv);

      // Set synchro id
      reg->set_sid(reg->get_ID()+1);                         // set synchronization id to 1 more than the absolute, process-wide region_id. So never sid==0,
      // and also generate unique sids when loading multiple files
      
      regs.push_back(reg);
    }

  return true;
}

