#include "region_file.h"

#include <vcl_iostream.h>
#include <vcl_fstream.h>

#include <misc_utility/defines.h>
#include <regions/region.h> 
#include <regions/ell_region.h>
#include <regions/parall_region.h>
#include <vgl/vgl_homg_point_2d.h>
#include <vcl/vcl_list.h>
#include <vil1/vil1_image.h>


region_file::region_file(bool verbose)
{
  _verbose = verbose; 
}


region_file::~region_file()
{
}


bool region_file::save_regions(regions_list_t& regs, 
			       const char* filename)
{
  if (_verbose) vcl_cout << "Saving regions" << vcl_endl;

  vcl_ofstream regfile(filename);

  // write header
  if (!write_header(regs, regfile))
    return false;

  // write regions 
  region* reg;
  for (regions_list_t::iterator it = regs.begin(); it != regs.end(); ++it)
  {
    reg = *it;
    if (!write_region(reg, regfile))
      return false;
  }

  return true;
}



bool region_file::load_regions(regions_list_t& regs, 
			       const char* filename, 
			       vil1_image* image, 
			       points_list_t* anchors)
{
  vcl_ifstream regfile(filename);

  // check header
  if (!load_header(regfile))
    return false;

  // load regions
  if (!load_regions_impl(regs, regfile, image, anchors))
    return false;

  regfile.close();

  if (_verbose) 
    {
      vcl_cout << regs.size() << " new regions loaded ";
      if (anchors)
	vcl_cout << " for " << anchors->size() << " new points";
      vcl_cout << vcl_endl;
    }

  return true;
}

