#include "simple_region_file.h"

#include <vcl_iostream.h>
#include <vcl_fstream.h>

#include <misc_utility/defines.h>
#include <regions/region.h> 
#include <regions/ell_region.h>
#include <regions/parall_region.h>
#include <vgl/vgl_homg_point_2d.h>
#include <vil1/vil1_image.h>

simple_region_file::simple_region_file(bool verbose)
  : region_file(verbose)
{
}


simple_region_file::~simple_region_file()
{
}


bool simple_region_file::write_region(region* reg, 
				      vcl_ofstream& regfile)
{
  const point_t *anchor = reg->get_anchor_point();
  point_t *pt0 = reg->get_pt0();
  point_t *pt1 = reg->get_pt1();
  point_t *pt2 = reg->get_pt2();
  
  regfile << reg->get_type() << "  " << reg->get_specific_type() << "  " 
	  << anchor->x() << "  " << anchor->y() << "  " 
	  << pt0->x() << "  " << pt0->y() << "  " 
	  << pt1->x() << "  " << pt1->y() << "  " 
	  << pt2->x() << "  " << pt2->y() << vcl_endl;
  
  return true;
}

bool simple_region_file::load_regions_impl(regions_list_t& regs, 
					   vcl_ifstream& regfile,
					   vil1_image* image, 
					   points_list_t* anchors)
{
  int type, spec_type;
  double x, y;
  float xo = 0, yo = 0; // coordinates of previous anchorpoint
  point_t* prev_anchor = NULL;
  point_t* anchor = NULL;
  point_t *pt0, *pt1, *pt2;  
  int nb_points = 0;
  while (regfile >> type)
    {
      regfile >> spec_type;

      // load anchorpoint
      regfile >> x;
      regfile >> y;

      if ( (xo != x) || (yo != y))	
      {
	  anchor = new point_t(x, y);
	  if (anchors)
	    anchors->push_back(anchor);
	  nb_points++;
	  xo = x; yo = y;
	  prev_anchor = anchor;
	}
      else
      {
	anchor = prev_anchor;
      }

      // load region
      regfile >> x;
      regfile >> y;
      pt0 = new point_t(x,y);

      regfile >> x;
      regfile >> y;
      pt1 = new point_t(x, y);
 
      regfile >> x;
      regfile >> y;
      pt2 = new point_t(x, y);

      // create new region
      region* reg;
      switch((region::region_type)type)
      {
      case region::ELL_REGIONTYPE:
	reg = new ell_region(pt0, pt1, pt2,
			     (region::region_specific_type)spec_type,
			     anchor, image);
	break;
      case region::PARALL_REGIONTYPE:
	reg = new parall_region(pt0, pt1, pt2,
				(region::region_specific_type)spec_type,
				anchor, image);
	break;
      }

      regs.push_back(reg);
    }

  return true;
}

