
/****************************************************************************
*
*	Source File for EllipseDistFunc()
*
****************************************************************************/

#include <misc_utility/abl_utility.h>
#include <regions/region.h>
#include <regions/ellipse_dist_func.h>

#include <vsol/vsol_conic_2d.h>

EllipseDistFunc::EllipseDistFunc(vsol_conic_2d *E1, vsol_conic_2d *E2) : 
vnl_cost_function(2)
{
	hE1 = E1;
	hE2 = E2;
	
	ha1 = hE1->a();	hb1 = hE1->b();
	hc1 = hE1->c();	hd1 = hE1->d();
	he1 = hE1->e();	hf1 = hE1->f();
	
	ha2 = hE2->a();	hb2 = hE2->b();
	hc2 = hE2->c();	hd2 = hE2->d();
	he2 = hE2->e();	hf2 = hE2->f();
}


EllipseDistFunc::~EllipseDistFunc()
{
	
}


double EllipseDistFunc::f(const vnl_vector<double> &x)
{
	//  The two parameters of this function are x1 and x2 such that (x1,y1)
	//  lies on hE1 and (x2,y2) lies on hE2.  Thus given x1 and x2, there
	//  two possibilites for y1 and two for y2 making a total of four to
	//  check.
	
	static double x1, x2, y11, y12, y21, y22, disc, temp, t2, td;
	
	x1 = x(0);
	x2 = x(1);
	
	td = (hb1*x1 + he1)*(hb1*x1 + he1) - 4*hc1*(ha1*x1*x1 + hd1*x1 + hf1);
	
	if ( td < 0.0 )
	{
		return ABL_BIGNUM + 10*MINTWO(fabs(x1 - hE1->get_min_x()), fabs(x1 - hE1->get_max_x()));
	}
	
	disc = sqrt(td);
	
	y11 = (-(hb1*x1 + he1) + disc)/(2*hc1);
	y12 = (-(hb1*x1 + he1) - disc)/(2*hc1);
	
	td = (hb2*x2 + he2)*(hb2*x2 + he2) - 4*hc2*(ha2*x2*x2 + hd2*x2 + hf2);
	
	if ( td < 0.0 )
	{
		return ABL_BIGNUM + 10*MINTWO(fabs(x2 - hE2->get_min_x()), fabs(x2 - hE2->get_max_x()));
	}
	
	disc = sqrt(td);
	
	y21 = (-(hb2*x2 + he2) + disc)/(2*hc2);
	y22 = (-(hb2*x2 + he2) - disc)/(2*hc2);
	
	temp = MINFOUR( fabs(y11-y21), fabs(y11-y22), fabs(y12-y21), fabs(y12-y22) );
	
	if ( temp == fabs(y11-y21) )
	{
		t2 = y11 - y21;
	}
	else if ( temp == fabs(y11-y22) )
	{
		t2 = y11 - y22;
	}
	else if ( temp == fabs(y12 - y21) )
	{
		t2 = y12 - y21;
	}
	else
	{
		t2 = y12 - y22;
	}
	
	return sqrt( (x1-x2)*(x1-x2) + t2*t2 );
	
}


