#include <regions/uniform_region.h>

//-------------------------------------------------------------------
//--- CONSTRUCTORS / DESTRUCTORS ------------------------------------
//-------------------------------------------------------------------
  
uniform_region::uniform_region() {
  vil1_rgb<unsigned char> black(0,0,0);
  _col = black;
  _sid = 0;
}


uniform_region::uniform_region(bool* flag, const color_buffer_t* buf) {
  _sid = 0;

  // Inits
  int w = buf->width();
  int h = buf->height();
  long tR = 0; long tG = 0; long tB = 0;                    // total RGB values, for mean computation
  long tp = 0;                                              // total amount of pixels
  vil1_rgb<unsigned char> col;                               // current pixel color

  // Assemble region and compute color mean
  bool solid = false;                                       // latest pixel's on or off ?
  int xs,ys,xe,ye;
  for(int y=0; y<h; y++) {
    solid = false;
    for(int x=0; x<w; x++) {
      if( flag[x + y*w] && !solid) {
	// start of a new solid run
	xs = x; ys = y;
	col = (*buf)(x,y);
	tR += col.R(); tG += col.G(); tB += col.B();
	tp++;
	solid = true;
      }
       
      if( ! flag[x + y*w] && solid) {
	// end a solid run
	xe = (x-1); ye = y;
	col = (*buf)(x-1,y);
	tR += col.R(); tG += col.G(); tB += col.B();
	tp++;
	solid = false;
	add_segment(xs, ys, xe, ye);
      }
    } // loop over x
    
    // Did the latest segment run till the right border ?
    if(solid) {
      xe = w-1; ye = y;
      add_segment(xs, ys, xe, ye);
    }
  } // loop over y

  // Color mean
  vil1_rgb<unsigned char> meancol(tR/tp,tG/tp,tB/tp);
  _col = meancol;
}


uniform_region::~uniform_region() {

}


void uniform_region::print() {
  vcl_cout << "Uniform region status" << vcl_endl;
  vcl_cout << "Color : " << _col << vcl_endl;
  for(int s=0; s<number_of_segments(); s++) {
    vcl_cout << "  segment " << s << " : " << (*_segments_start[s]) << ", " << (*_segments_end[s]) << vcl_endl;
  }
}
