#ifndef _uniform_region
#define _uniform_region

//------------------------------------------------------------------------------
// .NAME  uniform_region - a region of approximately uniform color
//                        Beware: it is not a region in the region base class sense
// .LIBRARY 
// .HEADER 
// .INCLUDE 
// .FILE   uniform_region.h
// .FILE   uniform_region.cpp
//
// .SECTION Description
//
// .SECTION Author:
//          Vittorio Ferrari
//                ETHZ     
//
// .SECTION Modifications:
//
//
//----------------------------------------------------------------------------

#include <vcl_cmath.h>
#include <vcl_iostream.h>

#include <misc_utility/defines.h>


class uniform_region{

 public:
  uniform_region();
  uniform_region(bool* flag, const color_buffer_t* buf);
  ~uniform_region();

 public:
  inline void add_segment(int xs, int ys, int xe, int ye);
  inline int number_of_segments() {return _segments_start.size();}
  inline point_t* get_segment_start(int s) {return _segments_start[s];}
  inline point_t* get_segment_end(int s) {return _segments_end[s];}
  inline vil1_rgb<unsigned char> get_color() {return _col;}
  inline void set_color(vil1_rgb<unsigned char> col) {_col = col;}
  //int area();  // may be useful in future
  void print();
  inline void set_sid(int sid) {_sid = sid;};
  inline int get_sid() const {return _sid;}

 protected:
  vcl_vector<point_t*>  _segments_start;                               // list-of-line segments
  vcl_vector<point_t*>  _segments_end;                                 // representation
  vil1_rgb<unsigned char> _col;                                         // color
  int _sid;                                                            // synchronization id (used in multiple view context)
};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------


inline void uniform_region::add_segment(int xs, int ys, int xe, int ye) {
  point_t* start = new point_t(xs, ys);
  point_t* end = new point_t(xe, ye);
  _segments_start.push_back(start);
  _segments_end.push_back(end);
}


#endif
