#ifndef FMatrixComputeMLESAC_h_Fix
#define FMatrixComputeMLESAC_h_Fix
//:
// \file
// \brief Robust FMatrix computation
//
//    FMatrixComputeMLESAC implements
//    Phil Torr's MLESAC (Maximum Likelyhood Estimation) of the fundamental matrix.
//
//    Points are preconditioned as described in [Hartley, ``In defence of
//    the 8-point algorithm'', ICCV95], and the resulting F matrix is rank-2
//    truncated.  The conditioning and truncation are optional and may be
//    omitted.
//
// \author
//     David N. McKinnon, UQ I.R.I.S., 02.12.2000
//
// FIX BY V. FERRARI (Aug 2004)
//-----------------------------------------------------------------------------

#include <vxl_mvl_fix/FMatrixComputeRobustFix.h>
#include <mvl/FManifoldProject.h>
#include <vcl_vector.h>

class FMatrixComputeMLESACFix : public FMatrixComputeRobustFix {
public:
  //: Initialize FMatrixComputeRobust object.
  // If rank2_truncate = false, the resulting solution is not forced to rank 2
  // using the vnl_svd<double>.
  FMatrixComputeMLESACFix(bool rank2_truncate, double std);
  virtual ~FMatrixComputeMLESACFix();

  virtual double calculate_term(vcl_vector<double>& residuals, vcl_vector<bool>& inlier_list, int& count);
  virtual double calculate_residual(HomgPoint2D& one, HomgPoint2D& two, FMatrix* F);
};

#endif // FMatrixComputeMLESAC_h_
