#ifndef FMatrixComputeRANSACFix_h_
#define FMatrixComputeRANSACFix_h_
//:
// \file
// \brief Robust FMatrix computation
//
//    FMatrixComputeRANSACFix implements
//    Phil Torr's RANSAC (Robust Sampling Concensus) of the fundamental matrix.
//
//    The resulting F matrix is rank-2 truncated.  The
//     conditioning and truncation are optional and may be
//    omitted. The RANSAC class is also responsible for maintaining the inliers
//    passed in the PairMatchSetCorner class.
//
// \author
//     David N. McKinnon, UQ I.R.I.S., 17.05.2001
//
// FIX BY V. FERRARI (Aug 2004)
//-----------------------------------------------------------------------------

#include <vxl_mvl_fix/FMatrixComputeRobustFix.h>

class FMatrixComputeRANSACFix : public FMatrixComputeRobustFix {
public:
  //: Initialize FMatrixComputeRobust object.
  // If rank2_truncate = false, the resulting solution is not forced to rank 2
  // using the vnl_svd<double>.
  FMatrixComputeRANSACFix(bool rank2_truncate, double std);
  virtual ~FMatrixComputeRANSACFix();

  virtual double calculate_term(vcl_vector<double>& residuals,
                                vcl_vector<bool>& inlier_list,
                                int& count);
  virtual double calculate_residual(HomgPoint2D& one,
                                    HomgPoint2D& two,
                                    FMatrix* F);
};

#endif // FMatrixComputeRANSACFix_h_
